/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator;

import java.util.Vector;
import java.util.List;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.control.task.TaskControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.ExternalControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalFileControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.ViewObserverImpl;
import org.xmloperator.option.impl.ParameterImpl;
import org.xmloperator.option.impl.DocumentTypeInfoImpl;
import org.xmloperator.option.impl.TransformerInfoImpl;
import org.xmloperator.option.impl.OperatorInfoImpl;
import org.xmloperator.option.impl.ResourceContextImpl;
import org.xmloperator.option.impl.NamespacePrefixPairImpl;
import org.xmloperator.option.OptionsImpl;
import org.xmloperator.option.OptionsHandler;
import org.xmloperator.compare.impl.DocumentAssociationAccessImpl;
import org.xmloperator.control.task.TaskRunnerThread;
import org.xmloperator.control.global.GlobalFileControlImpl;
import org.xmloperator.control.global.GlobalOptionsControlImpl;
import org.xmloperator.control.global.GlobalViewControlImpl;
import org.xmloperator.Tool; // Reciprocal import

public final class Global implements GlobalControl {
  private static final String TOOL_OPTIONS_DEFAULT_FILE_NAME =
    Constants.TOOL + '.' + Constants.TOOL_OPTIONS_FILE_EXTENSION;
  public int lastInstanceIdentifier = 0;
  public final List instanceList = new Vector();
  public boolean hasToolToExit = false;
  // Global resources
  public String toolOptionsPathName = TOOL_OPTIONS_DEFAULT_FILE_NAME;
  public final OptionsImpl toolOptions = new OptionsImpl();
  public final DocumentAssociationAccess documentAssociationAccess =
    new DocumentAssociationAccessImpl();
  // Global control
  public final TaskRunnerThread taskRunner = new TaskRunnerThread();
  public final ViewObserverImpl globalViewObserver =
    new ViewObserverImpl();
  public final GlobalViewControlImpl globalViewControl =
    new GlobalViewControlImpl(toolOptions, globalViewObserver);
  public final GlobalFileControlImpl globalFileControl =
    new GlobalFileControlImpl(globalViewControl);
  public GlobalOptionsControlImpl globalOptionsControl =
    new GlobalOptionsControlImpl(toolOptions, globalViewControl);

  public Global() {
    globalViewControl.setControleurs(
      (GlobalControl)this,
      this.globalFileControl,
      this.globalOptionsControl);
  }

  public void loadToolOptions() {
    OptionsHandler.load(toolOptions, toolOptionsPathName);
    DocumentTypeInfoImpl.initializeListIfEmpty(toolOptions,
        toolOptions.getOptionList(Options.DOCUMENTTYPE));
    TransformerInfoImpl.initializeListIfEmpty(toolOptions,
        toolOptions.getOptionList(Options.TRANSFORMER));
    OperatorInfoImpl.initializeListIfEmpty(
        toolOptions.getOptionList(Options.OPERATOR));
    ResourceContextImpl.initializeListIfEmpty(toolOptions,
        toolOptions.getOptionList(Options.RESOURCE));
    ParameterImpl.initializeListIfEmpty(
        toolOptions.getOptionList(Options.PARAMETER));
    NamespacePrefixPairImpl.initializeListIfEmpty(
        toolOptions.getOptionList(Options.NAMESPACE));
    {
      List targetOptionList = toolOptions.getOptionList(Options.TARGET);
      if (targetOptionList.isEmpty()) {
        targetOptionList.add(
          toolOptions.createStringOption(Options.TARGET, Constants.TOOL));
        targetOptionList.add(
          toolOptions.createStringOption(Options.TARGET, "xml-stylesheet"));
      }
    }
  }

  public ExternalControl createToolInstance(String documentSystemId) {
    return createToolInstance(documentSystemId, null);
  }

  public ExternalControl createToolInstance(
    String documentSystemId, String schemaSystemId) {
    boolean isFirstInstance = instanceList.size() == 0;
    if (isFirstInstance)
      loadToolOptions();
    Tool tool = new Tool();
    FileControl fileControl = tool.getFileControl();
    if (documentSystemId != null)
      fileControl.loadDocument(documentSystemId, schemaSystemId);
    else if (isFirstInstance
      && toolOptions.getThreeStateOptionValue(Options.AUTOMATICLOADING)
      == ThreeStateOption.TRUE)
      fileControl.loadLastDocument();
    if (isFirstInstance) {
      taskRunner.setGlobalControl(this);
      taskRunner.start();
    }
    return (ExternalControl)tool;
  }

  public Tool getToolInstance(int instanceIdentifier) {
    if (instanceIdentifier < 0)
      return null;
    for (int k = 0; k < instanceList.size(); k++) {
      Tool toolInstance = (Tool)instanceList.get(k);
      if (toolInstance.getInstanceIdentifier() == instanceIdentifier)
        return toolInstance;
    }
    return null;
  }

  ////////////////////////////////
  // MessageDisplay implementation
  ////////////////////////////////

  public void displayMessage(String message) {
    for (int k = 0; k < instanceList.size(); k++)
      ((Tool)instanceList.get(k)).getViewControl().getToolView().
        getToolStatusBar().displayMessage(message);
  }

  ///////////////////////////////
  // GlobalControl implementation
  ///////////////////////////////

  public int getInstanceCount() {
    return instanceList.size();
  }

  public int createInstance(boolean canUseFreeInstance,
    String documentSystemId, String schemaSystemId) {
    try  {
      Tool tool = null;
      if (canUseFreeInstance)
        for (int k = 0; k < instanceList.size(); k++) {
          Tool t = (Tool)instanceList.get(k);
          if (t.getFullyFledgedDocument().getDocument() == null) {
            tool = t;
            tool.getViewControl().getToolView().getFrame().requestFocus();
            break;
          }
        }
      if (tool == null)
        tool = new Tool();
      if (documentSystemId != null && documentSystemId.length() != 0)
        tool.getFileControl().loadDocument(documentSystemId, schemaSystemId);
      return tool.getInstanceIdentifier();
    }
    catch(Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return -1;
    }
  }

  public TaskControl getTaskControl() {
    return (TaskControl)taskRunner;
  }

  public void setEnabled(boolean isEnabled) {
    for (int k = 0; k < instanceList.size(); k++)
      ((Tool)instanceList.get(k)).getViewControl().setEnabled(isEnabled);
  }

  public void displaySelectedNodes() {
    for (int k = 0; k < instanceList.size(); k++) {
      Tool tool = (Tool)instanceList.get(k);
      tool.getDocumentViewControl().displaySelectedNode();
    }
  }

  public String getToolOptionsPathName() {
    return toolOptionsPathName;
  }
}
