/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.compare;

import java.io.PrintWriter;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.util.DomWriterReport;
import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;
import org.xmloperator.framework.document.compare.ElementAssociation;
import org.xmloperator.framework.document.compare.DocumentAssociation;
import org.xmloperator.util.dom.NodesComparator;
//import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.document.util.DocumentWriter;

public class DocumentsMerger extends DocumentWriter {
  private transient DocumentAssociation documentAssociation = null;

  public DocumentsMerger(PrintWriter out) {
    super(out);
  }

  // Return a sum of DomWriterReport.REPLACED_*
  public int printMergedDocument(
    DocumentAssociation documentAssociation,
    DTD dTD, String dTDPublicId, String dTDSystemId,
    String xMLVersion, String encodingName, boolean isStandalone) {
    if (documentAssociation == null)
      throw new IllegalArgumentException();
    this.documentAssociation = documentAssociation;
    printHeading(xMLVersion, encodingName, isStandalone);
    Document document1 = documentAssociation.getDocument1();
    Document document2 = documentAssociation.getDocument2();
    if (dTD != null)
      printDocumentType(document1.getDocumentElement().getNodeName(),
        dTD, dTDPublicId, dTDSystemId);
    int returnCode = printMergedChildLists(document1, document2);
    out.flush();
    return returnCode;
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private int printMergedChildLists(Node parentNode1, Node parentNode2) {
    int returnCode = 0;
    Node node1 = parentNode1.getFirstChild();
    Node node2 = parentNode2.getFirstChild();
    while (node1 != null || node2 != null)
      if (node1 == null) {
        returnCode |= printNode(node2);
        node2 = node2.getNextSibling();
      }
      else if (node2 == null) {
        returnCode |= printNode(node1);
        node1 = node1.getNextSibling();
      }
      else if (node1.getNodeType() == Node.ELEMENT_NODE
        && node2.getNodeType() == Node.ELEMENT_NODE) {
        ElementAssociation elementAssociation1 =
          documentAssociation.getElementAssociation((Element)node1);
        ElementAssociation elementAssociation2 =
          documentAssociation.getElementAssociation((Element)node2);
        if (elementAssociation1 == null) {
          returnCode |= printElementNode((Element)node1);
          node1 = node1.getNextSibling();
        }
        else if (elementAssociation2 == null) {
          returnCode |= printElementNode((Element)node2);
          node2 = node2.getNextSibling();
        }
        else {
          if (elementAssociation1.areElementsEqual(
              documentAssociation.getElementCharacterizationAccess()))
            returnCode |= printElementNode((Element)node1);
          else
            returnCode |= printMergedElement((Element)node1,
              elementAssociation1.getElement2());
          node1 = node1.getNextSibling();
          node2 = node2.getNextSibling();
        }
      }
      else {
        if (node1.getNodeType() == Node.ELEMENT_NODE) {
          returnCode |= printNode(node2);
          node2 = node2.getNextSibling();
        }
        else if (node2.getNodeType() == Node.ELEMENT_NODE) {
          returnCode |= printNode(node1);
          node1 = node1.getNextSibling();
        }
        else {
          if (searchDestinationNodeEqualToSourceNode(node1, node2) == null) {
            returnCode |= printNode(node1);
            node1 = node1.getNextSibling();
          }
          else if (searchDestinationNodeEqualToSourceNode(node2, node1) == null) {
            returnCode |= printNode(node2);
            node2 = node2.getNextSibling();
          }
          else {
            returnCode |= printNode(node1);
            node1 = node1.getNextSibling();
            node2 = node2.getNextSibling();
          }
        }
      }
    return returnCode;
  }

  // Return a sum of DomWriterReport.REPLACED_*
  private int printMergedElement(Element element1, Element element2) {
    int returnCode = 0;
    String elementName = element1.getNodeName();
    out.print('<');
    out.print(elementName);
    NamedNodeMap attrs = element1.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++) {
      out.print(' ');
      returnCode |= printAttributeNode((Attr)attrs.item(k));
    }
    attrs = element2.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++) {
      Attr attr = (Attr)attrs.item(k);
      if (element1.getAttributeNode(attr.getNodeName()) == null) {
        out.print(' ');
        returnCode |= printAttributeNode(attr);
      }
    }
    if (addLineFeedAtStartTagEnd)
      out.println();
    if (isShortcutUsedForEmptyElement
      && !hasChildNodes(element1) && !hasChildNodes(element2))
      out.print("/>");
    else {
      out.print('>');
      returnCode |= printMergedChildLists(element1, element2);
      out.print('<');
      out.print('/');
      out.print(elementName);
      out.print('>');
    }
    return returnCode;
  }

  private Node searchDestinationNodeEqualToSourceNode(
    Node sourceNode, Node destinationNode) {
    do {
      try {
        if (NodesComparator.areNodesEqual(sourceNode, destinationNode))
          return destinationNode;
      }
      catch (IllegalArgumentException exception) {
      }
      destinationNode = destinationNode.getNextSibling();
    }
    while (destinationNode != null
      && destinationNode.getNodeType() != Node.ELEMENT_NODE);
    return null;
  }
}

