/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.compare.impl;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.compare.ElementCharacterization;
import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;

public final class ElementCharacterizationImpl
    implements ElementCharacterization {
  private final int descendantElementCount;
  private final int localNameHashCode;
  private final int attributesHashCode;
  private final int textHashCode;
  private final int descendantsHashCode;
  private final int fullHashCode;

  public ElementCharacterizationImpl(
      final Element element,
      final ElementCharacterizationAccess elementCharacterizationAccess) {
    this.descendantElementCount = getDescendantElementCount(element);
    this.localNameHashCode = element.getLocalName().hashCode();
    this.attributesHashCode = getAttributesHashCode(element);
    this.textHashCode = getTextHashCode(element);
    this.descendantsHashCode =
        getDescendantsHashCode(element, elementCharacterizationAccess);
    this.fullHashCode = this.localNameHashCode + this.attributesHashCode
        + this.textHashCode + 2 * this.descendantsHashCode;
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    if (!(object instanceof ElementCharacterization))
      return false;
    ElementCharacterization other = (ElementCharacterization)object;
    return other.getFullHashCode() == this.getFullHashCode()
        && other.getDescendantElementCount() == this.getDescendantElementCount()
        && other.getLocalNameHashCode() == this.getLocalNameHashCode()
        && other.getAttributesHashCode() == this.getAttributesHashCode()
        && other.getTextHashCode() == this.getTextHashCode()
        && other.getDescendantsHashCode() == this.getDescendantsHashCode();
  }

  /////////////////////////////////////////
  // ElementCharacterization implementation
  /////////////////////////////////////////

  public int getDescendantElementCount() {
    return this.descendantElementCount;
  }

  public int getLocalNameHashCode() {
    return this.localNameHashCode;
  }

  public int getAttributesHashCode() {
    return this.attributesHashCode;
  }

  public int getTextHashCode() {
    return this.textHashCode;
  }

  public int getDescendantsHashCode() {
    return this.descendantsHashCode;
  }

  public int getFullHashCode() {
    return this.fullHashCode;
  }

  /////////////////
  // Static methods
  /////////////////

  private static final int getDescendantElementCount(final Element element) {
    int count = 0;
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        count += 1 + getDescendantElementCount((Element)childNode);
      childNode = childNode.getNextSibling();
    }
    return count;
  }

  private static final int getAttributesHashCode(final Element element) {
    int hashCode = 0;
    NamedNodeMap attrs = element.getAttributes();
    for (int k = 0; k < attrs.getLength(); k++)
      hashCode += getAttributeHashCode((Attr)attrs.item(k));
    return hashCode;
  }

  private static final int getAttributeHashCode(final Attr attr) {
    return attr.getLocalName().hashCode() ^ attr.getValue().hashCode();
  }

  private static final int getTextHashCode(final Element element) {
    Node childNode = element.getFirstChild();
    if (childNode == null)
      return 0;
    if (childNode.getNextSibling() == null)
      return childNode.getNodeType() == Node.TEXT_NODE ?
          childNode.getNodeValue().hashCode() : 0;
    StringBuffer sb = new StringBuffer();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.TEXT_NODE)
        sb.append(childNode.getNodeValue());
      childNode = childNode.getNextSibling();
    }
    return sb.toString().hashCode();
  }

  private static final int getDescendantsHashCode(
      final Element element,
      final ElementCharacterizationAccess elementCharacterisationAccess) {
    int hashCode = 0;
    Node childNode = element.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE)
        hashCode ^= elementCharacterisationAccess.getElementCharacterization(
            (Element)childNode).getFullHashCode();
      childNode = childNode.getNextSibling();
    }
    return hashCode;
  }
}
