/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.instance;

import java.util.List;
import java.util.ResourceBundle;
import java.awt.Frame;
import javax.swing.JOptionPane;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.TextView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.control.task.Task;
import org.xmloperator.framework.control.task.TaskControl;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.document.elementKey.ElementKeyUtil;
import org.xmloperator.control.task.ExpandNodesTask;

public final class DocumentViewControlImpl implements DocumentViewControl {
  private TaskControl taskControl = null;
  private GlobalControl globalControl;
  private FullyFledgedDocument fullyFledgedDocument;
  private ViewAccess viewAccess;
  private Options options;
  private boolean isDisplayingSelectedNode = false;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public DocumentViewControlImpl(
      TaskControl taskControl, GlobalControl globalControl,
    FullyFledgedDocument fullyFledgedDocument,
    ViewAccess viewAccess, Options options) {
    if (taskControl == null || globalControl == null
        || fullyFledgedDocument == null || viewAccess == null
        || options == null)
      throw new IllegalArgumentException();
    this.taskControl = taskControl;
    this.globalControl = globalControl;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.viewAccess = viewAccess;
    this.options = options;
  }

  private Node getSelectedNode() {
    return this.viewAccess.getToolView().getDocumentTreeView().getSelectedNode();
  }

  private void setSelectedNode(Node node) {
    this.viewAccess.getToolView().setSelectedNode(node, true);
  }

  private Node[] getSelectedNodes() {
    return this.viewAccess.getToolView().getDocumentTreeView().
      getSelectedNodes();
  }

  private Element getSelectionParentElement() {
    Node[] selectedNodes = getSelectedNodes();
    if (selectedNodes == null)
      return null;
    Node parentNode = selectedNodes[0].getParentNode();
    return parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE ?
      (Element)parentNode : null;
  }

  private ToolStatusBar getToolStatusBar() {
    ToolView toolView = viewAccess.getToolView();
    return toolView != null ? toolView.getToolStatusBar() : null;
  }

  private void displayMessage(String message) {
    ToolStatusBar toolStatusBar = getToolStatusBar();
    if (toolStatusBar != null)
      toolStatusBar.displayMessage(message);
  }

  private void clearMessage() {
    ToolStatusBar toolStatusBar = getToolStatusBar();
    if (toolStatusBar != null)
      toolStatusBar.displayMessage(" ");
  }

  private void gotoId(String id) {
    Element referencedElement =
      fullyFledgedDocument.getIdManager().getElementById(id);
    if (referencedElement != null)
      gotoNode(referencedElement);
  }

  private void refreshContentModelView() {
    TextView contentModelView = viewAccess.getContentModelView();
    if (contentModelView == null)
      return;
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    String contentModelString = null;
    Node selectedNode = getSelectedNode();
    boolean isElement =
        selectedNode != null && selectedNode.getNodeType() == Node.ELEMENT_NODE;
    if (isElement)
      contentModelString =
      documentUtil.contentModelToString((Element)selectedNode, false);
    contentModelView.setTitle(
        isElement ?
        getString(Label.CONTENT_MODEL_OF)
        + ElementKeyUtil.nodeToString(selectedNode, null) :
        getString(Label.CONTENT_MODEL));
    if (contentModelString == null)
      contentModelString = "";
    contentModelView.setText(contentModelString);
  }

  ////////////////////////////////
  // ToolViewAccess implementation
  ////////////////////////////////

  public ToolView getToolView() {
    return viewAccess.getToolView();
  }

  /////////////////////////////////////
  // DocumentViewControl implementation
  /////////////////////////////////////

  public boolean controlSelectedNodeSchemaConformance() {
    ElementDefinitionMapper elementDefinitionMapper =
      fullyFledgedDocument.getElementDefinitionMapper();
    Element selectionParentElement = getSelectionParentElement();
    if (selectionParentElement != null
      && elementDefinitionMapper.getElementDefinition(selectionParentElement)
      != null
      && !elementDefinitionMapper.isElementContentValid(selectionParentElement))
    {
      ToolStatusBar toolStatusBar = getToolStatusBar();
      if (toolStatusBar != null && toolStatusBar.getMessage().length() <= 1) {
        // Any other message is prioritary
        toolStatusBar.displayMessage(
            getString(Label.ELEMENT_SEQUENCE_NOT_SCHEMA_CONFORMANT));
      }
      return false;
    }
    return true;
  }

  public void displaySelectedNode() {
    try {
      clearMessage();
      isDisplayingSelectedNode = true;
      {
        ToolView toolView = viewAccess.getToolView();
        NodeDetailView nodeDetailView = toolView.getNodeDetailView();
        NodeInterval nodeIntervalForMove =
            fullyFledgedDocument.getNodeIntervalForMove();
        if (nodeIntervalForMove != null && nodeIntervalForMove.isDefined())
          nodeDetailView.setNodeInterval(
              nodeIntervalForMove.getFirstNode(),
              nodeIntervalForMove.getLastNode());
        else
          nodeDetailView.setNode(
              fullyFledgedDocument.getDocument() != null ?
              toolView.getDocumentTreeView().getSelectedNode() : null);
      }
      controlSelectedNodeSchemaConformance();
      refreshContentModelView();
    }
    finally {
      isDisplayingSelectedNode = false;
    }
  }

  public void gotoNode(Node node) {
    clearMessage();
    if (node == null)
      return;
    VariableNodeList nodeStack = this.fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODESTACK);
    Node selectedNode = getSelectedNode();
    if (selectedNode != null)
      nodeStack.add(selectedNode);
    ToolView toolView = viewAccess.getToolView();
    NodeInterval nodeIntervalForMove =
        fullyFledgedDocument.getNodeIntervalForMove();
    if (nodeIntervalForMove != null && nodeIntervalForMove.isDefined())
      toolView.getDocumentTreeView().setSelectedNode(node, true);
    else
      toolView.setSelectedNode(node, true);
    nodeStack.add(node);
    controlSelectedNodeSchemaConformance();
  }

  public void expandNodes(Node[] nodes) {
    this.clearMessage();
    if (nodes == null) {
      nodes = this.getSelectedNodes();
      if (nodes == null)
        return;
    }
    this.taskControl.appendTask(new ExpandNodesTask(
        this.viewAccess.getToolView(), nodes));
  }

  public void activateSelectedNode() {
    clearMessage();
    Node node = getSelectedNode();
    if (node == null)
      return;
    this.fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODESTACK).add(node);
    if (node.getNodeType() != Node.ELEMENT_NODE)
      return;
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    String[] references = documentUtil.referencesFromElement((Element)node);
    if (references != null && references.length == 1)
      gotoId(references[0]);
  }

  public void activateAttributeOfSelectedNode(String attributeName) {
    clearMessage();
    Node selectedNode = getSelectedNode();
    if (selectedNode != null
        && selectedNode.getNodeType() == Node.ELEMENT_NODE)
      gotoId(((Element)selectedNode).getAttribute(attributeName));
  }
}
