/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.instance;

import java.util.ResourceBundle;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.awt.Dimension;
import java.awt.Frame;
import javax.swing.JOptionPane;

import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.option.impl.DocumentTypeInfoImpl;
import org.xmloperator.option.OptionsImpl;
import org.xmloperator.option.OptionsHandler;
import org.xmloperator.option.DialogOptions;

public final class OptionsControlImpl implements OptionsControl {
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private int currentThemaIndex = 0;
  private GlobalControl globalControl = null;
  private GlobalViewControl globalViewControl = null;
  private ViewControl viewControl = null;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  public OptionsControlImpl(FullyFledgedDocument fullyFledgedDocument,
    Options options) {
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
  }

  public void setControleurs(GlobalControl globalControl,
    GlobalViewControl globalViewControl, ViewControl viewControl) {
    this.globalControl = globalControl;
    this.globalViewControl = globalViewControl;
    this.viewControl = viewControl;
  }

  private void setLocale() {
    int localeIndex = options.getIntOptionValue(Options.LOCALEINDEX);
    Locale.setDefault(Constants.locales[localeIndex >= 0 ? localeIndex : 0]);
  }

  private void clearMessage() {
    viewControl.getToolView().getToolStatusBar().displayMessage(" ");
  }

  private void displayResourceMessage(String key) {
    viewControl.getToolView().getToolStatusBar().
      displayMessage(getString(key));
  }

  ////////////////////////////////
  // OptionsControl implementation
  ////////////////////////////////

  public void saveToolViewLayout() {
    ToolView toolView = viewControl.getToolView();
    boolean isHorizontalSplit = toolView.isHorizontalSplit();
    Dimension mainDim = toolView.getContentPanelDimension();
    int detailPartPerCent = toolView.getDetailPartPerCent();
    options.setThreeStateOptionValue(Options.SPLIT,
      isHorizontalSplit ? ThreeStateOption.TRUE : ThreeStateOption.FALSE);
    options.setIntOptionValue(Options.MAINWIDTH, mainDim.width);
    options.setIntOptionValue(Options.MAINHEIGHT, mainDim.height);
    options.setIntOptionValue(Options.DETAILPART, detailPartPerCent);
  }

  public void editToolOptions() {
    clearMessage();
    boolean isLookAndFeelModifiable = globalControl.getInstanceCount() == 1;
    Frame frame = viewControl.getToolView().getFrame();
    DialogOptions dialogOptions = new DialogOptions(frame, options,
        options.getThreeStateOptionValue(Options.ADVANCEDUSE)
        == ThreeStateOption.TRUE,
        currentThemaIndex, isLookAndFeelModifiable);
    ViewUtils.moveComponentToCentre(dialogOptions);
    dialogOptions.show();
    currentThemaIndex = dialogOptions.getSelectedThemaIndex();
    setLocale();
    switch (dialogOptions.getActionAFaire()) {
      case DialogOptions.ACTION_A_FAIRE_RAFFRAICHISSEMENT_ARBRE_DOCUMENT:
        viewControl.rebuildDocumentTreeView();
        break;
      case DialogOptions.ACTION_A_FAIRE_RECONSTRUCTION_FENETRE_OUTIL:
        viewControl.rebuildUserInterface(null);
        break;
      case DialogOptions.ACTION_A_FAIRE_RECONSTRUCTION_COMPLETE: {
        int lookAndFeelIndex =
            options.getIntOptionValue(Options.LOOKANDFEELINDEX);
        if (lookAndFeelIndex < 0
            || lookAndFeelIndex >= Constants.lookAndFeelClassNames.length)
          lookAndFeelIndex = 0;
        viewControl.rebuildUserInterface(
            Constants.lookAndFeelClassNames[lookAndFeelIndex]);
        break;
      }
    }
    if (dialogOptions.isOK())
      saveToolOptions();
  }

  public void saveToolOptions() {
    clearMessage();
    saveToolViewLayout();
    OptionsHandler.save(options, globalControl.getToolOptionsPathName());
    displayResourceMessage(Label.OPTIONS_OUTILS_ENREGISTREES);
  }

  public void reloadToolOptions() {
    clearMessage();
    OptionsHandler.load(options, globalControl.getToolOptionsPathName());
    setLocale();
    globalViewControl.updateGlobalViews();
    viewControl.rebuildUserInterface(null);
    displayResourceMessage(Label.OPTIONS_OUTILS_RECHARGEES);
  }

  public void editToolOptionsDocument() {
    clearMessage();
    globalControl.createInstance(
        true, globalControl.getToolOptionsPathName(),
        options.getStringOptionValue(Options.RNGPATH)
        + Constants.TOOL_FILE + "_options_s.rng");
  }
}
