/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.instance;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.List;
import javax.swing.UIManager;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.MenuItemData;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.util.Observer;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.compare.DocumentAssociationAccess;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.DocumentManager;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.util.PresseFragment;
import org.xmloperator.framework.document.util.VariableNodeList;
import org.xmloperator.framework.document.util.NodeInterval;
import org.xmloperator.framework.document.elementKey.ElementKey;
import org.xmloperator.framework.document.elementKey.ElementKeyProvider;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.observer.ViewObserver;
import org.xmloperator.framework.view.observer.TextView;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.framework.view.tree.DomTree;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.node.TextSelection;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogSelectionIds;
import org.xmloperator.framework.view.dialog.DialogElementKey;
import org.xmloperator.framework.view.dialog.DialogLocator;
import org.xmloperator.framework.control.task.Task;
import org.xmloperator.framework.control.task.TaskControl;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.framework.control.instance.ToolControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.OptionsControl;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.ElementLocator;
import org.xmloperator.util.view.ViewObserverImpl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.TextFrame;
import org.xmloperator.util.view.TextFileEditorImpl;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.document.util.NodesSearcher;
import org.xmloperator.document.util.XPathEvaluator;
import org.xmloperator.view.shortcut.ShortcutsImpl;
import org.xmloperator.view.node.attribute.DialogSelectIdImpl;
import org.xmloperator.view.node.attribute.DialogSelectIdsImpl;
import org.xmloperator.view.node.attribute.DialogReplaceId;
import org.xmloperator.view.frame.DocumentFragmentFrame;
import org.xmloperator.view.frame.NodeListFrame;
import org.xmloperator.view.frame.SearchResultFrame;
import org.xmloperator.view.frame.ParseErrorListFrame;
import org.xmloperator.view.frame.ReferencersFrame;
import org.xmloperator.view.dialog.DialogSearch;
import org.xmloperator.view.dialog.DialogXPath;
import org.xmloperator.view.main.menu.MenuHelp;
import org.xmloperator.view.main.ToolFrame;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.operation.OperationStackFrame;
import org.xmloperator.control.task.RefreshingTask;

public final class ViewControlImpl implements ViewControl {
  private static ImageIcon toolImageIcon =
    new ImageIcon(Constants.TOOL + ".gif");
  private int instanceIdentifier;
  // Resources
  private Options options;
  private FullyFledgedDocument fullyFledgedDocument;
  private OperationStack operationStack;
  private DocumentAssociationAccess documentAssociationAccess;
  private List additionalMenuItemDataList;
  // Controls
  private GlobalControl globalControl = null;
  private GlobalViewControl globalViewControl = null;
  private GlobalOptionsControl globalOptionsControl = null;
  private TaskControl taskControl = null;
  private ToolControl toolControl = null;
  private FileControl fileControl = null;
  private EditControl editControl = null;
  private DocumentViewControl documentViewControl = null;
  private OptionsControl optionsControl = null;
  private RefreshingTask refreshingTask = null;
  public final Task treeViewRebuildTask =
    new Task() {
      public boolean run() {
        rebuildDocumentTreeViewInternal();
        return true;
      }
    };
  // View
  private ToolFrame toolFrame = null;
  private OperationStackFrame operationStackFrame = null;
  private TextFileEditorImpl sourceEditor = null;
  private ParseErrorListFrame parseErrorListFrame = null;
  private ViewObserverImpl referencersViewObserver;
  private DocumentFragmentFrame documentFragmentFrame = null;
  private TextFrame contentModelView = null;
  private NodeListFrame searchResultFrame = null;
  private NodeListFrame nodeStackFrame = null;
  private final Shortcuts shortcuts = new ShortcutsImpl();
  private DialogSelectIdImpl dialogSelectionId = null;
  private DialogSelectIdsImpl dialogSelectionIds = null;
  private DialogReplaceId dialogReplaceId = null;

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  private static final int leftParenthesis(String text) {
    int level = 0;
    for (int k = text.length() - 1; k >= 0; k--) {
      char ch = text.charAt(k);
      if (ch == ')')
        level++;
      else if (ch == '(') {
        level--;
        if (level < 0)
          return k + 1;
      }
    }
    return 0;
  }

  private static final int rigthParenthesis(String text) {
    int level = 0;
    for (int k = 0; k < text.length(); k++) {
      char ch = text.charAt(k);
      if (ch == '(')
        level++;
      else if (ch == ')') {
        level--;
        if (level < 0)
          return k;
      }
    }
    return text.length();
  }

  public ViewControlImpl(int instanceIdentifier, Options options,
    FullyFledgedDocument fullyFledgedDocument, OperationStack operationStack,
    DocumentAssociationAccess documentAssociationAccess,
    List additionalMenuItemDataList) {
    if (options == null || fullyFledgedDocument == null
      || operationStack == null || documentAssociationAccess == null)
      throw new IllegalArgumentException();
    this.instanceIdentifier = instanceIdentifier;
    this.options = options;
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.operationStack = operationStack;
    this.documentAssociationAccess = documentAssociationAccess;
    this.additionalMenuItemDataList = additionalMenuItemDataList;
    this.referencersViewObserver = new ViewObserverImpl();
  }

  public void setControleurs(
    GlobalControl globalControl, GlobalViewControl globalViewControl,
    GlobalOptionsControl globalOptionsControl, ToolControl toolControl,
    FileControl fileControl, EditControl editControl,
    DocumentViewControl documentViewControl, OptionsControl optionsControl) {
    this.globalControl = globalControl;
    this.globalViewControl = globalViewControl;
    this.globalOptionsControl = globalOptionsControl;
    this.taskControl = globalControl.getTaskControl();
    this.toolControl = toolControl;
    this.fileControl = fileControl;
    this.editControl = editControl;
    this.documentViewControl = documentViewControl;
    this.optionsControl = optionsControl;
  }

  public void buildToolFrame() {
    toolFrame =
      new ToolFrame(instanceIdentifier, fullyFledgedDocument,
      documentAssociationAccess, options,
      globalControl, globalViewControl, globalOptionsControl, toolControl,
      (ViewControl)this, editControl, documentViewControl, fileControl,
      optionsControl, shortcuts);
    setIconImage(toolFrame);
    ViewUtils.moveComponentToCentre(toolFrame);
    toolFrame.show();
    refreshingTask = new RefreshingTask((ToolViewAccess)this, options);
    taskControl.addRefreshingTask(refreshingTask);
  }

  private void setIconImage(Frame frame) {
    if (toolImageIcon.getIconHeight() > 0)
      frame.setIconImage(toolImageIcon.getImage());
  }

  public void deleteInstanceView() {
    shortcuts.clearActionListeners();
    detruitCadreOutil();
    if (operationStackFrame != null) {
      operationStack.setObserver(null);
      operationStackFrame.dispose();
      operationStackFrame = null;
    }
    if (sourceEditor != null) {
      sourceEditor.dispose();
      sourceEditor = null;
    }
    if (parseErrorListFrame != null) {
      parseErrorListFrame.dispose();
      parseErrorListFrame = null;
    }
    referencersViewObserver.deleteAllFrames();
    if (documentFragmentFrame != null) {
      documentFragmentFrame.dispose();
      documentFragmentFrame = null;
    }
    if (contentModelView != null) {
      contentModelView.dispose();
      contentModelView = null;
    }
    if (searchResultFrame != null) {
      searchResultFrame.dispose();
      searchResultFrame = null;
    }
    if (nodeStackFrame != null) {
      nodeStackFrame.dispose();
      nodeStackFrame = null;
    }
    this.dialogSelectionId = null;
    this.dialogSelectionIds = null;
    this.dialogReplaceId = null;
  }

  private void detruitCadreOutil() {
    taskControl.removeRefreshingTask(refreshingTask);
    refreshingTask = null;
    try {
      toolFrame.dispose();
    }
    catch (Exception exception) {
    }
    toolFrame = null;
  }

  public MenuHelp getMenuHelp() {
    return toolFrame.getMenuHelp();
  }

  private void setSelectedNode(Node node) {
    if (toolFrame != null)
      toolFrame.setSelectedNode(node, true);
  }

  private void displayResourceMessage(String key) {
    displayMessage(getString(key));
  }

  private void displayMessage(String message) {
    if (toolFrame != null)
      toolFrame.getToolStatusBar().displayMessage(message);
  }

  private boolean isFontLarge() {
    return this.options.getThreeStateOptionValue(Options.FONT)
        == ThreeStateOption.TRUE;
  }

  private Font getCurrentFont() {
    return this.isFontLarge() ? Constants.FONT_LARGE : Constants.FONT_NORMAL;
  }

  ////////////////////////////////
  // ToolViewAccess implementation
  ////////////////////////////////

  public ToolView getToolView() {
    return toolFrame;
  }

  /////////////////////////////////
  // IdsDialogAccess implementation
  /////////////////////////////////

  public DialogSelectionIds getDialogSelectionId(boolean isNoValueEnabled) {
    if (this.dialogSelectionId == null) {
      this.dialogSelectionId =
          new DialogSelectIdImpl(null, this.fullyFledgedDocument);
      this.dialogSelectionId.setContentFont(this.isFontLarge());
      ViewUtils.moveComponentToCentre(this.dialogSelectionId);
    }
    this.dialogSelectionId.setNoValueEnabled(isNoValueEnabled);
    return this.dialogSelectionId;
  }

  public DialogSelectionIds getDialogSelectionIds() {
    if (this.dialogSelectionIds == null) {
      this.dialogSelectionIds =
          new DialogSelectIdsImpl(null, this.fullyFledgedDocument);
      this.dialogSelectionIds.setContentFont(this.isFontLarge());
      ViewUtils.moveComponentToCentre(this.dialogSelectionIds);
    }
    return this.dialogSelectionIds;
  }

  public DialogText getDialogReplaceId(boolean isValueRequired) {
    if (this.dialogReplaceId == null) {
      this.dialogReplaceId =
          new DialogReplaceId(null, this.fullyFledgedDocument.getIdManager());
      this.dialogReplaceId.setContentFont(this.getCurrentFont());
      ViewUtils.moveComponentToCentre(this.dialogReplaceId);
    }
    this.dialogReplaceId.setValueRequired(isValueRequired);
    return this.dialogReplaceId;
  }

  ////////////////////////////
  // ViewAccess implementation
  ////////////////////////////


  public TextFileEditor createSourceEditor() {
    if (sourceEditor == null) {
      sourceEditor = new TextFileEditorImpl();
      setIconImage(sourceEditor);
      ViewUtils.moveComponentToCentre(sourceEditor);
    }
    return sourceEditor;
  }

  public void showParseErrorListView() {
    if (parseErrorListFrame == null) {
      parseErrorListFrame = new ParseErrorListFrame(instanceIdentifier,
        fullyFledgedDocument, (ViewAccess)this, shortcuts);
      setIconImage(parseErrorListFrame);
      ViewUtils.moveComponentToCentre(parseErrorListFrame);
    }
    parseErrorListFrame.show();
    parseErrorListFrame.update();
  }

  public void showOperationStackView() {
    if (operationStackFrame == null) {
      operationStackFrame = new OperationStackFrame(instanceIdentifier,
        options, operationStack, editControl, shortcuts);
      setIconImage(operationStackFrame);
      ViewUtils.moveComponentToCentre(operationStackFrame);
      operationStack.setObserver((Observer)operationStackFrame);
    }
    operationStackFrame.show();
  }

  public void showSelectedNodeReferencersView() {
    Node selectedNode = toolFrame.getDocumentTreeView().getSelectedNode();
    if (selectedNode == null || selectedNode.getNodeType() != Node.ELEMENT_NODE)
      return;
    ReferencersFrame referencersFrame = referencersFrame((Element)selectedNode);
    if (referencersFrame != null) {
      referencersFrame.show();
      return;
    }
    DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
    if (documentUtil == null)
      return;
    AttributesContent attributesContent =
      documentUtil.getAttributesContent((Element)selectedNode);
    if (attributesContent == null)
      return;
    String idAttrName = attributesContent.getIdAttributeName();
    if (idAttrName == null)
      return;
    Attr idAttr = ((Element)selectedNode).getAttributeNode(idAttrName);
    if (idAttr == null)
      return;
    referencersFrame =
      new ReferencersFrame(instanceIdentifier, fullyFledgedDocument,
      (ToolViewAccess)this, options, (Element)selectedNode, shortcuts);
    setIconImage(referencersFrame);
    ViewUtils.moveComponentToCentre(referencersFrame);
    referencersViewObserver.addFrame(referencersFrame);
    referencersFrame.setViewObserver(referencersViewObserver);
    fullyFledgedDocument.getIdManager().setObserverView(
      (ObserverView)referencersFrame, idAttr);
    referencersFrame.show();
  }

  private ReferencersFrame referencersFrame(Element referencedElement) {
    for (int k = 0; k < referencersViewObserver.getCount(); k++) {
      ReferencersFrame referencersFrame =
        (ReferencersFrame)referencersViewObserver.getFrame(k);
      if (referencersFrame.getReferencedElement() == referencedElement)
        return referencersFrame;
    }
    return null;
  }

  public void searchInvalidContent() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null
      || fullyFledgedDocument.getElementDefinitionMapper() == null)
      return;
    Element documentElement = document.getDocumentElement();
    Node rootNode = toolFrame.getDocumentTreeView().getSelectedNode();
    if (rootNode == null || rootNode == document)
      rootNode = documentElement;
    if (rootNode.getNodeType() != Node.ELEMENT_NODE)
      return;
    toolFrame.setSelectedNode(rootNode, false);
    Element invalidContentElement = searchInvalidContent((Element)rootNode);
    if (invalidContentElement != null)
      toolFrame.setSelectedNode(invalidContentElement, true);
    else
      this.displayResourceMessage(Label.THIS_ELEMENT_IS_DEEPLY_VALID);
  }

  private Element searchInvalidContent(Element parentElement) {
    if (!fullyFledgedDocument.getElementDefinitionMapper().
      isElementContentValid(parentElement))
      return parentElement;
    Node childNode = parentElement.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        Element invalidContentElement =
          searchInvalidContent((Element)childNode);
        if (invalidContentElement != null)
          return invalidContentElement;
      }
      childNode = childNode.getNextSibling();
    }
    return null;
  }

  public void showDocumentFragmentView() {
    PresseFragment presseFragment = fullyFledgedDocument.getPresseFragment();
    if (presseFragment == null)
      return;
    if (documentFragmentFrame == null) {
      documentFragmentFrame = new DocumentFragmentFrame(
        fullyFledgedDocument, options, editControl, toolControl, shortcuts);
      setIconImage(documentFragmentFrame);
      ViewUtils.moveComponentToCentre(documentFragmentFrame);
    }
    presseFragment.setObserverView(documentFragmentFrame);
    documentFragmentFrame.show();
  }

  public TextView createContentModelView() {
    if (contentModelView == null) {
      contentModelView = new TextFrame(new Dimension(640, 120)) {
        protected void text_mouseClicked() {
          if (!isSelecting) {
            String text = this.textField.getText();
            int location = this.textField.getCaretPosition();
            int start = leftParenthesis(text.substring(0, location));
            int end = location + rigthParenthesis(text.substring(location));
            if ((start != 0) || (end != text.length()))
              this.selectText(start, end);
            else
              this.selectText(location, location);
          }
        }
      };
      contentModelView.setTitle(getString(Label.CONTENT_MODEL));
      setIconImage(contentModelView);
      ViewUtils.moveComponentToCentre(contentModelView);
    }
    return contentModelView;
  }

  public TextView getContentModelView() {
    return contentModelView;
  }

  public void showSearchResultView() {
    VariableNodeList nodeList = fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODELIST);
    if (nodeList == null)
      return;
    if (searchResultFrame == null) {
      searchResultFrame = new SearchResultFrame(fullyFledgedDocument, options,
        (ViewControl)this, shortcuts);
      searchResultFrame.setTitle(Constants.TOOL + '.' + instanceIdentifier
        + " - " + getString(Label.RESULTAT_DE_RECHERCHE));
      setIconImage(searchResultFrame);
      ViewUtils.moveComponentToCentre(searchResultFrame);
    }
    nodeList.setObserverView(searchResultFrame);
    searchResultFrame.show();
  }

  public void showNodeStackView() {
    VariableNodeList nodeList = fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODESTACK);
    if (nodeList == null)
      return;
    if (nodeStackFrame == null) {
      nodeStackFrame = new NodeListFrame(
          fullyFledgedDocument, FullyFledgedDocument.NODELIST_NODESTACK,
          options, (ViewControl)this, shortcuts);
      nodeStackFrame.setTitle(Constants.TOOL + '.' + instanceIdentifier
        + " - " + getString(Label.BROWSING_HISTORY));
      setIconImage(nodeStackFrame);
      ViewUtils.moveComponentToCentre(nodeStackFrame);
    }
    nodeList.setObserverView(nodeStackFrame);
    nodeStackFrame.show();
  }

  public void closeDocumentViews() {
    if (parseErrorListFrame != null)
      parseErrorListFrame.dispose();
    if (sourceEditor != null)
      sourceEditor.dispose();
    if (contentModelView != null)
      contentModelView.dispose();
    if (operationStackFrame != null)
      operationStackFrame.dispose();
  }

  /////////////////////////////
  // ViewControl implementation
  /////////////////////////////

  public void setEnabled(boolean isEnabled) {
    if (toolFrame != null)
      toolFrame.setMenuEnabled(isEnabled);
    if (operationStackFrame != null)
      operationStackFrame.setMenuEnabled(isEnabled);
  }

  public void rebuildUserInterface(String lookAndFeelClassName) {
    taskControl.appendTask(new ViewRebuildTask(lookAndFeelClassName));
  }

  public void rebuildDocumentTreeView() {
    taskControl.appendTask(treeViewRebuildTask);
  }

  private class ViewRebuildTask implements Task {
    private String lookAndFeelClassName;

    public ViewRebuildTask(String lookAndFeelClassName) {
      this.lookAndFeelClassName = lookAndFeelClassName;
    }

    public boolean run() {
      rebuildUserInterfaceInternal(lookAndFeelClassName);
      return true;
    }
  }

  private void rebuildUserInterfaceInternal(String lookAndFeelClassName) {
    if (lookAndFeelClassName != null && toolControl.getInstanceCount() > 1)
      return;
    DocumentManager documentManager = fullyFledgedDocument.getDocumentManager();
    if (documentManager != null)
      documentManager.removeDocumentModifiedListener(
          toolFrame.getToolStatusBar());
    Node[] selectedNodes =
        toolFrame.getDocumentTreeView().getSelectedNodes();
    Document document = fullyFledgedDocument.getDocument();
    if (document != null)
      fullyFledgedDocument.getNodeLinker().clearNodeUserData(document);
    if (lookAndFeelClassName != null) {
      deleteInstanceView();
      globalViewControl.deleteGlobalView();
      try {
        UIManager.setLookAndFeel(lookAndFeelClassName);
      }
      catch (Exception exception) {
        ExceptionControler.printStackTrace(exception);
      }
    }
    else
      detruitCadreOutil();
    buildToolFrame();
    for (int k = 0; k < additionalMenuItemDataList.size(); k++) {
      MenuItemData menuItemData =
                 (MenuItemData)additionalMenuItemDataList.get(k);
      toolFrame.getMenuHelp().addMenuItem(
          menuItemData.getLabel(), menuItemData.getActionListener(),
          menuItemData.getCommand());
    }
    ToolStatusBar toolStatusBar = toolFrame.getToolStatusBar();
    if (documentManager != null)
      documentManager.addDocumentModifiedListener(toolStatusBar);
    toolStatusBar.displayEditStatus(
        fullyFledgedDocument.getDocument() == null ?
        fullyFledgedDocument.getParseErrorList() != null ?
        ToolStatusBar.EDIT_READONLY_NOTSWITCHABLE :
        ToolStatusBar.EDIT_NONE :
        fullyFledgedDocument.getDocumentManager().isDocumentModified() ?
        ToolStatusBar.EDIT_MODIFIED :
        !fullyFledgedDocument.isReadOnly() ?
        ToolStatusBar.EDIT_READY :
        fullyFledgedDocument.getDocumentSystemId().startsWith("http://") ?
        ToolStatusBar.EDIT_READONLY_NOTSWITCHABLE :
        ToolStatusBar.EDIT_READONLY_SWITCHABLE);
    toolStatusBar.displayLazy(fullyFledgedDocument.isLazy());
    if (document != null) {
      DocumentTreeView documentTreeView = toolFrame.getDocumentTreeView();
      documentTreeView.setRootNode(document);
      documentTreeView.expandNode(document);
      documentTreeView.expandNode(document.getDocumentElement());
      displaySelectedNodes(selectedNodes);
    }
  }

  private void rebuildDocumentTreeViewInternal() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    Node[] selectedNodes =
        toolFrame.getDocumentTreeView().getSelectedNodes();
    DocumentTreeView documentTreeView = toolFrame.getDocumentTreeView();
    documentTreeView.getDomTree().structureChanged();
    documentTreeView.expandNode(document);
    documentTreeView.expandNode(document.getDocumentElement());
    displaySelectedNodes(selectedNodes);
  }

  private void displaySelectedNodes(Node[] selectedNodes) {
    NodeInterval nodeIntervalForMove =
      fullyFledgedDocument.getNodeIntervalForMove();
    if (nodeIntervalForMove != null && nodeIntervalForMove.isDefined()) {
      if (selectedNodes != null)
        toolFrame.getDocumentTreeView().setSelectedNodeInterval(
          selectedNodes[0], selectedNodes[selectedNodes.length - 1], true);
      toolFrame.getNodeDetailView().setNodeInterval(
        nodeIntervalForMove.getFirstNode(),
        nodeIntervalForMove.getLastNode());
    }
    else if (selectedNodes != null)
      toolFrame.setSelectedNodeInterval(
        selectedNodes[0], selectedNodes[selectedNodes.length - 1], true);
  }

  public void changeOrientation() {
    optionsControl.saveToolViewLayout();
    options.setThreeStateOptionValue(
        Options.SPLIT, toolFrame.isHorizontalSplit() ?
        ThreeStateOption.FALSE : ThreeStateOption.TRUE);
    toolFrame.setLayout();
    displaySelectedNodes(toolFrame.getDocumentTreeView().getSelectedNodes());
  }

  public void searchText() {
    String text = "";
    TextSelection textSelection =
        toolFrame.getNodeDetailView().getTextSelection();
    if (textSelection != null)
      text = textSelection.getSelectedText();
    DialogSearch dialogSearch = new DialogSearch(null, options);
    ViewUtils.moveComponentToCentre(dialogSearch);
    if (text.length() != 0)
      dialogSearch.setText(text);
    dialogSearch.show();
    if (!dialogSearch.isOK())
      return;
    VariableNodeList nodeList = fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODELIST);
    if (nodeList == null)
      return;
    String lastSearch = options.getStringOptionValue(Options.LASTSEARCH);
    boolean isCaseIgnored =
        options.getThreeStateOptionValue(Options.CASEIGNOREDFORSEARCH)
        == ThreeStateOption.TRUE;
    boolean entireComparison =
        options.getThreeStateOptionValue(Options.ENTIRECONFORMANCEFORSEARCH)
        == ThreeStateOption.TRUE;
    nodeList.clear();
    if (lastSearch.length() == 0)
      return;
    new NodesSearcher(nodeList, lastSearch, isCaseIgnored, entireComparison).
        searchWithin(fullyFledgedDocument.getDocument());
    if (nodeList.getLength() > 0) {
      showSearchResultView();
      setSelectedNode(nodeList.item(0));
      if (toolFrame != null)
        toolFrame.getNodeDetailView().selectLastSearchNextInstance();
    }
    else {
      if (searchResultFrame != null)
        searchResultFrame.dispose();
      displayMessage("\"" + lastSearch + "\" " + getString(Label.NOT_FOUND));
    }
  }

  public void evaluateXPathExpression() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    Node contextNode = document.getDocumentElement();
    if (contextNode == null)
      return;
    {
      DialogXPath dialogXPath =
          new DialogXPath(null, options, contextNode.getNodeName());
      ViewUtils.moveComponentToCentre(dialogXPath);
      dialogXPath.show();
      if (!dialogXPath.isOK())
        return;
    }
    VariableNodeList nodeList = fullyFledgedDocument.getNodeList(
        FullyFledgedDocument.NODELIST_NODELIST);
    if (nodeList == null)
      return;
    nodeList.clear();
    String lastExpression =
        options.getStringOptionValue(Options.LASTXPATHEXPRESSION);
    if (lastExpression.length() == 0)
      return;
    try {
      String stringResult =
          XPathEvaluator.evaluate(contextNode, lastExpression, nodeList);
      if (stringResult != null) {
        if (searchResultFrame != null)
          searchResultFrame.dispose();
        JOptionPane.showMessageDialog(
            toolFrame.getFrame(),
            getString(Label.RESULT) + "=" + stringResult,
            "XPath evaluation", JOptionPane.WARNING_MESSAGE);
      }
      else if (nodeList.getLength() > 0) {
        showSearchResultView();
        setSelectedNode(nodeList.item(0));
      }
      else {
        if (searchResultFrame != null)
          searchResultFrame.dispose();
        displayMessage(getString(Label.NOT_FOUND));
      }
    }
    catch (TransformerException exception) {
      ExceptionDialogs.displayException("Transformer exception", exception);
    }
  }

  public void showElementAt(int lineNumber, int columnNumber) {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null || toolFrame == null)
      return;
    if (lineNumber <= 0 || columnNumber <= 0) {
      Frame frame = toolFrame.getFrame();
      DialogLocator dialog = this.globalViewControl.getDialogLocator();
      dialog.setTitle(getString(Label.LOCATION));
      dialog.setLineNumber(
          options.getIntOptionValue(Options.LOCATORLINENUMBER));
      dialog.setColumnNumber(
          options.getIntOptionValue(Options.LOCATORCOLUMNNUMBER));
      dialog.show();
      if (!dialog.isOK())
        return;
      lineNumber = dialog.getLineNumber();
      columnNumber = dialog.getColumnNumber();
      options.setIntOptionValue(Options.LOCATORLINENUMBER, lineNumber);
      options.setIntOptionValue(Options.LOCATORCOLUMNNUMBER, columnNumber);
    }
    try {
      Element element = ElementLocator.locateElement(
          document.getDocumentElement(),
          fullyFledgedDocument.getDocumentSystemId(), lineNumber, columnNumber);
      if (element != null)
        this.setSelectedNode(element);
      else
        displayMessage(getString(Label.NOT_FOUND));
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException("IO exception", exception);
    }
    catch (SAXException exception) {
      ExceptionDialogs.displayException("SAX exception", exception);
    }
  }

  public void changeElementKey(String elementName) {
    if (elementName == null) {
      Node node = this.toolFrame.getDocumentTreeView().getSelectedNode();
      if (node == null || node.getNodeType() != Node.ELEMENT_NODE) {
        displayMessage(getString(Label.AN_ELEMENT_MUST_BE_SELECTED));
        return;
      }
      elementName = node.getNodeName();
    }
    Document document = this.fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    ElementKeyProvider elementKeyProvider =
        this.fullyFledgedDocument.getElementKeyProvider();
    DialogElementKey dialogElementKey =
        this.globalViewControl.getDialogElementKey();
    dialogElementKey.setNames(document, elementName);
    dialogElementKey.setElementKey(
        elementKeyProvider.getElementKey(elementName));
    dialogElementKey.show();
    if (!dialogElementKey.isOK())
      return;
    ElementKey elementKey = dialogElementKey.getElementKey();
    if (elementKey == null)
      return;
    elementKeyProvider.setElementKey(elementName, elementKey);
    elementNameChanged(this.toolFrame.getDocumentTreeView().getDomTree(),
                       document, elementName);
  }

  protected static final void elementNameChanged(
      DomTree domTree, Node parentNode, String elementName) {
    Node childNode = parentNode.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        if (childNode.getNodeName().equals(elementName))
          domTree.nodeChanged(childNode);
        elementNameChanged(domTree, childNode, elementName);
      }
      childNode = childNode.getNextSibling();
    }
  }
}
