/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.control.task;

import java.awt.Frame;
import java.io.IOException;
import java.io.File;
import java.util.StringTokenizer;
import java.util.ResourceBundle;
import java.util.Collections;
import java.util.List;
import javax.swing.JOptionPane;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.apache.xerces.xni.parser.XMLParseException;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.relaxng.Schema;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.parse.ParseError;
import org.xmloperator.framework.document.parse.ParseErrorList;
import org.xmloperator.framework.document.parse.ToolEntityResolver;
import org.xmloperator.framework.document.manage.DocumentModifiedListener;
import org.xmloperator.framework.document.manage.DocumentManager;
import org.xmloperator.framework.document.doctype.DocumentTypeInfo;
import org.xmloperator.framework.document.util.ResourceContext;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.status.MessageDisplay;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.framework.view.instance.ViewAccess;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.control.task.CancelableTask;
import org.xmloperator.framework.control.global.GlobalOptionsControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.util.view.ExceptionDialogs;
import org.xmloperator.option.OptionsHandler;
import org.xmloperator.document.util.DocumentTypeInfoUtils;
import org.xmloperator.view.dialog.DialogDocumentTypeSelection;
import org.xmloperator.operation.OperationPrinter;
import org.xmloperator.operation.OperationStack;
import org.xmloperator.operation.OperationSaver;
import org.xmloperator.operation.OperationLoader;
import org.xmloperator.control.io.ToolEntityResolverImpl;
import org.xmloperator.control.io.IOUtils;

public final class DocumentLoadTask implements CancelableTask {
  private GlobalOptionsControl globalOptionsControl;
  private FullyFledgedDocument fullyFledgedDocument;
  private Options options;
  private OperationStack operationStack;
  private OperationSaver operationSaver;
  private GlobalViewAccess globalViewAccess;
  private ViewAccess viewAccess;
  private String documentSystemId;
  private boolean isSchemaKept;
  private ToolEntityResolverImpl toolEntityResolver = null;

  public static final void closeDocument(
    FullyFledgedDocument fullyFledgedDocument, Options options,
    OperationStack operationStack, OperationPrinter operationPrinter,
    ViewAccess viewAccess) {
    ToolView toolView = viewAccess.getToolView();
    Document document = fullyFledgedDocument.getDocument();
    if (document != null) {
      Node selectedNode = toolView != null ?
        toolView.getDocumentTreeView().getSelectedNode() : null;
      if (selectedNode == null)
        selectedNode = document;
      ResourceContext resourceContext = IOUtils.addResourceFirst(
        options, fullyFledgedDocument.getDocumentSystemId());
      resourceContext.setDocumentTypeAdditionalKey(
        fullyFledgedDocument.getDocumentTypeAdditionalKey());
      IOUtils.saveSelectedNodePath(selectedNode, resourceContext);
    }
    operationPrinter.close();
    operationStack.clear();
    fullyFledgedDocument.reset();
    viewAccess.closeDocumentViews();
    if (toolView != null)
      toolView.clear();
  }

  public DocumentLoadTask(
    FullyFledgedDocument fullyFledgedDocument, Options options,
    OperationStack operationStack, OperationSaver operationSaver,
    GlobalOptionsControl globalOptionsControl,
    GlobalViewAccess globalViewAccess, ViewAccess viewAccess,
    String documentSystemId, boolean isSchemaKept) {
    if (fullyFledgedDocument == null || options == null
      || operationStack == null || operationSaver == null
      || globalOptionsControl == null
      || globalViewAccess == null || viewAccess == null
      || documentSystemId == null)
      throw new IllegalArgumentException();
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.operationStack = operationStack;
    this.operationSaver = operationSaver;
    this.globalOptionsControl = globalOptionsControl;
    this.globalViewAccess = globalViewAccess;
    this.viewAccess = viewAccess;
    this.documentSystemId = documentSystemId;
    this.isSchemaKept = isSchemaKept;
  }

  public boolean loadDocument() {
    ToolView toolView = viewAccess.getToolView();
    if (toolView == null)
      return false;
    ToolStatusBar toolStatusBar = toolView.getToolStatusBar();
    clearMessage();
    {
      String normalisedSystemId = FileUtils.reducedSystemId(documentSystemId);
      if (normalisedSystemId == null) {
        displayMessage(getString(Label.RESSOURCE_INVALIDE)
          + " : " + documentSystemId);
        return false;
      }
      documentSystemId = normalisedSystemId;
    }
    DTD dTD = isSchemaKept ? fullyFledgedDocument.getDTD() : null;
    Schema schema = isSchemaKept ? fullyFledgedDocument.getSchema() : null;
    if (fullyFledgedDocument.getOriginalDTD() == dTD)
      dTD = null;
    closeDocument(fullyFledgedDocument, options,
      operationStack, operationSaver, viewAccess);
    displayMessage(getString(Label.CHARGE) + " " + documentSystemId);
    toolEntityResolver =
      new ToolEntityResolverImpl(options, (MessageDisplay)toolStatusBar);
    Element exceptionLocationElement = null;
    try {
      fullyFledgedDocument.parse(documentSystemId,
        (ToolEntityResolver)toolEntityResolver);
      displayResourceMessage(Label.NOTE_IDS);
      if (dTD != null)
        fullyFledgedDocument.setDTD(dTD);
      if (schema != null)
        fullyFledgedDocument.setSchema(schema);
    }
    catch (SAXParseException exception) {
      // trace de l'exception (FATAL ERROR) dans parseErrorList
    }
    catch (XMLParseException exception) {
      // trace de l'exception (FATAL ERROR) dans parseErrorList
    }
    catch (SAXException exception) {
      ExceptionDialogs.displaySAXException(exception);
    }
    catch (IOException exception) {
      ExceptionDialogs.displayException(exception);
    }
    catch (IdException exception) {
      ExceptionDialogs.displayException("ID exception", exception);
      exceptionLocationElement = exception.getAttribute().getOwnerElement();
      fullyFledgedDocument.getIdManager().clearAndSetEnabled(false);
    }
    /*
    catch (UndefinedNamespaceException exception) {
      ExceptionDialogs.displayException("Namespace exception", exception);
      exceptionLocationElement = exception.getElement();
    }
    */
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    toolEntityResolver = null;
    clearMessage();
    Document document = fullyFledgedDocument.getDocument();
    ParseErrorList parseErrorList = fullyFledgedDocument.getParseErrorList();
    if (document == null) {
      if (parseErrorList != null && parseErrorList.size() != 0)
        displayParseErrorList();
      toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READONLY_NOTSWITCHABLE);
      return false;
    }
    DocumentManager documentManager = fullyFledgedDocument.getDocumentManager();
    documentManager.addDocumentModifiedListener(
      (DocumentModifiedListener)toolStatusBar);
    ResourceContext resourceContext =
      IOUtils.addResourceFirst(options, documentSystemId);
    setDocumentTypeAdditionalKey(resourceContext);
    displayResourceMessage(Label.PREPARE_VUE);
    prepareVue(resourceContext);
    clearMessage();
    if (exceptionLocationElement != null)
      toolView.setSelectedNode(exceptionLocationElement, true);
    if (parseErrorList.size() != 0)
      displayParseErrorList();
    if (fullyFledgedDocument.isReadOnly()) {
      toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READONLY_SWITCHABLE);
      return true;
    }
    String documentPath =
      FileUtils.fromSystemIdToFilePathName(documentSystemId);
    if (documentPath == null) {
      fullyFledgedDocument.setReadOnly(true);
      toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READONLY_NOTSWITCHABLE);
      return true;
    }
    int dotPos = documentPath.lastIndexOf('.');
    String pathNameFirstPart = documentPath.substring(0, dotPos + 1);
    if (documentPath.substring(dotPos + 1).equalsIgnoreCase("ope")) {
      toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READY);
      return true;
    }
    String operationsPathName = pathNameFirstPart + "ope";
    String tempPathName = pathNameFirstPart + "tmp";
    File operationsFile = new File(operationsPathName);
    File tempFile = new File(tempPathName);
    if (tempFile.exists()) {
      fullyFledgedDocument.setReadOnly(true);
      toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READONLY_NOTSWITCHABLE);
      displayResourceMessage(Label.
        OUVERTURE_EN_LECTURE_SEULE_POUR_CAUSE_D_EXISTENCE_DE_FICHIER_TMP);
      return true;
    }
    toolStatusBar.displayEditStatus(ToolStatusBar.EDIT_READY);
    boolean isThereAnOperationsFile = operationsFile.exists();
    if (isThereAnOperationsFile)
      operationsFile.renameTo(tempFile);
    if (options.getThreeStateOptionValue(Options.OPERATIONSSTORED)
      == ThreeStateOption.TRUE)
      try {
        operationSaver.open(operationsPathName, tempFile);
      }
      catch (IOException exception) {
        ExceptionDialogs.displayException(exception);
      }
    if (isThereAnOperationsFile)
      try {
        boolean isLazy = fullyFledgedDocument.isLazy();
        OperationLoader.loadAndExecuteOperations(
          fullyFledgedDocument.getDocument(),
          documentManager.getNodeManager(),
          documentManager.getAttributeManager(),
          tempFile.getPath(), operationStack);
        if (!isLazy) {
          fullyFledgedDocument.setLazy(false);
          toolStatusBar.displayLazy(false);
        }
      }
      catch (OperationException exception) {
        ExceptionDialogs.displayException(exception);
      }
    return true;
  }

  private void displayParseErrorList() {
    if (options.getThreeStateOptionValue(Options.OPENERRORS)
        == ThreeStateOption.TRUE)
      viewAccess.showParseErrorListView();
    int mostSeriousErrorType =
      fullyFledgedDocument.getParseErrorList().getMostSeriousErrorType();
    if (mostSeriousErrorType != ParseError.ERRORTYPE_NO)
      displayMessage("Parsing "
        + ParseError.ERROR_TYPE_LABEL[mostSeriousErrorType] + "(S)");
  }

  private void setDocumentTypeAdditionalKey(ResourceContext resourceContext) {
    List documentTypeInfoList = options.getOptionList(Options.DOCUMENTTYPE);
    String documentTypeName = fullyFledgedDocument.getDocumentTypeName();
    String givenAdditionalKey = resourceContext.getDocumentTypeAdditionalKey();
    if (givenAdditionalKey != null
      && DocumentTypeInfoUtils.getDocumentTypeInfoFromList(documentTypeInfoList,
        documentTypeName, givenAdditionalKey) != null) {
      fullyFledgedDocument.setDocumentTypeAdditionalKey(givenAdditionalKey);
      return;
    }
    List documentTypeInfoSubList =
      DocumentTypeInfoUtils.getDocumentTypeInfoSubList(documentTypeInfoList,
      documentTypeName);
    int candidateAdditionalKeyCount = documentTypeInfoSubList.size();
    DocumentTypeInfo documentTypeInfo = null;
    if (candidateAdditionalKeyCount == 0)
      documentTypeInfo = globalOptionsControl.newDocumentType(documentTypeName,
        fullyFledgedDocument.getDocument().getDoctype());
    else if (candidateAdditionalKeyCount == 1)
      documentTypeInfo = (DocumentTypeInfo)documentTypeInfoSubList.get(0);
    else {
      Collections.sort(documentTypeInfoSubList);
      DialogSelection dialog =
          this.globalViewAccess.getDialogSelectionDocumentType(false, false);
      dialog.setItemList(documentTypeInfoSubList);
      dialog.setTitle(Label.SELECT_DOCUMENT_TYPE);
      dialog.show();
      documentTypeInfo = (DocumentTypeInfo)dialog.getSelectedItem();
    }
    String additionalKey = documentTypeInfo != null ?
      documentTypeInfo.getDocumentTypeAdditionalKey() : null;
    fullyFledgedDocument.setDocumentTypeAdditionalKey(additionalKey);
    resourceContext.setDocumentTypeAdditionalKey(additionalKey);
  }

  private void prepareVue(ResourceContext resourceContext) {
    ToolView toolView = viewAccess.getToolView();
    if (toolView == null)
      return;
    Document document = fullyFledgedDocument.getDocument();
    {
      DocumentTreeView documentTreeView = toolView.getDocumentTreeView();
      documentTreeView.setRootNode(document);
      documentTreeView.expandNode(document);
      documentTreeView.expandNode(document.getDocumentElement());
    }
    {
      Node selectedNode = NodePathBuilder.getNodeFromPath(document,
        resourceContext.getSelectedElementPath(),
        resourceContext.getSelectedNodeIndex());
      if (selectedNode == null)
        selectedNode = document;
      toolView.setSelectedNode(selectedNode, true);
    }
  }

  private void displayMessage(String message) {
    ToolView toolView = viewAccess.getToolView();
    if (toolView != null)
      toolView.getToolStatusBar().displayMessage(message);
  }

  private void clearMessage() {
    displayMessage(" ");
  }

  private void displayResourceMessage(String key) {
    displayMessage(getString(key));
  }

  private static final String getString(String key) {
    return ResourceBundle.getBundle(Label.CLASS).getString(key);
  }

  ////////////////////////////////
  // CancelableTask implementation
  ////////////////////////////////

  public boolean run() {
    return loadDocument();
  }

  public void cancel() {
    ToolEntityResolverImpl toolEntityResolver = this.toolEntityResolver;
    if (toolEntityResolver != null)
      try {
        toolEntityResolver.closeAll();
      }
      catch (IOException exception) {
      }
  }
}
