/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.impl;

import org.xmloperator.framework.document.dtd.ExternalDefinition;
import org.xmloperator.framework.document.dtd.EntityDefinition;
import org.xmloperator.document.dtd.impl.ExternalDefinitionImpl;

public class EntityDefinitionImpl implements EntityDefinition {
  public static final int INTERNAL_ENTITY = 0;
  public static final int EXTERNAL_ENTITY = 1;
  public static final int UNPARSED_ENTITY = 2;
  public static final EntityDefinition LT =
    new EntityDefinitionImpl("&#60;");
  public static final EntityDefinition GT =
    new EntityDefinitionImpl("&#62;");
  public static final EntityDefinition AMP =
    new EntityDefinitionImpl("&#38;");
  public static final EntityDefinition APOS =
    new EntityDefinitionImpl("&#39;");
  public static final EntityDefinition QUOT =
    new EntityDefinitionImpl("&#34;");
  protected int entityType;
  protected String entityValue;
  protected ExternalDefinition externalDefinition;
  protected String notationName;

  // Internal entity
  public EntityDefinitionImpl(String entityValue) {
    this.entityType = INTERNAL_ENTITY;
    this.entityValue = entityValue;
    this.externalDefinition = null;
    this.notationName = null;
  }

  // External entity
  public EntityDefinitionImpl(String publicId, String systemId) {
    this.entityType = EXTERNAL_ENTITY;
    this.entityValue = null;
    this.externalDefinition = new ExternalDefinitionImpl(publicId, systemId);
    this.notationName = null;
  }

  // Unparsed entity
  public EntityDefinitionImpl(String publicId, String systemId,
    String notationName) {
    this.entityType = UNPARSED_ENTITY;
    this.entityValue = null;
    this.externalDefinition = new ExternalDefinitionImpl(publicId, systemId);
    this.notationName = notationName;
  }

  //////////////////////////////////
  // EntityDefinition implementation
  //////////////////////////////////

  public int getEntityType() {
    return entityType;
  }

  public String getEntityValue() {
    return entityValue;
  }

  public String getPublicId() {
    return externalDefinition != null ? externalDefinition.getPublicId(): null;
  }

  public String getSystemId() {
    return externalDefinition != null ? externalDefinition.getSystemId(): null;
  }

  public String getNotationName() {
    return notationName;
  }
}
