/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.dtd.parse;

import java.util.List;
import java.util.ArrayList;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.document.dtd.impl.SpecificationNodeImpl;
import org.xmloperator.document.dtd.impl.ElementSpecificationImpl;
import org.xmloperator.document.dtd.parse.SpecificationNodeBuilder;

public final class ElementSpecificationBuilder {
  private String specType = ElementSpecification.EMPTY;
  private List specNodeBuilderStack = new ArrayList();
  private int specNodeStackSize = 0;
  private SpecificationNode rootSpecNode = null;

  public ElementSpecificationBuilder() {
  }

  public void empty() {
    specType = ElementSpecification.EMPTY;
    specNodeStackSize = 0;
    rootSpecNode = null;
  }

  public void any() {
    specType = ElementSpecification.ANY;
    specNodeStackSize = 0;
    rootSpecNode = null;
  }

  private SpecificationNodeBuilder peekSpecNodeBuilder() {
    if (specNodeStackSize == 0)
      throw new RuntimeException("Empty spec node stack");
    return (SpecificationNodeBuilder)specNodeBuilderStack.
      get(specNodeStackSize - 1);
  }

  public void startGroup() {
    if (specNodeStackSize == 0) {
      specType = ElementSpecification.CHILDREN;
      rootSpecNode = null;
    }
    specNodeStackSize++;
    if (specNodeBuilderStack.size() < specNodeStackSize)
      specNodeBuilderStack.add(new SpecificationNodeBuilder());
    else
      peekSpecNodeBuilder().clear();
  }

  public void pcdata() {
    specType = ElementSpecification.MIXED;
    peekSpecNodeBuilder().pcdata();
  }

  public void element(String elementName) {
    peekSpecNodeBuilder().element(elementName);
  }

  // separator = ',' | '|'
  public void separator(char separator) {
    peekSpecNodeBuilder().separator(separator);
  }

  // occurence = '?' | '*' | '+'
  public void occurrence(char occurrence) {
    if (specNodeStackSize != 0)
      peekSpecNodeBuilder().occurrence(occurrence);
    else if (rootSpecNode != null)
      rootSpecNode.setOccurrence(occurrence);
    else throw new RuntimeException("Occurrence symbol unexpected here");
  }

  public void endGroup() {
    SpecificationNode resultSpecNode = peekSpecNodeBuilder().result();
    if (resultSpecNode == null)
      throw new RuntimeException("Empty spec group");
    specNodeStackSize--;
    if (specNodeStackSize != 0)
      peekSpecNodeBuilder().specificationNode(resultSpecNode);
    else
      rootSpecNode = resultSpecNode;
  }

  public ElementSpecification result() {
    return (specType == ElementSpecification.EMPTY
      || specType == ElementSpecification.ANY) ?
      new ElementSpecificationImpl(specType) :
      rootSpecNode != null ?
      new ElementSpecificationImpl(specType, rootSpecNode) : null;
  }
}
