/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.Arrays;

import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.ElementContent;
import org.xmloperator.framework.document.schema.AttributesContent;

public final class ContentCache {
  private static final int STRUCT_SIZE = 3;
  private static final int POS_ELEMENT = 0;
  private static final int POS_CHILDREN_CONTENT = 1;
  private static final int POS_ATTRIBUTES_CONTENT = 2;
  private int count;
  private Object[] element_elementContent_attributesContent_memories;

  public ContentCache() {
    this(16);
  }

  public ContentCache(int count) {
    this.count = count;
    this.element_elementContent_attributesContent_memories =
      new Object[count * STRUCT_SIZE];
    Arrays.fill(this.element_elementContent_attributesContent_memories, null);
  }

  private int indexOf(Element element) {
    if (element == null)
      throw new IllegalArgumentException();
    for (int k = 0; k < count; k++) {
      Element cacheElement =
        (Element)element_elementContent_attributesContent_memories
        [k * STRUCT_SIZE + POS_ELEMENT];
      if (cacheElement == element)
        return k;
      if (cacheElement == null)
        return -1;
    }
    return -1;
  }

  private void shiftRight(Element element, ElementContent elementContent,
    AttributesContent attributesContent, int count) {
    System.arraycopy(element_elementContent_attributesContent_memories, 0,
      element_elementContent_attributesContent_memories, STRUCT_SIZE,
      count * STRUCT_SIZE);
    element_elementContent_attributesContent_memories[POS_ELEMENT] = element;
    element_elementContent_attributesContent_memories[POS_CHILDREN_CONTENT] =
      elementContent;
    element_elementContent_attributesContent_memories[POS_ATTRIBUTES_CONTENT] =
      attributesContent;
  }

  private void shiftLeft(int index) {
    System.arraycopy(element_elementContent_attributesContent_memories,
      (index + 1) * STRUCT_SIZE,
      element_elementContent_attributesContent_memories, index * STRUCT_SIZE,
      (count - 1 - index) * STRUCT_SIZE);
    Arrays.fill(element_elementContent_attributesContent_memories,
      (count - 1) * STRUCT_SIZE, count * STRUCT_SIZE, null);
  }

  public void put(Element element, ElementContent elementContent) {
    if (element == null)
      throw new IllegalArgumentException();
    int index = indexOf(element);
    AttributesContent attributesContent = index >= 0 ?
      (AttributesContent)element_elementContent_attributesContent_memories
      [index * STRUCT_SIZE + POS_ATTRIBUTES_CONTENT] : null;
    shiftRight(element, elementContent, attributesContent,
      index >= 0 ? index : count - 1);
  }

  public void put(Element element, AttributesContent attributesContent) {
    if (element == null)
      throw new IllegalArgumentException();
    int index = indexOf(element);
    ElementContent elementContent = index >= 0 ?
      (ElementContent)element_elementContent_attributesContent_memories
      [index * STRUCT_SIZE + POS_CHILDREN_CONTENT] : null;
    shiftRight(element, elementContent, attributesContent,
      index >= 0 ? index : count - 1);
  }

  public boolean contains(Element element) {
    return indexOf(element) >= 0;
  }

  public ElementContent getElementContent(Element element) {
    return moveOnTop(element) ? (ElementContent)
      element_elementContent_attributesContent_memories[POS_CHILDREN_CONTENT] :
      null;
  }

  public AttributesContent getAttributesContent(Element element) {
    return moveOnTop(element) ? (AttributesContent)
      element_elementContent_attributesContent_memories[POS_ATTRIBUTES_CONTENT]
      : null;
  }

  private boolean moveOnTop(Element element) {
    int index = indexOf(element);
    if (index < 0)
      return false;
    if (index > 0) {
      ElementContent elementContent =
        (ElementContent)element_elementContent_attributesContent_memories
        [index * STRUCT_SIZE + POS_CHILDREN_CONTENT];
      AttributesContent attributesContent =
        (AttributesContent)element_elementContent_attributesContent_memories
        [index * STRUCT_SIZE + POS_ATTRIBUTES_CONTENT];
      shiftRight(element, elementContent, attributesContent, index);
    }
    return true;
  }

  public void remove(Element element) {
    int index = indexOf(element);
    if (index >= 0)
      shiftLeft(index);
  }
}
