/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.dtd.SpecificationNode;
import org.xmloperator.framework.document.dtd.ElementSpecification;
import org.xmloperator.framework.document.dtd.AttributeDefinition;
import org.xmloperator.framework.document.dtd.AttributeDefinitionList;
import org.xmloperator.framework.document.dtd.DTD;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.framework.document.schema.ElementContent;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.schema.
    NotConformantToContentModelException;
import org.xmloperator.framework.document.schema.UnsatisfiedPatternException;
import org.xmloperator.framework.document.schema.
    InvalidElementSpecificationException;
import org.xmloperator.framework.document.schema.InvalidAttributesException;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.document.manage.AttributesContentAccess;
import org.xmloperator.util.dom.ElementNamesUtils;
import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.document.dtd.ElementContentImpl;
import org.xmloperator.document.dtd.AttributesContentImpl;
import org.xmloperator.document.dtd.AnyElementContent;
import org.xmloperator.document.manage.ContentCache;
import org.xmloperator.document.manage.AbstractElementManager;
import org.xmloperator.document.manage.EmptyElementContent;

public final class DTDElementManager extends AbstractElementManager {
  private DTD dTD;

  public DTDElementManager(IdManager idManager, DTD dTD) {
    super(idManager);
    if (dTD == null)
      throw new IllegalArgumentException();
    this.dTD = dTD;
  }

  protected void addElementContentToCache(Element parentElement)
    throws NotConformantToContentModelException {
    if (contentCache.getElementContent(parentElement) != null)
      return;
    ElementSpecification parentElementSpec =
      dTD.getElementSpecification(parentElement.getNodeName());
    if (parentElementSpec == null)
      return;
    controlInvalidAttributes(parentElement);
    controlMissingAttributes(parentElement);
    String[] contentElementNames =
        ElementNamesUtils.elementNames(parentElement);
    ElementContent elementContent;
    String specType = parentElementSpec.getSpecType();
    if (specType.equals(ElementSpecification.EMPTY)) {
      if (ElementBrowser.hasChildElements(parentElement))
        throw new InvalidElementSpecificationException(0);
      elementContent = EmptyElementContent.EMPTY_CHILDREN_CONTENT;
    }
    else if (specType.equals(ElementSpecification.ANY))
      elementContent =
        new AnyElementContent(dTD.getElementNameList(), contentElementNames);
    else
      elementContent = new ElementContentImpl(parentElementSpec.getSpecNode(),
          contentElementNames);
    contentCache.put(parentElement, elementContent);
  }

  private void controlInvalidAttributes(Element element)
      throws InvalidAttributesException {
    NamedNodeMap attributes = element.getAttributes();
    AttributeDefinitionList attributeDefinitionList =
        dTD.getAttributeDefinitionList(element.getNodeName());
    boolean hasInvalidAttribute = false;
    for (int k = 0; k < attributes.getLength(); k++) {
      String attributeName = attributes.item(k).getNodeName();
      if (!attributeName.equals("xmlns") && !attributeName.startsWith("xmlns:")
          && (attributeDefinitionList == null
          || attributeDefinitionList.get(attributeName) == null)) {
        hasInvalidAttribute = true;
        break;
      }
    }
    if (!hasInvalidAttribute)
      return;
    List invalidAttributeNameList = new ArrayList();
    for (int k = 0; k < attributes.getLength(); k++) {
      String attributeName = attributes.item(k).getNodeName();
      if (!attributeName.equals("xmlns") && !attributeName.startsWith("xmlns:")
          && (attributeDefinitionList == null
          || attributeDefinitionList.get(attributeName) == null))
        invalidAttributeNameList.add(attributeName);
    }
    String[] invalidAttributeNames =
        new String[invalidAttributeNameList.size()];
    invalidAttributeNameList.toArray(invalidAttributeNames);
    throw new InvalidAttributesException(invalidAttributeNames);
  }

  private void controlMissingAttributes(Element element)
      throws UnsatisfiedPatternException {
    AttributeDefinitionList attributeDefinitionList =
        dTD.getAttributeDefinitionList(element.getNodeName());
    if (attributeDefinitionList == null)
      return;
    NamedNodeMap attributes = element.getAttributes();
    boolean hasMissingAttribute = false;
    for (int k = 0; k < attributeDefinitionList.size(); k++) {
      AttributeDefinition attributeDefinition =
                 attributeDefinitionList.get(k);
      String defaultType = attributeDefinition.getAttrDefaultType();
      if ((defaultType.equals(AttributeDefinition.REQUIRED)
           || defaultType.equals(AttributeDefinition.FIXED))
           && attributes.getNamedItem(attributeDefinition.getAttrName())
           == null)
      {
        hasMissingAttribute = true;
        break;
      }
    }
    if (!hasMissingAttribute)
      return;
    List missingAttributeNameList = new ArrayList();
    for (int k = 0; k < attributeDefinitionList.size(); k++) {
      AttributeDefinition attributeDefinition =
                 attributeDefinitionList.get(k);
      String defaultType = attributeDefinition.getAttrDefaultType();
      if ((defaultType.equals(AttributeDefinition.REQUIRED)
           || defaultType.equals(AttributeDefinition.FIXED))
           && attributes.getNamedItem(attributeDefinition.getAttrName()) == null)
        missingAttributeNameList.add("@" + attributeDefinition.getAttrName());
    }
    String[] missingAttributeNames =
        new String[missingAttributeNameList.size()];
    missingAttributeNameList.toArray(missingAttributeNames);
    throw new UnsatisfiedPatternException(missingAttributeNames);
  }

  protected void addAttributesContentToCache(Element parentElement) {
    if (contentCache.getAttributesContent(parentElement) == null)
      contentCache.put(parentElement, new AttributesContentImpl(
        dTD.getAttributeDefinitionList(parentElement.getNodeName())));
  }

  /////////////////////////////////////////
  // ElementDefinitionMapper implementation
  /////////////////////////////////////////

  public Object getSchema() {
    return this.dTD;
  }

  public Object getElementDefinition(Element element) {
    return element.getNodeName();
  }
}
