/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.List;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.framework.document.manage.AttributesContentAccess;
import org.xmloperator.framework.document.manage.AttributeChangeListener;
import org.xmloperator.framework.document.manage.ElementOperationListener;
import org.xmloperator.framework.document.manage.NodeManager;
import org.xmloperator.framework.document.manage.AttributeManager;
import org.xmloperator.framework.document.manage.DocumentManager;
import org.xmloperator.framework.document.manage.TreeViewDocumentListener;
import org.xmloperator.framework.document.manage.DocumentModifiedListener;
import org.xmloperator.framework.document.manage.NodeChangeListener;
import org.xmloperator.document.manage.NodeManagerImpl;
import org.xmloperator.document.manage.AttributeManagerImpl;

public final class DocumentManagerImpl implements DocumentManager {
  private final List nodeChangeListenerList = new ArrayList();
  private final List documentModifiedListenerList = new ArrayList();;
  private final NodeManagerImpl nodeManager;
  private final AttributeManagerImpl attributeManager;
  private boolean isDocumentModified = false;

  public DocumentManagerImpl(IdManager idManager) {
    this.nodeManager = new NodeManagerImpl(
        this.nodeChangeListenerList, this.documentModifiedListenerList);
    this.attributeManager = new AttributeManagerImpl(
        idManager, this.nodeChangeListenerList, this.documentModifiedListenerList);
  }

  public void setTreeViewDocumentListener(
      TreeViewDocumentListener treeViewDocumentListener) {
    this.nodeManager.setTreeViewDocumentListener(treeViewDocumentListener);
    this.attributeManager.setTreeViewDocumentListener(treeViewDocumentListener);
  }

  public void setAttributesContentAccess(
      AttributesContentAccess attributesContentAccess) {
    this.attributeManager.setAttributesContentAccess(attributesContentAccess);
  }

  public void setAttributeChangeListener(
      AttributeChangeListener attributeChangeListener) {
    this.attributeManager.setAttributeChangeListener(attributeChangeListener);
  }

  public void setElementOperationListener(
      ElementOperationListener elementOperationListener) {
    this.nodeManager.setElementOperationListener(elementOperationListener);
  }

  private void notifyDocumentModified() {
    if (this.documentModifiedListenerList == null)
      return;
    for (int k = 0; k < this.documentModifiedListenerList.size(); k++)
      ((DocumentModifiedListener)this.documentModifiedListenerList.get(k)).
          notifyDocumentModified();
  }

  /////////////////////////////////
  // DocumentManager implementation
  /////////////////////////////////

  public void addNodeChangeListener(NodeChangeListener nodeChangeListener) {
    this.nodeChangeListenerList.add(nodeChangeListener);
  }

  public void removeNodeChangeListener(NodeChangeListener nodeChangeListener) {
    this.nodeChangeListenerList.remove(nodeChangeListener);
  }

  public void addDocumentModifiedListener(
      DocumentModifiedListener documentModifiedListener) {
    this.documentModifiedListenerList.add(documentModifiedListener);
  }

  public void removeDocumentModifiedListener(
      DocumentModifiedListener documentModifiedListener) {
    this.documentModifiedListenerList.remove(documentModifiedListener);
  }

  public NodeManager getNodeManager() {
    return this.nodeManager;
  }

  public AttributeManager getAttributeManager() {
    return this.attributeManager;
  }

  public boolean isDocumentModified() {
    return this.isDocumentModified;
  }

  public void setDocumentModified(boolean isDocumentModified) {
    this.isDocumentModified = isDocumentModified;
    if (isDocumentModified)
      this.notifyDocumentModified();
  }
}
