/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.manage;

import java.util.Iterator;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Vector;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Comment;
import org.w3c.dom.Document;

import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.namespace.QualifiedName;
import org.xmloperator.framework.document.relaxng.NameClass;
import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.framework.document.relaxng.NonEmptyPattern;
import org.xmloperator.framework.document.relaxng.ElementDefinition;
import org.xmloperator.framework.document.relaxng.Ref;
import org.xmloperator.framework.document.relaxng.Attribute;
import org.xmloperator.framework.document.schema.ElementContent;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.schema.
  NotConformantToContentModelException;
import org.xmloperator.framework.document.manage.ElementDefinitionMapper;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.util.dom.ElementIndexer;
import org.xmloperator.util.dom.QualifiedNameImpl;
import org.xmloperator.util.dom.QualifiedNameUtils;
import org.xmloperator.util.dom.NamespaceUtils;
import org.xmloperator.util.dom.ElementSpecListUtils;
import org.xmloperator.document.relaxng.NameClassUtils;
import org.xmloperator.document.relaxng.PatternUtils;
import org.xmloperator.document.relaxng.ElementContentImpl;
import org.xmloperator.document.relaxng.ElementDefinitionUtils;
import org.xmloperator.document.manage.AbstractDocumentUtil;

public final class RNGDocumentUtil extends AbstractDocumentUtil {

  public RNGDocumentUtil(ElementDefinitionMapper elementDefinitionMapper) {
    super(elementDefinitionMapper);
  }

  private NonEmptyPattern getPattern(Element parentElement) {
    ElementDefinition parentElementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(
      parentElement);
    return parentElementDefinition != null ?
      ElementDefinitionUtils.getPattern(parentElementDefinition) : null;
  }

  //////////////////////////////
  // DocumentUtil implementation
  //////////////////////////////

  public String contentModelToString(Element parentElement,
    boolean isSummarized) {
    ElementDefinition parentElementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(
      parentElement);
    if (parentElementDefinition == null)
      return null;
    Pattern pattern = parentElementDefinition.getPattern();
    return pattern != null ?
      PatternUtils.childrenPatternToString(pattern, parentElement) : null;
  }

  public String elementFromContentModelToString(
    final Element element, boolean isSummarized) {
    Node parentNode = element.getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    try {
    ElementContent elementContent =
      elementDefinitionMapper.getElementContent((Element)parentNode);
    if (elementContent == null)
      return null;
    int qualifiedNameIndex = ElementIndexer.getFlatIndex(element);
    return elementContent.elementFromContentModelToString(qualifiedNameIndex,
      isSummarized, (Element)parentNode);
    }
    catch (NotConformantToContentModelException exception) {
      return null;
    }
  }

  public boolean areNodesMovable(final Node[] nodes, final int moveType) {
    switch (moveType) {
      case EditControl.MOVE_PREVIOUS:
      case EditControl.MOVE_NEXT:
        return areNodesMovableWithParentUnchanged(nodes, moveType);
      case EditControl.MOVE_CHILD:
      case EditControl.MOVE_PARENT: {
        if (!areNodesRemovable(nodes))
          return false;
        Node parentNode = moveType == EditControl.MOVE_PARENT ?
          nodes[0].getParentNode().getParentNode() :
          nodes[0].getPreviousSibling();
        if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
          return false;
        Node nextSibling = moveType == EditControl.MOVE_PARENT ?
          nodes[0].getParentNode().getNextSibling() : null;
        try {
          return areElementSpecsInsertable(
              QualifiedNameImpl.qualifiedNames(nodes),
              (Element)parentNode, nextSibling);
        }
        catch (UndefinedNamespaceException exception) {
          return true;
        }
      }
      default:
        throw new IllegalArgumentException();
    }
  }

  private boolean areNodesMovableWithParentUnchanged(
    final Node[] nodes, final int moveType) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE
      || !ElementIndexer.hasSignificantNodes(nodes))
      return true;
    Node movedNode = moveType == EditControl.MOVE_PREVIOUS ?
      nodes[0].getPreviousSibling() : nodes[nodes.length - 1].getNextSibling();
    if (movedNode == null)
      return false;
    if (!ElementIndexer.isSignificant(movedNode))
      return true;
    // Return true if the content is not valid BEFORE move
    if (!elementDefinitionMapper.isElementContentValid((Element)parentNode))
      return true;
    // Return false if the content is not valid AFTER move
    NonEmptyPattern pattern = getPattern((Element)parentNode);
    if (pattern == null)
      return true;
    try {
      QualifiedName[] qualifiedNames =
          QualifiedNameImpl.qualifiedNames((Element)parentNode);
      int firstQualifiedNameIndex = ElementIndexer.getFlatIndex(nodes[0]);
      int lastQualifiedNameIndex = ElementIndexer.getLastFlatIndex(nodes);
      if (moveType == EditControl.MOVE_PREVIOUS)
        ElementSpecListUtils.moveElementSpecsPrevious(qualifiedNames,
            firstQualifiedNameIndex, lastQualifiedNameIndex);
      else
        ElementSpecListUtils.moveElementSpecsNext(qualifiedNames,
            firstQualifiedNameIndex, lastQualifiedNameIndex);
      return ElementContentImpl.isContentValid(pattern, qualifiedNames,
          QualifiedNameImpl.attributeQualifiedNames((Element)parentNode));
    }
    catch (UndefinedNamespaceException exception) {
      return true;
    }
  }

  public boolean areNodesRemovable(final Node[] nodes) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode.getNodeType() != Node.ELEMENT_NODE
      || !ElementIndexer.hasSignificantNodes(nodes))
      return true;
    // Return true if the content is not valid BEFORE remove
    if (!elementDefinitionMapper.isElementContentValid((Element)parentNode))
      return true;
    // Return false if the content is not valid AFTER remove
    NonEmptyPattern pattern = getPattern((Element)parentNode);
    if (pattern == null)
      return true;
    try {
      Object[] qualifiedNames =
          QualifiedNameImpl.qualifiedNames((Element)parentNode);
      int firstQualifiedNameIndex = ElementIndexer.getFlatIndex(nodes[0]);
      int lastQualifiedNameIndex = ElementIndexer.getLastFlatIndex(nodes);
      qualifiedNames = ElementSpecListUtils.removeElementSpecsAt(
          qualifiedNames, firstQualifiedNameIndex,
          lastQualifiedNameIndex - firstQualifiedNameIndex + 1);
      return ElementContentImpl.isContentValid(pattern, qualifiedNames,
          QualifiedNameImpl.attributeQualifiedNames((Element)parentNode));
    }
    catch (UndefinedNamespaceException exception) {
      return true;
    }
  }

  public boolean canElementHaveChilds(Element element) {
    ElementDefinition elementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(element);
    return elementDefinition != null ?
      ElementDefinitionUtils.canContainChilds(elementDefinition) : true;
  }

  public boolean canElementHaveText(Element element) {
    ElementDefinition elementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(element);
    return elementDefinition != null ?
      ElementDefinitionUtils.canContainText(elementDefinition) : true;
  }

  public boolean shouldElementHaveText(Element element) {
    ElementDefinition elementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(element);
    return elementDefinition != null ?
      ElementDefinitionUtils.shouldContainText(elementDefinition) : false;
  }

  public boolean areElementNamesInsertable(
    final String[] insertedElementNames, final Element parentElement,
    final Node nextSibling) {
    QualifiedName[] insertedQualifiedNames =
      new QualifiedName[insertedElementNames.length];
    try {
      for (int k = 0; k < insertedQualifiedNames.length; k++)
        insertedQualifiedNames[k] =
        new QualifiedNameImpl(parentElement, insertedElementNames[k]);
      return areElementSpecsInsertable(insertedQualifiedNames, parentElement,
                                       nextSibling);
    }
    catch (UndefinedNamespaceException exception) {
      return false;
    }
  }

  // Each elementSpec must be either an QualifiedName or an ElementDefinition.
  private boolean areElementSpecsInsertable(
    final Object[] insertedElementSpecs, final Element parentElement,
    final Node nextSibling) {
    // Return true if the content is not valid BEFORE insert
    if (!elementDefinitionMapper.isElementContentValid(parentElement))
      return true;
    // Return false if the content is not valid AFTER insert
    NonEmptyPattern pattern = getPattern(parentElement);
    if (pattern == null)
      return true;
    try {
      Object[] elementSpecs = QualifiedNameImpl.qualifiedNames(parentElement);
      int elementSpecIndex =
          nextSibling != null ?
          ElementIndexer.getFlatIndex(nextSibling) : elementSpecs.length;
      elementSpecs = ElementSpecListUtils.insertElementSpecsAt(
          insertedElementSpecs, elementSpecs, elementSpecIndex);
      return ElementContentImpl.isContentValid(pattern, elementSpecs,
          QualifiedNameImpl.attributeQualifiedNames(parentElement));
    }
    catch (UndefinedNamespaceException exception) {
      return false;
    }
  }

  public boolean isElementUnpackable(final Element element) {
    Element parentElement = (Element)element.getParentNode();
    // Return true if the content is not valid BEFORE unpack
    if (!elementDefinitionMapper.isElementContentValid(parentElement))
      return true;
    // Return false if the content is not valid AFTER unpack
    NonEmptyPattern pattern = getPattern(parentElement);
    if (pattern == null)
      return true;
    try {
      Object[] qualifiedNames = QualifiedNameImpl.qualifiedNames(parentElement);
      int qualifiedNameIndex = ElementIndexer.getFlatIndex(element);
      Object[] newQualifiedNames = ElementSpecListUtils.removeElementSpecsAt(
          qualifiedNames, qualifiedNameIndex, 1);
      Object[] insertedQualifiedNames = QualifiedNameImpl.qualifiedNames(element);
      newQualifiedNames = ElementSpecListUtils.insertElementSpecsAt(
          insertedQualifiedNames, newQualifiedNames, qualifiedNameIndex);
      return ElementContentImpl.isContentValid(pattern, newQualifiedNames,
          QualifiedNameImpl.attributeQualifiedNames(parentElement));
    }
    catch (UndefinedNamespaceException exception) {
      return true;
    }
  }

  public boolean isElementNameReplacable(Element element,
    String newElementName) {
    Collection elementDefinitions = elementDefinitionsForReplace(element);
    if (elementDefinitions == null)
      return false;
    try {
    ElementDefinition elementDefinition =
      ElementDefinitionUtils.fromElementNameToElementDefinition(
      elementDefinitions, element, newElementName);
    if (elementDefinition == null)
      return false;
    Pattern childPattern = ((ElementDefinition)elementDefinition).getPattern();
    if (childPattern == null)
      return false;
    if (childPattern.isEmpty())
      return !element.hasChildNodes();
    return ElementContentImpl.isContentValid((NonEmptyPattern)childPattern,
      QualifiedNameImpl.qualifiedNames(element),
      QualifiedNameImpl.attributeQualifiedNames(element));
    }
    catch (UndefinedNamespaceException exception) {
      return false;
    }
  }

  public Collection elementNamesForAppend(boolean isLazy,
    Element parentElement) {
    Set elementNames =
      ElementDefinitionUtils.fromElementDefinitionSetToElementNameSet(
      parentElement, elementDefinitionsForAppend(parentElement));
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementDefinitionsForAppend(Element parentElement) {
    try {
      ElementContent elementContent =
        elementDefinitionMapper.getElementContent(parentElement);
      if (elementContent == null)
        return null;
      Set elementDefinitionSet =
        elementContent.elementDefinitionsInsertableBefore(
        elementContent.getElementSpecCount());
      return elementDefinitionSet.size() != 0 ? elementDefinitionSet : null;
    }
    catch (NotConformantToContentModelException exception) {
      return null;
    }
  }

  public Collection elementNamesForInsert(boolean isLazy,
    final Element element, final boolean isInsertedAfter) {
    Set elementNames = null;
    Node parentNode = element.getParentNode();
    if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE)
      elementNames =
        ElementDefinitionUtils.fromElementDefinitionSetToElementNameSet(
        (Element)parentNode,
        elementDefinitionsForInsert(element, isInsertedAfter));
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementDefinitionsForInsert(
    final Element element, final boolean isInsertedAfter) {
    Node parentNode = element.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      throw new IllegalArgumentException();
    ElementContent elementContent;
    try {
      elementContent =
        elementDefinitionMapper.getElementContent((Element)parentNode);
      if (elementContent == null)
        return null;
    }
    catch (NotConformantToContentModelException exception) {
      return null;
    }
    int qualifiedNameIndex = ElementIndexer.getFlatIndex(element);
    if (isInsertedAfter)
      qualifiedNameIndex++;
    Collection elementDefinitionSet =
      elementContent.elementDefinitionsInsertableBefore(qualifiedNameIndex);
    return elementDefinitionSet.size() != 0 ? elementDefinitionSet : null;
  }

  public Collection elementNamesForReplace(boolean isLazy, final Node node) {
    Set elementNames = null;
    Node parentNode = node.getParentNode();
    if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE) {
      if (elementDefinitionMapper.isElementContentValid((Element)parentNode))
        elementNames =
          ElementDefinitionUtils.fromElementDefinitionSetToElementNameSet(
          (Element)parentNode, elementDefinitionsForReplace(node));
      else
        isLazy = true;
    }
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementDefinitionsForReplace(final Node node) {
    if (node.getNodeType() == Node.ELEMENT_NODE)
      return elementDefinitionsForReplace((Element)node);
    Node parentNode = node.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    Element element = ElementBrowser.getPreviousElement(node);
    boolean isInsertedAfter = true;
    if (element == null) {
      element = ElementBrowser.getNextElement(node);
      isInsertedAfter = false;
    }
    return element != null ?
      this.elementDefinitionsForInsert(element, isInsertedAfter) :
      this.elementDefinitionsForAppend((Element)parentNode);
  }

  private Set elementDefinitionsForReplace(final Element element) {
    Node parentNode = element.getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    ElementContent elementContent;
    try {
      elementContent =
        elementDefinitionMapper.getElementContent((Element)parentNode);
      if (elementContent == null)
        return null;
    }
    catch (NotConformantToContentModelException exception) {
      return null;
    }
    Set elementDefinitionSet = new HashSet();
    int qualifiedNameIndex = ElementIndexer.getFlatIndex(element);
    elementContent.addElementDefinitionsForReplace(elementDefinitionSet,
      qualifiedNameIndex);
    Object[] qualifiedNames;
    QualifiedName[] attributeQualifiedNames;
    try {
      qualifiedNames = QualifiedNameImpl.qualifiedNames((Element)parentNode);
      attributeQualifiedNames =
        QualifiedNameImpl.attributeQualifiedNames((Element)parentNode);
    }
    catch (UndefinedNamespaceException exception) {
      return null;
    }
    qualifiedNames = ElementSpecListUtils.removeElementSpecsAt(qualifiedNames,
      qualifiedNameIndex, 1);
    NonEmptyPattern pattern = getPattern((Element)parentNode);
    try {
      ElementContent subContent = new ElementContentImpl(pattern,
        qualifiedNames, attributeQualifiedNames);
      Set insertableElementDefinitionSet =
        subContent.elementDefinitionsInsertableBefore(qualifiedNameIndex);
      if (insertableElementDefinitionSet != null)
        elementDefinitionSet.addAll(insertableElementDefinitionSet);
    }
    catch (NotConformantToContentModelException exception) {
    }
    return elementDefinitionSet;
  }

  public Collection elementNamesForPack(final boolean isLazy,
    final Node[] nodes) {
    Set elementNames = null;
    Node parentNode = nodes[0].getParentNode();
    if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE)
      elementNames =
        ElementDefinitionUtils.fromElementDefinitionSetToElementNameSet(
        (Element)parentNode, elementDefinitionsForPack(nodes));
    if (isLazy)
      if (elementNames != null)
        elementNames.add(":");
      else
        elementNames = Collections.singleton(":");
    return elementNames;
  }

  private Collection elementDefinitionsForPack(final Node[] nodes) {
    Node parentNode = nodes[0].getParentNode();
    if (parentNode == null || parentNode.getNodeType() != Node.ELEMENT_NODE)
      return null;
    ElementDefinition parentElementDefinition =
      (ElementDefinition)elementDefinitionMapper.getElementDefinition(
      (Element)parentNode);
    if (parentElementDefinition == null)
      return null;
    if (!ElementIndexer.hasSignificantChilds(parentNode))
      return this.elementDefinitionsForAppend((Element)parentNode);
    Set elementDefinitionSet = ElementDefinitionUtils.childElementDefinitionSet(
      parentElementDefinition); // to be revisited ...
    return elementDefinitionSet;
  }

  public Collection elementAndAttributeNamesForRequiredInsert(
    Element parentElement, Object[] unsatisfiedPatterns) {
    Set attributeQualifiedNameSet = new HashSet();
    Set elementQualifiedNameSet = new HashSet();
    for (int k = 0; k < unsatisfiedPatterns.length; k++) {
      Object unsatisfiedPattern = unsatisfiedPatterns[k];
      if (unsatisfiedPattern instanceof Ref) {
        NameClass nameClass =
          ((Ref)unsatisfiedPattern).getElementDefinition().getNameClass();
        NameClassUtils.appendNameClassToQualifiedNameSet(
          elementQualifiedNameSet, nameClass);
      }
      else if (unsatisfiedPattern instanceof Attribute) {
        NameClass nameClass = ((Attribute)unsatisfiedPattern).getNameClass();
        NameClassUtils.appendNameClassToQualifiedNameSet(
          attributeQualifiedNameSet, nameClass);
      }
      else
        throw new IllegalArgumentException();
    }
    Set elementAndAttributeNameSet = new HashSet();
    {
      Iterator iterator = attributeQualifiedNameSet.iterator();
      while (iterator.hasNext()) {
        QualifiedName qualifiedName = (QualifiedName)iterator.next();
        elementAndAttributeNameSet.add(
            "@" + QualifiedNameUtils.getPseudoAttributeName(
            qualifiedName, parentElement));
      }
    }
    {
      Iterator iterator = elementQualifiedNameSet.iterator();
      while (iterator.hasNext()) {
        QualifiedName qualifiedName = (QualifiedName)iterator.next();
        elementAndAttributeNameSet.add(
            QualifiedNameUtils.getPseudoElementName(
            qualifiedName, parentElement));
      }
    }
    return elementAndAttributeNameSet;
  }
}
