/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng;

import java.util.Set;

import org.w3c.dom.Element;

import org.xmloperator.framework.document.relaxng.NameClass;
import org.xmloperator.framework.document.relaxng.AnyName;
import org.xmloperator.framework.document.relaxng.NsName;
import org.xmloperator.framework.document.relaxng.Name;
import org.xmloperator.framework.document.relaxng.ChoiceNameClass;
import org.xmloperator.util.dom.NamespaceUtils;
import org.xmloperator.util.dom.QualifiedNameImpl;

public abstract class NameClassUtils {

  ///////////////////////////////////////////////////////////
  // Methods for geting general information about a NameClass
  ///////////////////////////////////////////////////////////

  // return a string representing the NameClass
  public static final String nameClassToString(
      final NameClass nameClass, Element contextElement) {
    final StringBuffer s = new StringBuffer();
    appendNameClassToString(s, true, nameClass, contextElement);
    return s.toString();
  }

  private static final void appendNameClassToString(final StringBuffer s,
      final boolean isRoot, final NameClass nameClass,
      final Element contextElement) {
    switch (nameClass.getNameClassType()) {
      case NameClass.NAMECLASS_ANYNAME: {
        s.append("*");
        NameClass except = ((AnyName)nameClass).getExcept();
        if (except != null) {
          s.append('-');
          appendNameClassToString(s, true, except, contextElement);
        }
        break;
      }
      case NameClass.NAMECLASS_NSNAME: {
        String nsURI = ((NsName)nameClass).getNsURI();
        String prefix = nsURI.length() != 0 ?
            NamespaceUtils.namespaceToPrefix(contextElement, nsURI) : "";
        s.append(prefix != null ? prefix :
                 nsURI.equals(NamespaceUtils.defaultNamespace(contextElement)) ?
                 "D" : "?");
        s.append(":*");
        NameClass except = ((NsName)nameClass).getExcept();
        if (except != null) {
          s.append('-');
          appendNameClassToString(s, true, except, contextElement);
        }
        break;
      }
      case NameClass.NAMECLASS_NAME: {
        String nsURI = ((Name)nameClass).getNsURI();
        if (nsURI.length() != 0) {
          String prefix =
              NamespaceUtils.namespaceToPrefix(contextElement, nsURI);
          if (prefix != null) {
            s.append(prefix);
            s.append(':');
          }
          else if (!nsURI.
                   equals(NamespaceUtils.defaultNamespace(contextElement)))
            s.append("?:");
        }
        s.append(((Name)nameClass).getName());
        break;
      }
      case NameClass.NAMECLASS_CHOICE:
        if (isRoot)
          s.append('(');
        for (int k = 0; k < ((ChoiceNameClass)nameClass).getChildCount(); k++) {
          if (k != 0)
            s.append('|');
          appendNameClassToString(s, false,
            ((ChoiceNameClass)nameClass).getChild(k), contextElement);
        }
        if (isRoot)
          s.append(')');
        break;
    }
  }

  ////////////////////////////////////////
  // Methods for collecting possible names
  ////////////////////////////////////////

  public static final void appendNameClassToQualifiedNameSet(
    Set qualifiedNameSet, NameClass nameClass) {
    switch (nameClass.getNameClassType()) {
      case NameClass.NAMECLASS_ANYNAME:
        qualifiedNameSet.add(QualifiedNameImpl.ANY_NAME);
        break;
      case NameClass.NAMECLASS_CHOICE:
        for (int k = 0; k < ((ChoiceNameClass)nameClass).getChildCount(); k++)
          appendNameClassToQualifiedNameSet(
            qualifiedNameSet, ((ChoiceNameClass)nameClass).getChild(k));
        break;
      case NameClass.NAMECLASS_NAME:
        qualifiedNameSet.add(new QualifiedNameImpl(
          ((Name)nameClass).getNsURI(), ((Name)nameClass).getName()));
        break;
      case NameClass.NAMECLASS_NSNAME:
        qualifiedNameSet.add(
          new QualifiedNameImpl(((NsName)nameClass).getNsURI()));
        break;
    }
  }

  ////////////////////////////////
  // Methods for validating a name
  ////////////////////////////////

  // Return true if a couple (namespaceURI, localName) is accepted
  //   by a nameClass.
  // Return always false for an empty name. This is used by the ElementContent
  //   class for verifying that a pattern is fully satisfied.
  public static final boolean isNameContained(NameClass nameClass,
    String namespaceURI, String localName) {
    if (nameClass == null || localName.length() == 0)
      return false;
    switch (nameClass.getNameClassType()) {
      case NameClass.NAMECLASS_ANYNAME:
        return !isNameContained(((AnyName)nameClass).getExcept(),
          namespaceURI, localName);
      case NameClass.NAMECLASS_NSNAME:
        return ((NsName)nameClass).getNsURI().equals(namespaceURI)
          && !isNameContained(((NsName)nameClass).getExcept(),
          namespaceURI, localName);
      case NameClass.NAMECLASS_NAME:
        return ((Name)nameClass).getNsURI().equals(namespaceURI)
          && ((Name)nameClass).getName().equals(localName);
      case NameClass.NAMECLASS_CHOICE:
        for (int k = 0; k < ((ChoiceNameClass)nameClass).getChildCount(); k++)
          if (isNameContained(((ChoiceNameClass)nameClass).getChild(k),
            namespaceURI, localName))
            return true;
        return false;
      default:
        throw new IllegalArgumentException();
    }
  }
}
