/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng;

import java.io.IOException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.ErrorHandler;
import org.apache.xerces.xni.parser.XMLConfigurationException;
import org.apache.xerces.xni.parser.XMLEntityResolver;
import org.apache.xerces.parsers.SAXParser;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.document.relaxng.Schema;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.SchemaBuilder;

public final class SchemaHandler implements ContentHandler {
  private static final String RELAX_NG_URI =
    "http://relaxng.org/ns/structure/1.0";

  private static final class XNISAXParser extends SAXParser {

    public XNISAXParser() {
    }

    public void setXMLEntityResolver(XMLEntityResolver entityResolver) {
      try {
        fConfiguration.setProperty(ENTITY_RESOLVER, entityResolver);
      }
      catch (XMLConfigurationException exception) {
      }
    }
  }

  public static final Schema loadSchema(String systemId,
    XMLEntityResolver entityResolver)
    throws IOException, SAXException {
    SchemaBuilder schemaBuilder = new SchemaBuilder();
    SchemaHandler schemaHandler = new SchemaHandler(schemaBuilder);
    XNISAXParser parser = new XNISAXParser();
    parser.setContentHandler(schemaHandler);
    parser.setErrorHandler(
      new ErrorHandler() {
        public void warning(SAXParseException exception) throws SAXException {
        }
        public void error(SAXParseException exception) throws SAXException {
          throw exception;
        }
        public void fatalError(SAXParseException exception)
          throws SAXException {
          throw exception;
        }
      });
    if (entityResolver != null)
      parser.setXMLEntityResolver(entityResolver);
    try {
      parser.parse(systemId);
    }
    catch (RelaxNGParseException exception) {
      //System.out.println("EventCount=" + schemaHandler.eventCount);
      exception.eventCount = schemaHandler.eventCount;
      throw exception;
    }
    return schemaBuilder.result();
  }

  private SchemaBuilder schemaBuilder;
  private long eventCount = 0;
  private int ignoredElementLevelCount = 0;

  private SchemaHandler(SchemaBuilder schemaBuilder) {
    this.schemaBuilder = schemaBuilder;
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void setDocumentLocator (Locator locator) {
  }

  public void startDocument () throws SAXException {
  }

  public void endDocument () throws SAXException {
  }

  public void startPrefixMapping (String prefix, String uri)
    throws SAXException {
  }

  public void endPrefixMapping (String prefix) throws SAXException {
  }

  public void startElement (String namespaceURI, String localName,
  String rawName, Attributes atts) throws SAXException {
    eventCount++;
    if (ignoredElementLevelCount == 0 && namespaceURI == RELAX_NG_URI)
      schemaBuilder.startElement(localName, atts);
    else
      ignoredElementLevelCount++;
  }

  public void endElement (String namespaceURI, String localName,
    String rawName)	throws SAXException {
    eventCount++;
    if (ignoredElementLevelCount == 0 && namespaceURI == RELAX_NG_URI)
      schemaBuilder.endElement();
    else
      ignoredElementLevelCount--;
  }

  public void characters (char ch[], int start, int length)
    throws SAXException {
    if (ignoredElementLevelCount == 0)
      schemaBuilder.characters(ch, start, length);
  }

  public void ignorableWhitespace (char ch[], int start, int length)
    throws SAXException {
  }

  public void processingInstruction (String target, String data)
    throws SAXException {
  }

  public void skippedEntity (String name)	throws SAXException {
  }
}
