/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng;

import java.io.PrintWriter;

import org.xmloperator.framework.document.relaxng.ElementDefinition;
import org.xmloperator.framework.document.relaxng.NameClass;
import org.xmloperator.framework.document.relaxng.Name;
import org.xmloperator.framework.document.relaxng.NsName;
import org.xmloperator.framework.document.relaxng.AnyName;
import org.xmloperator.framework.document.relaxng.ChoiceNameClass;
import org.xmloperator.framework.document.relaxng.NonEmptyPattern;
import org.xmloperator.framework.document.relaxng.Param;
import org.xmloperator.framework.document.relaxng.Data;
import org.xmloperator.framework.document.relaxng.Value;
import org.xmloperator.framework.document.relaxng.ListPattern;
import org.xmloperator.framework.document.relaxng.Attribute;
import org.xmloperator.framework.document.relaxng.Ref;
import org.xmloperator.framework.document.relaxng.GroupPattern;
import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.framework.document.relaxng.Schema;

public abstract class SchemaToXMLTranslator {

  public static void printSchema(PrintWriter out, Schema schema) {
    if (out == null || schema == null)
      throw new IllegalArgumentException();
    out.println("<?xml version=\"1.0\" ?>");
    out.print("<grammar");
    out.print(attrToString("xmlns", "http://relaxng.org/ns/structure/1.0"));
    out.print("><start>");
    printPattern(out, schema.getStart());
    out.print("</start>");
    for (int k = 0; k < schema.getElementDefinitionCount(); k++)
      printDefine(out, schema.getElementDefinition(k));
    out.print("</grammar>");
  }

  private static void printNameClass(PrintWriter out, NameClass nameClass) {
    switch (nameClass.getNameClassType()) {
      case NameClass.NAMECLASS_ANYNAME:
        out.print("<anyName>");
        printExceptNameClass(out, ((AnyName)nameClass).getExcept());
        out.print("</anyName>");
        break;
      case NameClass.NAMECLASS_NSNAME:
        out.print("<nsName");
        out.print(attrToString("ns", ((NsName)nameClass).getNsURI()));
        out.print('>');
        printExceptNameClass(out, ((NsName)nameClass).getExcept());
        out.print("</nsName>");
        break;
      case NameClass.NAMECLASS_NAME:
        out.print("<name");
        out.print(attrToString("ns", ((Name)nameClass).getNsURI()));
        out.print('>');
        out.print(((Name)nameClass).getName());
        out.print("</name>");
        break;
      case NameClass.NAMECLASS_CHOICE:
        out.print("<choice>");
        for (int k = 0; k < ((ChoiceNameClass)nameClass).getChildCount(); k++)
          printNameClass(out, ((ChoiceNameClass)nameClass).getChild(k));
        out.print("</choice>");
        break;
      default:
        throw new IllegalArgumentException();
    }
  }

  private static void printExceptNameClass(PrintWriter out,
    NameClass nameClass) {
    if (nameClass == null)
      return;
    out.print("<except>");
    printNameClass(out, nameClass);
    out.print("</except>");
  }

  private static void printPattern(PrintWriter out, Pattern pattern) {
    if (pattern == null)
      out.print("<notAllowed/>");
    else if (pattern.isEmpty())
      out.print("<empty/>");
    else
      printNonEmptyPattern(out, (NonEmptyPattern)pattern);
  }

  private static void printNonEmptyPattern(PrintWriter out,
    NonEmptyPattern pattern) {
    boolean canBeRepeated = pattern.canBeRepeated();
    if (canBeRepeated)
      out.print("<oneOrMore>");
    switch (pattern.getPatternType()) {
      case NonEmptyPattern.PATTERN_TEXT:
        out.print("<text/>");
        break;
      case NonEmptyPattern.PATTERN_DATA:
        printDataPattern(out, (Data)pattern);
        break;
      case NonEmptyPattern.PATTERN_VALUE:
        printValuePattern(out, (Value)pattern);
        break;
      case NonEmptyPattern.PATTERN_LIST:
        printListPattern(out, (ListPattern)pattern);
        break;
      case NonEmptyPattern.PATTERN_ATTRIBUTE:
        printAttributePattern(out, (Attribute)pattern);
        break;
      case NonEmptyPattern.PATTERN_REF:
        printRefPattern(out, (Ref)pattern);
        break;
      case NonEmptyPattern.PATTERN_GROUP:
        printGroupPattern(out, (GroupPattern)pattern);
        break;
      default:
        throw new IllegalArgumentException();
    }
    if (canBeRepeated)
      out.print("</oneOrMore>");
  }

  private static void printDataPattern(PrintWriter out, Data pattern) {
    out.print("<data");
    out.print(attrToString("type", pattern.getType()));
    out.print(attrToString("datatypeLibrary", pattern.getDatatypeLibraryURI()));
    out.print('>');
    for (int k = 0; k < pattern.getParamCount(); k++)
      printParam(out, pattern.getParam(k));
    Pattern exceptPattern = pattern.getExcept();
    if (exceptPattern != null) {
      out.print("<except>");
      printPattern(out, exceptPattern);
      out.print("</except>");
    }
    out.print("</data>");
  }

  private static void printParam(PrintWriter out, Param param) {
    out.print("<param");
    out.print(attrToString("name", param.getName()));
    out.print('>');
    out.print(exportValue(param.getValue()));
    out.print("</param>");
  }

  private static void printValuePattern(PrintWriter out, Value pattern) {
    out.print("<value");
    out.print(attrToString("datatypeLibrary", pattern.getDatatypeLibraryURI()));
    out.print(attrToString("type", pattern.getType()));
    out.print(attrToString("ns", pattern.getNsURI()));
    out.print('>');
    out.print(exportValue(pattern.getValue()));
    out.print("</value>");
  }

  private static void printListPattern(PrintWriter out, ListPattern pattern) {
    out.print("<list>");
    printPattern(out, pattern.getPattern());
    out.print("</list>");
  }

  private static void printAttributePattern(PrintWriter out, Attribute pattern)
    {
    out.print("<attribute>");
    printNameClass(out, pattern.getNameClass());
    printPattern(out, pattern.getPattern());
    out.print("</attribute>");
  }

  private static void printRefPattern(PrintWriter out, Ref pattern) {
    out.print("<ref");
    out.print(attrToString("name", defineName(pattern.getElementDefinition())));
    out.print("/>");
  }

  private static String defineName(ElementDefinition elementDefinition) {
    String defineName = Integer.toHexString(elementDefinition.hashCode());
    NameClass nameClass = elementDefinition.getNameClass();
    if (nameClass instanceof Name)
      defineName = ((Name)nameClass).getName() + "-" + defineName;
    return defineName;
  }

  private static void printGroupPattern(PrintWriter out, GroupPattern pattern) {
    String patternName;
    int patternType = pattern.getGroupType();
    switch (patternType) {
      case GroupPattern.GROUP_CHOICE_INCLUDING_EMPTY_PATTERN:
      case GroupPattern.GROUP_CHOICE_EXCLUDING_EMPTY_PATTERN:
        patternName = "choice";
        break;
      case GroupPattern.GROUP_GROUP:
        patternName = "group";
        break;
      case GroupPattern.GROUP_INTERLEAVE:
        patternName = "interleave";
        break;
      default:
        throw new IllegalArgumentException();
    }
    out.print('<');
    out.print(patternName);
    out.print('>');
    if (patternType == GroupPattern.GROUP_CHOICE_INCLUDING_EMPTY_PATTERN)
      out.print("<empty/>");
    for (int k = 0; k < pattern.getChildCount(); k++)
      printNonEmptyPattern(out, pattern.getChild(k));
    out.print("</");
    out.print(patternName);
    out.print('>');
  }

  private static void printDefine(PrintWriter out,
    ElementDefinition elementDefinition) {
    out.print("<define");
    out.print(attrToString("name", defineName(elementDefinition)));
    out.print("><element>");
    printNameClass(out, elementDefinition.getNameClass());
    printPattern(out, elementDefinition.getPattern());
    out.print("</element></define>");
  }

  private static String attrToString(String name, String value) {
    StringBuffer s = new StringBuffer();
    s.append(' ');
    s.append(name);
    s.append("=\"");
    s.append(exportValue(value));
    s.append('"');
    return s.toString();
  }

  private static String exportValue(String value) {
    StringBuffer s = new StringBuffer();
    for (int k = 0; k < value.length(); k++) {
      char ch = value.charAt(k);
      if (ch == '"')
        s.append("&#34;");
      else if (ch == '&')
        s.append("&#38;");
      else if (ch == '<')
        s.append("&#60;");
      else
        s.append(ch);
    }
    return s.toString();
  }
}
