/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import java.util.Map;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.document.relaxng.impl.EmptyPatternImpl;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.ContentHandler;
import org.xmloperator.document.relaxng.parse.NonEmptyPatternBuilder;

// pattern  ::= <empty/> | nonEmptyPattern
public class PatternBuilder implements ContentHandler {
  private static final int STATE_EXPECT_PATTERN = 0;
  private static final int STATE_INSIDE_PATTERN = 1;
  private static final int STATE_EXPECT_END_PATTERN = 2;
  private static final int STATE_END = 3;

  private int state = STATE_EXPECT_PATTERN; // STATE_*
  private NonEmptyPatternBuilder nonEmptyPatternBuilder = null;

  public static final boolean isStart(String elementName) {
    return elementName.equals("empty")
      || NonEmptyPatternBuilder.isStart(elementName);
  }

  public PatternBuilder() {
  }

  public Pattern result(Map fromNameToElementDefinition) {
    if (state != STATE_END)
      return null;
    return nonEmptyPatternBuilder != null ?
      (Pattern)nonEmptyPatternBuilder.result(fromNameToElementDefinition) :
      (Pattern)EmptyPatternImpl.EMPTY_PATTERN;
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("pattern", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    switch (state) {
      case STATE_EXPECT_PATTERN:
        if (name.equals("empty")) {
          state = STATE_EXPECT_END_PATTERN;
          break;
        }
        else {
          nonEmptyPatternBuilder = new NonEmptyPatternBuilder();
          state = STATE_INSIDE_PATTERN;
        }
      case STATE_INSIDE_PATTERN:
        nonEmptyPatternBuilder.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    switch (state) {
      case STATE_INSIDE_PATTERN:
        if (nonEmptyPatternBuilder.endElement()) {
          state = STATE_END;
          return true;
        }
        return false;
      case STATE_EXPECT_END_PATTERN:
        state = STATE_END;
        return true;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_PATTERN:
        nonEmptyPatternBuilder.characters(ch, start, length);
        break;
    }
  }
}
