/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.relaxng.parse;

import java.util.Map;
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import org.xmloperator.framework.document.relaxng.Pattern;
import org.xmloperator.framework.document.relaxng.ElementDefinition;
import org.xmloperator.framework.document.relaxng.Schema;
import org.xmloperator.document.relaxng.impl.ElementDefinitionImpl;
import org.xmloperator.document.relaxng.impl.SchemaImpl;
import org.xmloperator.document.relaxng.parse.RelaxNGParseException;
import org.xmloperator.document.relaxng.parse.ContentHandler;
import org.xmloperator.document.relaxng.parse.PatternBuilder;
import org.xmloperator.document.relaxng.parse.ElementDefinitionBuilder;

// grammar  ::= <grammar> <start> top </start> define* </grammar>
// top      ::= <notAllowed/> | pattern
public class SchemaBuilder implements ContentHandler {
  private static final boolean debug = false;
  private static final int STATE_EXPECT_GRAMMAR = 0;
  private static final int STATE_EXPECT_START = 1;
  private static final int STATE_EXPECT_TOP = 2;
  private static final int STATE_INSIDE_TOP = 3;
  private static final int STATE_EXPECT_END_TOP = 4;
  private static final int STATE_EXPECT_END_START = 5;
  private static final int STATE_EXPECT_DEFINE = 6;
  private static final int STATE_INSIDE_DEFINE = 7;
  private static final int STATE_END = 8;

  private int state = STATE_EXPECT_GRAMMAR; // STATE_*
  private final List elementDefinitionBuilderList = new ArrayList();
  private PatternBuilder topBuilder = null;
  private ElementDefinitionBuilder elementDefinitionBuilder = null;

  public static final boolean isStart(String elementName) {
    return elementName.equals("grammar");
  }

  public SchemaBuilder() {
  }

  public Schema result() {
    if (state != STATE_END)
      return null;
    int elementDefinitionCount = elementDefinitionBuilderList.size();
    final ElementDefinition[] elementDefinitions =
      new ElementDefinition[elementDefinitionCount];
    final Hashtable fromNameToElementDefinition = new Hashtable();
    for (int k = 0; k < elementDefinitionCount; k++) {
      ElementDefinitionBuilder elementDefinitionBuilder =
        (ElementDefinitionBuilder)elementDefinitionBuilderList.get(k);
      ElementDefinitionImpl elementDefinition =
        elementDefinitionBuilder.partialResult();
      elementDefinitions[k] = elementDefinition;
      fromNameToElementDefinition.put(elementDefinitionBuilder.getName(),
        elementDefinition);
    }
    for (int k = 0; k < elementDefinitionCount; k++)
      ((ElementDefinitionBuilder)elementDefinitionBuilderList.get(k)).
        completeResult((ElementDefinitionImpl)elementDefinitions[k],
        fromNameToElementDefinition);
    SchemaImpl schema = new SchemaImpl(elementDefinitions);
    if (topBuilder != null)
      schema.setStart(topBuilder.result(fromNameToElementDefinition));
    return schema;
  }

  private void error() throws SAXException {
    throw new RelaxNGParseException("grammar", state);
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void startElement(String name, Attributes atts) throws SAXException {
    if (debug)
      System.out.println("start " + name);
    switch (state) {
      case STATE_EXPECT_GRAMMAR:
        if (!name.equals("grammar"))
          error();
        state = STATE_EXPECT_START;
        break;
      case STATE_EXPECT_START:
        if (!name.equals("start"))
          error();
        state = STATE_EXPECT_TOP;
        break;
      case STATE_EXPECT_TOP:
        if (name.equals("notAllowed")) {
          state = STATE_EXPECT_END_TOP;
          break;
        }
        topBuilder = new PatternBuilder();
        state = STATE_INSIDE_TOP;
      case STATE_INSIDE_TOP:
        topBuilder.startElement(name, atts);
        break;
      case STATE_EXPECT_DEFINE:
        elementDefinitionBuilder = new ElementDefinitionBuilder();
        state = STATE_INSIDE_DEFINE;
      case STATE_INSIDE_DEFINE:
        elementDefinitionBuilder.startElement(name, atts);
        break;
      default:
        error();
    }
  }

  public boolean endElement() throws SAXException {
    if (debug)
      System.out.println("end");
    switch (state) {
      case STATE_INSIDE_TOP:
        if (topBuilder.endElement())
          state = STATE_EXPECT_END_START;
        return false;
      case STATE_EXPECT_END_TOP:
        state = STATE_EXPECT_END_START;
        return false;
      case STATE_EXPECT_END_START:
        state = STATE_EXPECT_DEFINE;
        return false;
      case STATE_INSIDE_DEFINE:
        if (elementDefinitionBuilder.endElement()) {
          elementDefinitionBuilderList.add(elementDefinitionBuilder);
          elementDefinitionBuilder = null;
          state = STATE_EXPECT_DEFINE;
        }
        return false;
      case STATE_EXPECT_DEFINE:
        state = STATE_END;
        return true;
      default:
        error();
        return true;
    }
  }

  public void characters(char ch[], int start, int length) {
    switch (state) {
      case STATE_INSIDE_TOP:
        topBuilder.characters(ch, start, length);
        break;
      case STATE_INSIDE_DEFINE:
        elementDefinitionBuilder.characters(ch, start, length);
        break;
    }
  }
}
