/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.document.util;

import java.util.Iterator;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.manage.AttributesContentAccess;
import org.xmloperator.framework.document.id.IdManager;
import org.xmloperator.util.dom.DomRefIds;

public final class FragmentModifier {
  private AttributesContentAccess attributesContentAccess;
  private IdManager idManager;
  private Node modifiedFragment;
  private int newIdCount = 0;
  private int refIdClearedCount = 0;

  private FragmentModifier(AttributesContentAccess attributesContentAccess,
    IdManager idManager, Node modifiedFragment) {
    if (attributesContentAccess == null || idManager == null
      || modifiedFragment == null)
      throw new IllegalArgumentException();
    this.attributesContentAccess = attributesContentAccess;
    this.idManager = idManager;
    this.modifiedFragment = modifiedFragment;
  }

  // fragment peut tre un DocumentFragment ou un Element
  // Retourne le nombre d'ids effectivement changs
  public static final int changeIdWhenNecessary(
    AttributesContentAccess attributesContentAccess, IdManager idManager,
    Node fragment) {
    FragmentModifier fragmentModifier = new FragmentModifier(
      attributesContentAccess, idManager, fragment);
    fragmentModifier.changeIdWhenNecessary(fragment);
    return fragmentModifier.newIdCount;
  }

  private void changeIdWhenNecessary(Node fragment) {
    if (fragment.getNodeType() == Node.ELEMENT_NODE) {
      AttributesContent attributesContent =
        attributesContentAccess.getAttributesContent((Element)fragment);
      String idAttrName = attributesContent != null ?
        attributesContent.getIdAttributeName() : null;
      if (idAttrName != null) {
        Attr attr = ((Element)fragment).getAttributeNode(idAttrName);
        if (attr != null) {
          String oldId = attr.getValue();
          if (oldId.length() == 0 ? attributesContent.isRequired(idAttrName) :
            idManager.isIdDefined(oldId)) {
            String newId = idManager.getNewVerifiedId();
            attr.setValue(newId);
            if (oldId.length() != 0)
              changeRefIdWithinFragment(modifiedFragment, oldId, newId);
            newIdCount++;
          }
        }
      }
    }
    {
      Node childNode = fragment.getFirstChild();
      while (childNode != null) {
        changeIdWhenNecessary(childNode);
        childNode = childNode.getNextSibling();
      }
    }
  }

  private void changeRefIdWithinFragment(Node fragment,
    String oldId, String newId) {
    if (fragment.getNodeType() == Node.ELEMENT_NODE) {
      AttributesContent attributesContent =
        attributesContentAccess.getAttributesContent((Element)fragment);
      int attrCount = attributesContent != null ?
        attributesContent.getPossibleNameCount() : 0;
      for (int k = 0; k < attrCount; k++) {
        String attrName = attributesContent.getPossibleName(k);
        Attr attr = ((Element)fragment).getAttributeNode(attrName);
        if (attr == null)
          continue;
        String attrValue = attr.getValue();
        String attrType = attributesContent.getType(attrName);
        if (attrType.equals(AttributesContent.TYPE_IDREF)) {
          if (attrValue.equals(oldId))
            attr.setValue(newId);
        }
        else if (attrType.equals(AttributesContent.TYPE_IDREFS)) {
          DomRefIds refIds = new DomRefIds(attrValue);
          if (refIds.replace(oldId, newId))
            attr.setValue(refIds.toString());
        }
      }
    }
    {
      Node childNode = fragment.getFirstChild();
      while (childNode != null) {
        changeRefIdWithinFragment(childNode, oldId, newId);
        childNode = childNode.getNextSibling();
      }
    }
  }
}
