/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.control.operators;

import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.control.operators.LeafElement;
  // Reciprocal import
import org.xmloperator.framework.control.operators.TreeElement;
  // Reciprocal import
import org.xmloperator.framework.control.operators.Document;
  // Reciprocal import

/**
 * This is like the DOM Element but with these severe restrictions :
 * - Comments, processing instructions and CDATA sections are ignored
 *     or removed.
 * - This can have a child text but no child element.
 * - This can have child elements but no child text.
 */
public interface Element {
  public abstract Document getDocument(); // not null
  public abstract String getName(); // not null
  public abstract String getAttribute(String attributeName); // not null
  public abstract void setAttribute(String attributeName, String value)
    throws UndefinedNamespaceException, IdException, OperationException;
  public abstract boolean isEqualTo(Element element);
    // return true if name, attributes and childs are equal
  public abstract boolean hasSameChildsThan(Element element);
    // return true if childs are equals
  public abstract TreeElement getParentElement();
  public abstract Element getPreviousSiblingElement(); // null if first element
  public abstract Element getPreviousSiblingElement(String elementName);
    // null if first element with that name
  public abstract Element getPreviousSiblingElement(String[] elementNames);
    // null if first element with one of these names
  public abstract Element getNextSiblingElement(); // null if last element
  public abstract Element getNextSiblingElement(String elementName);
    // null if last element with that name
  public abstract Element getNextSiblingElement(String[] elementNames);
    // null if last element with one of these names
  public abstract void insertNewPreviousSiblingElement(Element newElement)
    throws UndefinedNamespaceException, IdException, OperationException;
    // pre-condition : newElement must not have a parent element
  public abstract void insertNewNextSiblingElement(Element newElement)
    throws UndefinedNamespaceException, IdException, OperationException;
    // pre-condition : newElement must not have a parent element
  public abstract void remove()
    throws UndefinedNamespaceException, IdException, OperationException;
    // without effect if this has no parent element
    // post-condition : will not have a parent element
  public abstract void replaceBy(Element newElement)
    throws UndefinedNamespaceException, IdException, OperationException;
    // pre-condition : this must have a parent
    // pre-condition : newElement must not have a parent element
  public abstract LeafElement leafElement();
    // pre-condition : this must not be a TreeElement
  public abstract TreeElement treeElement();
    // pre-condition : this must not be a LeafElement
}
