/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.compare;

import org.w3c.dom.Element;

import org.xmloperator.framework.document.compare.ElementCharacterizationAccess;

/**
 * An element association associates an element of a first document with
 *   an element of a second document.
 * A required condition for two elements to be associated is to have
 *   the same local name and namespace URI.
 */
public interface ElementAssociation {

  /**
   * Return the common element namespace URI.
   * May be null.
   */
  public String getNamespaceURI();

  /**
   * Return the common element local name.
   * Is not null neither empty.
   */
  public String getLocalName();

  /**
   * Return the first element, which belongs to the first document.
   * Is not null.
   */
  public Element getElement1();

  /**
   * Return the second element, which belongs to the second document.
   * Is not null.
   */
  public Element getElement2();

  /**
   * Return the element associated to <code>element</code>
   *   by this element association.
   * Return null if <code>element</code> is neither the first nor the second
   *   element of this association.
   */
  public Element getAssociatedElement(Element element);

  /**
   * Two elements are said equal if and only if :
   * - they have the same attributes, name and value,
   * - they have the same child text nodes concatenation value,
   * - their element children are considered as equal upon the same criterion.
   * The comparison result doesn't depend on element's order.
   */
  public boolean areElementsEqual(
      ElementCharacterizationAccess elementCharacterizationAccess);

  /**
   * Return the count of descendant element associations, including itself.
   * Equals to 1 if this has no descendant element association.
   */
  public int getDescendantCount();

  /**
   * Add <code>addedCount</code> to the descendant count of this association.
   */
  public void addDescendantCount(int addedCount);
}