/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.manage;

import org.w3c.dom.Element;

import org.xmloperator.framework.document.schema.ElementContent;
import org.xmloperator.framework.document.schema.AttributesContent;
import org.xmloperator.framework.document.schema.
  NotConformantToContentModelException;
import org.xmloperator.framework.document.manage.AttributesContentAccess;

/**
 * Mapping from document to schema.
 */
public interface ElementDefinitionMapper extends AttributesContentAccess {

  /**
   * case DTD : return the DTD.
   * case RNG : return the Schema.
   */
  public abstract Object getSchema();

  /**
   * case DTD : return the element name.
   * case RNG : return the ElementDefinition associated to element.
   * Return null if no element definition can be associated to element.
   */
  public abstract Object getElementDefinition(Element element);

  /**
   * Return the ElementContent of parentElement.
   * Return null if no usable element definition (with non empty pattern)
   *   can be associated to parentElement.
   * Raises a NotConformantToContentModelException if the children content
   *   of parentElement is not conformant to the content model.
   */
  public abstract ElementContent getElementContent(Element parentElement)
    throws NotConformantToContentModelException;

  /**
   * Return true if the element content of parentElement is valid
   *   i.e. is conformant to its content model.
   * Return false if parentElement is not allowed.
   * Return true if parentElement cannot be associated to an ElementDefinition.
   */
  public abstract boolean isElementContentValid(Element parentElement);

  public abstract void removeElementFromCache(Element element);
}
