/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.framework.document.schema;

/**
 * Maps the attribute set of an element to a content model
 *   in order to support attributes editing.
 * This release doesn't consider any constraint.
 */
public interface AttributesContent {
  public static final String TYPE_CDATA = "CDATA";
  public static final String TYPE_ID = "ID";
  public static final String TYPE_IDREF = "IDREF";
  public static final String TYPE_IDREFS = "IDREFS";
  public static final String TYPE_ENUMERATION = "ENUMERATION";

  /**
   * Return the count of all possible names which can occur in the element
   *   disregarding incompatibilities which may exist between them.
   * An NsName counts for one.
   * An AnyName counts for one.
   */
  public abstract int getPossibleNameCount();

  /**
   * Pre-condition: 0 <= index < possibleNameCount.
   * Return the possible name at index position.
   * The names are ordered : first by local name, secondly by ns name.
   * The local name is empty for an NsName.
   * Return ":" for an AnyName.
   * Return null if no prefix maps to the attribute namespace.
   */
  public abstract String getPossibleName(int index);

  /**
   * Return the attribute type (TYPE_*).
   */
  public abstract String getType(String attrName);

  /**
   * Return the attribute value enumeration or null if not an enumeration.
   */
  public abstract String[] getEnumeration(String attrName);

  /**
   * Return true if the attribute is required.
   */
  public abstract boolean isRequired(String attrName);

  /**
   * Return the ID attribute name if it exists, elsewhere null.
   */
  public abstract String getIdAttributeName();
}
