/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.operation;

import java.io.IOException;
import java.io.File;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.awt.Point;
import java.util.StringTokenizer;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.dom.NodePathBuilder;
import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.util.dom.Cleaner;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.framework.document.id.IdException;
import org.xmloperator.framework.document.manage.AttributeManager;
import org.xmloperator.framework.document.manage.NodeManager;
/*
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.instance.ToolViewAccess;
*/
import org.xmloperator.framework.operation.OperationException;
import org.xmloperator.framework.operation.Operation;
import org.xmloperator.operation.core.MetaDocument;
import org.xmloperator.operation.core.NullOperation;
import org.xmloperator.operation.core.TransitionOperation;
import org.xmloperator.operation.core.NodesInsertOperation;
import org.xmloperator.operation.core.NodesRemoveOperation;
import org.xmloperator.operation.core.NodesReplaceOperation;
import org.xmloperator.operation.core.NodeIntervalMoveOperation;
import org.xmloperator.operation.core.NodeIntervalReorderOperation;
import org.xmloperator.operation.core.NodeIntervalPackOperation;
import org.xmloperator.operation.core.ElementUnpackOperation;
import org.xmloperator.operation.core.ElementNameReplaceOperation;
import org.xmloperator.operation.core.TextSplitOperation;
import org.xmloperator.operation.core.NextTextJoinOperation;
import org.xmloperator.operation.core.NodeSetValueOperation;
import org.xmloperator.operation.core.AttrSetValueOperation;
import org.xmloperator.operation.DialogOperation;

public final class OperationLoader {
  private static final String OPERATION_ELEMENT_EXPECTED =
    "<operation> expected";
  private static final String OPERATION_EXPECTED =
    "Operation expected";
  private static final String ILLEGAL_ELEMENT_PATH =
    "Illegal element path";
  private static final String ILLEGAL_NODE_INDEX = "Illegal node index";
  private static final String NOT_A_NODE_INTERVAL = "Not a node interval";
  private static final String NOT_A_VALUABLE_NODE = "Not a valuable node";
  private static final String NODE_ELEMENT_EXPECTED = "<node> expected";
  private static final String ATTRIBUTE_NAME_EXPECTED =
    "Attribute name expected";
  private static final String TEXT_NODE_EXPECTED = "Text node expected";
  private static final String ELEMENT_NODE_EXPECTED = "Element node expected";
  private static final String OFFSET_EXPECTED = "Offset expected";
  private final Document document;
  private final NodeManager nodeManager;
  private final AttributeManager attributeManager;
  private Document operationDocument = null;

  public static final void loadAndExecuteOperations(
    Document document,
    NodeManager nodeManager, AttributeManager attributeManager,
    String pathName, OperationStack operationStack) {
    OperationLoader operationLoader = new OperationLoader(
      document, nodeManager, attributeManager);
    operationLoader.loadCompleteOrNot(pathName);
    operationLoader.executeOperations(operationStack);
  }

  private OperationLoader(
      Document document,
      NodeManager nodeManager, AttributeManager attributeManager) {
    if (document == null || nodeManager == null || attributeManager == null)
      throw new IllegalArgumentException();
    this.document = document;
    this.nodeManager = nodeManager;
    this.attributeManager = attributeManager;
  }

  private void loadCompleteOrNot(String pathName) {
    operationDocument = null;
    if (!new File(pathName).exists())
      return;
    String systemId = FileUtils.expandedSystemId(pathName);
    loadComplete(systemId);
    if (operationDocument != null)
      return;
    try {
      PrintWriter printWriter = new PrintWriter(
        new OutputStreamWriter(new FileOutputStream(pathName, true), "UTF8"));
      printWriter.println("</operations>");
      printWriter.close();
    }
    catch (IOException exception) {
    }
    loadComplete(systemId);
  }

  private void loadComplete(String systemId) {
    DOMParser parser = new DOMParser();
    try {
      parser.parse(systemId);
      operationDocument = parser.getDocument();
    }
    catch (IOException exception) {
    }
    catch (SAXException exception) {
    }
  }

  private void executeOperations(OperationStack operationStack) {
    if (operationDocument == null)
      return;
    Element operationsElement = operationDocument.getDocumentElement();
    int operationCount = ElementBrowser.getChildElementCount(operationsElement);
    if (operationCount == 0)
      return;
    Cleaner.removeEmptyTexts(operationsElement);
    Element operationElement =
      ElementBrowser.getFirstChildElement(operationsElement);
    int operationNumber = 0;
    Point centrageDialogue = new Point();
    boolean confirmationAChaqueOperation = true;
    while (operationElement != null) {
      operationNumber++;
      try {
        Operation operation =
          createOperationFromOperationElement(operationElement);
        if (confirmationAChaqueOperation)
          switch (DialogOperation.showDialog(null,
            "Operation " + operationNumber + " / " + operationCount,
            ElementBrowser.getFirstChildElement(operationElement),
            centrageDialogue)) {
            case DialogOperation.TOUT_EXECUTER:
              confirmationAChaqueOperation = false;
              break;
            case DialogOperation.EXECUTER:
              break;
            case DialogOperation.REJETER:
              operation = null;
              break;
            case DialogOperation.TOUT_REJETER:
            default:
              return;
          }
        if (operation != null && !operation.isNull())
          operationStack.executeAndAdd(operation);
      }
      catch (Exception exception) {
        if (DialogOperation.showDialog(null,
          "Operation " + operationNumber + " / " + operationCount + " rejected",
          ElementBrowser.getFirstChildElement(operationElement),
            exception.toString(),
          centrageDialogue) == DialogOperation.TOUT_REJETER)
          return;
        confirmationAChaqueOperation = true;
      }
      operationElement = ElementBrowser.getNextSiblingElement(operationElement);
    }
  }

  private Operation createOperationFromOperationElement(
    Element operationElement) {
    if (!operationElement.getNodeName().equals("operation")) {
      throw new OperationException(OPERATION_ELEMENT_EXPECTED);
    }
    Element internalOperationElement =
      ElementBrowser.getFirstChildElement(operationElement);
    if (internalOperationElement == null)
      throw new OperationException(OPERATION_EXPECTED);
    return createOperationFromInternalOperationElement(
      internalOperationElement);
  }

  private Operation createOperationFromInternalOperationElement(
    Element operationElement) throws OperationException {
    String operationElementName = operationElement.getNodeName();
    if (operationElementName.equals("nullOperation"))
      return new NullOperation();
    if (operationElementName.equals("transitionOperation"))
      return new TransitionOperation();
    else if (operationElementName.equals("nodesInsertOperation")) {
      Element element =
        getElementFromElementPath(operationElement, "parentElementPath");
      Node parentNode = element != null ? (Node)element : document;
      int nodeIndex = getNodeIndex(operationElement);
      Node nextSibling = nodeIndex >= 0 ?
        getNodeFromIndex(element, nodeIndex) : null;
      Node[] newNodes = MetaDocument.createNodes(document, operationElement);
      return new NodesInsertOperation(
          this.nodeManager, parentNode, nextSibling, newNodes);
    }
    else if (operationElementName.equals("nodeIntervalRemoveOperation")) {
      Element element = getElementFromElementPath(operationElement);
      int firstNodeIndex = getNodeIndex(operationElement, "firstNodeIndex");
      int lastNodeIndex = getNodeIndex(operationElement, "lastNodeIndex");
      if (firstNodeIndex < 0 || lastNodeIndex < 0
        || firstNodeIndex > lastNodeIndex)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node[] nodesToRemove =
        getNodesFromInterval(element, firstNodeIndex, lastNodeIndex);
      return new NodesRemoveOperation(this.nodeManager, nodesToRemove);
    }
    else if (operationElementName.equals("nodeIntervalReplaceOperation")) {
      Element element = getElementFromElementPath(operationElement);
      int firstNodeIndex = getNodeIndex(operationElement, "firstNodeIndex");
      int lastNodeIndex = getNodeIndex(operationElement, "lastNodeIndex");
      if (firstNodeIndex < 0 || lastNodeIndex < 0
        || firstNodeIndex > lastNodeIndex)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node[] oldNodes =
        getNodesFromInterval(element, firstNodeIndex, lastNodeIndex);
      Node[] newNodes = MetaDocument.createNodes(document, operationElement);
      return new NodesReplaceOperation(this.nodeManager, oldNodes, newNodes);
    }
    else if (operationElementName.equals("nodeIntervalMoveOperation")) {
      Element sourceParentElement =
        getElementFromElementPath(operationElement, "sourceParentElementPath");
      int firstSourceNodeIndex =
        getNodeIndex(operationElement, "firstSourceNodeIndex");
      int lastSourceNodeIndex =
        getNodeIndex(operationElement, "lastSourceNodeIndex");
      if (firstSourceNodeIndex < 0 || lastSourceNodeIndex < 0
        || firstSourceNodeIndex > lastSourceNodeIndex)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node firstSourceNode =
        getNodeFromIndex(sourceParentElement, firstSourceNodeIndex);
      Node lastSourceNode =
        getNodeFromIndex(sourceParentElement, lastSourceNodeIndex);
      Element destinationElement =
        getElementFromElementPath(operationElement, "destinationElementPath");
      Node destinationParentNode = destinationElement != null ?
        (Node)destinationElement : document;
      int destinationNodeIndex =
        getNodeIndex(operationElement, "destinationNodeIndex");
      Node destinationNextSibling = destinationNodeIndex >= 0 ?
        getNodeFromIndex(destinationElement, destinationNodeIndex) : null;
      return new NodeIntervalMoveOperation(
          this.nodeManager, firstSourceNode, lastSourceNode,
          destinationParentNode, destinationNextSibling);
    }
    else if (operationElementName.equals("nodeIntervalReorderOperation")) {
      Element parentElement =
        getElementFromElementPath(operationElement, "parentElementPath");
      String nodeIndexes = operationElement.getAttribute("nodeIndexes");
      Node[] nodesInDestinationOrder = new Node[tokensCount(nodeIndexes)];
      StringTokenizer nodeIndexesTokenizer = new StringTokenizer(nodeIndexes);
      int firstIndex = Integer.MAX_VALUE;
      int lastIndex = 0;
      int k = 0;
      while (nodeIndexesTokenizer.hasMoreElements()) {
        int nodeIndex =
          decode(nodeIndexesTokenizer.nextToken(), ILLEGAL_NODE_INDEX);
        nodesInDestinationOrder[k++] =
          getNodeFromIndex(parentElement, nodeIndex);
        if (nodeIndex < firstIndex)
          firstIndex = nodeIndex;
        if (nodeIndex > lastIndex)
          lastIndex = nodeIndex;
      }
      if (lastIndex - firstIndex + 1 != nodesInDestinationOrder.length)
        throw new OperationException(NOT_A_NODE_INTERVAL);
      return new NodeIntervalReorderOperation(
          this.nodeManager, nodesInDestinationOrder);
    }
    else if (operationElementName.equals("nodeIntervalPackOperation")) {
      Element sourceParentElement =
        getElementFromElementPath(operationElement, "sourceParentElementPath");
      int firstSourceNodeIndex =
        getNodeIndex(operationElement, "firstSourceNodeIndex");
      int lastSourceNodeIndex =
        getNodeIndex(operationElement, "lastSourceNodeIndex");
      if (firstSourceNodeIndex < 0 || lastSourceNodeIndex < 0
        || firstSourceNodeIndex > lastSourceNodeIndex)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node firstSourceNode =
        getNodeFromIndex(sourceParentElement, firstSourceNodeIndex);
      Node lastSourceNode =
        getNodeFromIndex(sourceParentElement, lastSourceNodeIndex);
      Element newElement = createElement(operationElement);
      return new NodeIntervalPackOperation(
          this.nodeManager, firstSourceNode, lastSourceNode, newElement);
    }
    else if (operationElementName.equals("elementUnpackOperation")) {
      Element element = getElementFromElementPath(operationElement);
      return new ElementUnpackOperation(this.nodeManager, element);
    }
    else if (operationElementName.equals("nodeReplaceOperation")) {
      Element element = getElementFromElementPath(operationElement);
      int nodeIndex = getNodeIndex(operationElement);
      Node oldNode = nodeIndex >= 0 ?
        getNodeFromIndex(element, nodeIndex) : element;
      Node newNode = createNode(operationElement);
      return new NodesReplaceOperation(this.nodeManager, oldNode, newNode);
    }
    else if (operationElementName.equals("elementNameReplaceOperation")) {
      Element oldElement = getElementFromElementPath(operationElement);
      Element newElement = createElement(operationElement);
      return new ElementNameReplaceOperation(
          this.nodeManager, oldElement, newElement);
    }
    else if (operationElementName.equals("textSplitOperation")) {
      Element element = getElementFromElementPath(operationElement);
      if (element == null)
        throw new OperationException(ILLEGAL_ELEMENT_PATH);
      int nodeIndex = getNodeIndex(operationElement);
      if (nodeIndex < 0)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node textToSplit = getNodeFromIndex(element, nodeIndex);
      if (textToSplit.getNodeType() != Node.TEXT_NODE)
        throw new OperationException(TEXT_NODE_EXPECTED);
      int offset =
        decode(operationElement.getAttribute("offset"), OFFSET_EXPECTED);
      return new TextSplitOperation(
          this.nodeManager, (Text)textToSplit, offset);
    }
    else if (operationElementName.equals("nextTextJoinOperation")) {
      Element element = getElementFromElementPath(operationElement);
      if (element == null)
        throw new OperationException(ILLEGAL_ELEMENT_PATH);
      int nodeIndex = getNodeIndex(operationElement);
      if (nodeIndex < 0)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node textToJoinToNextOne = getNodeFromIndex(element, nodeIndex);
      if (textToJoinToNextOne.getNodeType() != Node.TEXT_NODE)
        throw new OperationException(TEXT_NODE_EXPECTED);
      return new NextTextJoinOperation(
          this.nodeManager, (Text)textToJoinToNextOne);
    }
    else if (operationElementName.equals("nodeSetValueOperation")) {
      Element element = getElementFromElementPath(operationElement);
      int nodeIndex = getNodeIndex(operationElement);
      if (nodeIndex < 0)
        throw new OperationException(ILLEGAL_NODE_INDEX);
      Node node = getNodeFromIndex(element, nodeIndex);
      if (node.getNodeValue() == null)
        throw new OperationException(NOT_A_VALUABLE_NODE);
      Node firstOperationElementChild = operationElement.getFirstChild();
      String newValue = firstOperationElementChild != null ?
        firstOperationElementChild.getNodeValue() : "";
      return new NodeSetValueOperation(this.nodeManager, node, newValue);
    }
    else if (operationElementName.equals("attrSetValueOperation")) {
      Element element = getElementFromElementPath(operationElement);
      if (element == null)
        throw new OperationException(ILLEGAL_ELEMENT_PATH);
      String attrName = operationElement.getAttribute("attrName");
      if (attrName.length() == 0)
        throw new OperationException(ATTRIBUTE_NAME_EXPECTED);
      String newValue = null;
      boolean isNull = operationElement.getAttribute("isNull").equals("true");
      if (!isNull) {
        Node firstOperationElementChild = operationElement.getFirstChild();
        newValue = firstOperationElementChild != null ?
          firstOperationElementChild.getNodeValue() : "";
      }
      return new AttrSetValueOperation(this.attributeManager,
        element, attrName, newValue);
    }
    else {
      //System.out.println("operationElementName=" + operationElementName);
      throw new OperationException(OPERATION_EXPECTED);
    }
  }

  private Element getElementFromElementPath(Element operationElement) {
    return getElementFromElementPath(operationElement, "elementPath");
  }

  private Element getElementFromElementPath(Element operationElement,
    String elementPathAttrName) {
    String elementPath = operationElement.getAttribute(elementPathAttrName);
    StringTokenizer elementPathTokenizer = new StringTokenizer(elementPath);
    if (!elementPathTokenizer.hasMoreElements())
      return null;
    Element element = NodePathBuilder.getElementFromElementPathWithNames(
      document, elementPath);
    if (element == null)
        throw new OperationException(ILLEGAL_ELEMENT_PATH);
    return element;
  }

  private int getNodeIndex(Element operationElement) {
    return getNodeIndex(operationElement, "nodeIndex");
  }

  private int getNodeIndex(Element operationElement, String nodeIndexAttrName) {
    String nodeIndexString = operationElement.getAttribute(nodeIndexAttrName);
    return nodeIndexString.length() != 0 ?
      decode(nodeIndexString, ILLEGAL_NODE_INDEX) : -1;
  }

  private static final int decode(String stringValue, String exceptionMessage) {
    if (stringValue.length() == 0)
      throw new OperationException(exceptionMessage);
    int intValue;
    try {
      intValue = Integer.decode(stringValue).intValue();
    }
    catch (NumberFormatException exception) {
      throw new OperationException(exceptionMessage);
    }
    if (intValue < 0)
      throw new OperationException(exceptionMessage);
    return intValue;
  }

  private Node getNodeFromIndex(Node parentNode, int nodeIndex) {
    if (parentNode == null)
      parentNode = document;
    Node childNode =
      NodePathBuilder.getChildNodeFromIndex(parentNode, nodeIndex);
    if (childNode == null)
      throw new OperationException(ILLEGAL_NODE_INDEX);
    return childNode;
  }

  private Node[] getNodesFromInterval(Node parentNode,
    int firstNodeIndex, int lastNodeIndex) {
    if (parentNode == null)
      parentNode = document;
    NodeList childNodes = parentNode.getChildNodes();
    int childCount = childNodes.getLength();
    if (firstNodeIndex < 0 || firstNodeIndex > lastNodeIndex
      || lastNodeIndex >= childCount)
      throw new OperationException(ILLEGAL_NODE_INDEX);
    int intervalLength = lastNodeIndex - firstNodeIndex + 1;
    Node[] nodes = new Node[intervalLength];
    for (int k = 0; k < intervalLength; k++)
      nodes[k] = childNodes.item(firstNodeIndex + k);
    return nodes;
  }

  private Node createNode(Element operationElement) {
    Node[] newNodes = MetaDocument.createNodes(document, operationElement);
    if (newNodes.length == 0)
      throw new OperationException(NODE_ELEMENT_EXPECTED);
    return newNodes[0];
  }

  private Element createElement(Element operationElement) {
    Node newElement = createNode(operationElement);
    if (newElement.getNodeType() != Node.ELEMENT_NODE)
      throw new OperationException(ELEMENT_NODE_EXPECTED);
    return (Element)newElement;
  }

  private static final int tokensCount(String tokens) {
    StringTokenizer tokenizer = new StringTokenizer(tokens);
    int count = 0;
    while (tokenizer.hasMoreTokens()) {
      count++;
      tokenizer.nextToken();
    }
    return count;
  }
}
