/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option;

import java.util.List;
import java.util.Vector;
import java.util.Hashtable;
import java.io.IOException;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.OutputStreamWriter;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.ErrorHandler;
import org.xml.sax.XMLReader;
import org.apache.xerces.parsers.SAXParser;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.part.OptionPart;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.dom.DomWriter;
import org.xmloperator.option.OptionsImpl;

public final class OptionsHandler implements ContentHandler {

  public static final void save(Options options, String fileName) {
    try {
      OutputStream outputStream = new FileOutputStream(fileName);
      PrintWriter out =
        new PrintWriter(new OutputStreamWriter(outputStream, "UTF8"));
      out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      printStartTag(out, Constants.TOOL);
      List lastOptionPackage = new Vector();
      for (int optionIndex = 0; optionIndex < options.getOptionDefCount();
        optionIndex++) {
        List optionList = options.getOptionList(optionIndex);
        if (optionList.isEmpty())
          continue;
        List optionPackage = OptionsImpl.getOptionPackage(optionIndex);
        int commonElementCount = 0;
        while (commonElementCount < lastOptionPackage.size()
          && commonElementCount < optionPackage.size()
          && lastOptionPackage.get(commonElementCount).equals(
          optionPackage.get(commonElementCount)))
          commonElementCount++;
        for (int elementIndex = lastOptionPackage.size() - 1;
          elementIndex >= commonElementCount; elementIndex--)
          printEndTag(out, (String)lastOptionPackage.get(elementIndex));
        for (int elementIndex = commonElementCount;
          elementIndex < optionPackage.size(); elementIndex++)
          printStartTag(out, (String)optionPackage.get(elementIndex));
        for (int valueIndex = 0; valueIndex < optionList.size(); valueIndex++)
          printValueTag(out, optionIndex, (Option)optionList.get(valueIndex));
        lastOptionPackage = optionPackage;
      }
      for (int elementIndex = lastOptionPackage.size() - 1;
        elementIndex >= 0; elementIndex--)
        printEndTag(out, (String)lastOptionPackage.get(elementIndex));
      printEndTag(out, Constants.TOOL);
      out.close();
    }
    catch (IOException exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private static final void printStartTag(PrintWriter out, String elementName) {
    out.print('<');
    out.print(elementName);
    out.print('>');
  }

  private static final void printEndTag(PrintWriter out, String elementName) {
    out.print("</");
    out.print(elementName);
    out.print('>');
  }

  private static final void printValueTag(
      PrintWriter out, int optionIndex, Option option) {
    out.print('<');
    out.print(OptionsImpl.getOptionId(optionIndex));
    for (int attIndex = 0; attIndex < option.getOptionPartCount(); attIndex++)
      DomWriter.printAttribute(
          out, option.getOptionPartName(attIndex),
          option.getOptionPart(attIndex).getValue());
    out.print("/>");
  }

  public static final boolean load(Options options, String systemId) {
    try {
      options.clear();
      XMLReader xmlReader = new SAXParser();
      xmlReader.setContentHandler(new OptionsHandler(options));
      xmlReader.setErrorHandler(
        new ErrorHandler() {
          public void warning(SAXParseException exception) throws SAXException {
          }
          public void error(SAXParseException exception) throws SAXException {
          }
          public void fatalError(SAXParseException exception)
            throws SAXException {
            throw exception;
          }
        });
      xmlReader.parse(systemId);
      return true;
    }
    catch (Exception exception) {
      options.clear();
      return false;
    }
  }

  private Options options;

  public OptionsHandler(Options options) {
    this.options = options;
  }

  ////////////////////////////////
  // ContentHandler implementation
  ////////////////////////////////

  public void setDocumentLocator(Locator locator) {
  }

  public void startDocument() throws SAXException {
  }

  public void endDocument() throws SAXException {
  }

  public void startPrefixMapping(String prefix, String uri)
      throws SAXException {
  }

  public void endPrefixMapping(String prefix) throws SAXException {
  }

  public void startElement(
      String namespaceURI, String localName, String rawName, Attributes atts)
      throws SAXException {
    String optionId = localName;
    int optionIndex = OptionsImpl.getOptionIndex(optionId);
    if (optionIndex < 0)
      return;
    Option option = OptionsImpl.getOptionType(optionIndex).createOption();
    for (int k = 0; k < atts.getLength(); k++) {
      String attId = atts.getLocalName(k);
      OptionPart optionPart = option.getOptionPart(attId);
      if (optionPart != null)
        optionPart.setValue(atts.getValue(k));
    }
    options.getOptionList(optionId).add(option);
  }

  public void endElement (String namespaceURI, String localName,
   String rawName)	throws SAXException {
  }

  public void characters (char ch[], int start, int length)
    throws SAXException {
  }

  public void ignorableWhitespace (char ch[], int start, int length)
    throws SAXException {
  }

  public void processingInstruction (String target, String data)
    throws SAXException {
  }

  public void skippedEntity (String name)	throws SAXException {
  }
}
