/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;

import org.xmloperator.framework.document.namespace.NamespacePrefixPair;
import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public class NamespacePrefixPairImpl extends OptionImpl
    implements NamespacePrefixPair {
  private static final int NAMESPACE = 0;
  private static final int PREFIX = 1;
  public static final OptionType optionType =
      new OptionTypeImpl(
      new String[] {"namespace", "prefix"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType("")})
        {
          public Option createOption() {
            return new NamespacePrefixPairImpl();
          }
        };

  public static final void initializeListIfEmpty(List list) {
    if (!list.isEmpty())
      return;
    list.add(new NamespacePrefixPairImpl(
        "http://www.w3.org/1998/Math/MathML", "math"));
    list.add(new NamespacePrefixPairImpl(
        "http://www.w3.org/1999/xlink", "xlink"));
    list.add(new NamespacePrefixPairImpl(
        "http://www.w3.org/1999/xhtml", "html"));
    list.add(new NamespacePrefixPairImpl(
        "http://www.w3.org/1999/XSL/Transform", "xsl"));
    list.add(new NamespacePrefixPairImpl(
        "http://www.w3.org/2000/svg", "svg"));
    list.add(new NamespacePrefixPairImpl(
        "http://relaxng.org/ns/structure/1.0", "rng"));
    list.add(new NamespacePrefixPairImpl(
        "http://relaxng.org/ns/compatibility/annotations/1.0", "a"));
    list.add(new NamespacePrefixPairImpl(
        "http://www.rdf.net/rpv/", "rpv"));
  }

  private NamespacePrefixPairImpl() {
    super(optionType);
  }

  public NamespacePrefixPairImpl(String namespaceURI, String prefix) {
    this();
    setNamespaceURI(namespaceURI);
    setPrefix(prefix);
  }

  private void setNamespaceURI(String namespaceURI) {
    if (namespaceURI == null || namespaceURI.length() == 0)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(NAMESPACE)).setStringValue(namespaceURI);
  }

  private void setPrefix(String prefix) {
    if (prefix == null || prefix.length() == 0)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(PREFIX)).setStringValue(prefix);
  }

  /////////////////////
  // Object overloading
  /////////////////////

  public boolean equals(Object object) {
    return object instanceof NamespacePrefixPair
    && ((NamespacePrefixPair)object).getNamespaceURI().equals(
    this.getNamespaceURI());
  }

  public String toString() {
    return "xmlns:" + getPrefix() + "=\"" + getNamespaceURI() + "\"";
  }

  /////////////////////////////////////
  // NamespacePrefixPair Implementation
  /////////////////////////////////////

  public String getNamespaceURI() {
    return ((StringOptionPartImpl)getOptionPart(NAMESPACE)).getStringValue();
  }

  public String getPrefix() {
    return ((StringOptionPartImpl)getOptionPart(PREFIX)).getStringValue();
  }
}
