/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.option.impl;

import java.util.List;
import java.util.Vector;

import org.xmloperator.framework.option.part.OptionPartType;
import org.xmloperator.framework.option.part.BooleanOptionPart;
import org.xmloperator.framework.option.part.StringOptionPart;
import org.xmloperator.framework.option.OptionType;
import org.xmloperator.framework.option.Option;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.control.operators.OperatorInfo;
import org.xmloperator.option.part.BooleanOptionPartType;
import org.xmloperator.option.part.BooleanOptionPartImpl;
import org.xmloperator.option.part.StringOptionPartType;
import org.xmloperator.option.part.StringOptionPartImpl;
import org.xmloperator.option.core.OptionTypeImpl;
import org.xmloperator.option.core.OptionImpl;

public final class OperatorInfoImpl extends OptionImpl
  implements OperatorInfo, Comparable {
  private static final int NAME = 0;
  private static final int TYPENAME = 1;
  private static final int CLASSNAME = 2;
  public static final OptionType optionType =
    new OptionTypeImpl(
      new String[] {
        "name",
        "typeName",
        "className"},
      new OptionPartType[] {
        new StringOptionPartType(""),
        new StringOptionPartType(""),
        new StringOptionPartType("")}) {
      public Option createOption() {
        return new OperatorInfoImpl();
      }
    };

  public static final void initializeListIfEmpty(List list) {
    if (!list.isEmpty())
      return;
    list.add(new OperatorInfoImpl("ConfigurationStep", "change",
      "org.xmloperator.operators.change.ChangeOperator"));
    list.add(new OperatorInfoImpl("MakeUseOfPattern", "dtd",
      "org.xmloperator.operators.dtd.DtdFactorizingOperator"));
  }

  public static final List createOperatorInfoSubList(
    List operatorInfoList, String documentTypeName) {
    List subList = new Vector();
    for (int k = 0; k < operatorInfoList.size(); k++) {
      OperatorInfo operatorInfo = (OperatorInfo)operatorInfoList.get(k);
      if (documentTypeName.equals(operatorInfo.getDocumentTypeName()))
        subList.add(operatorInfo);
    }
    return subList;
  }

  public static final OperatorInfo getOperatorInfo(
    List operatorInfoList, String operatorName) {
    for (int k = 0; k < operatorInfoList.size(); k++) {
      OperatorInfo operatorInfo = (OperatorInfo)operatorInfoList.get(k);
      if (operatorName.equals(operatorInfo.getOperatorName()))
        return operatorInfo;
    }
    return null;
  }

  public OperatorInfoImpl() {
    super(optionType);
  }

  public OperatorInfoImpl(
    String operatorName, String documentTypeName, String operatorClassName) {
    this();
    if (operatorName == null || documentTypeName == null
      || operatorClassName == null)
      throw new IllegalArgumentException();
    setOperatorName(operatorName);
    setDocumentTypeName(documentTypeName);
    setOperatorClassName(operatorClassName);
  }

  public OperatorInfoImpl(OperatorInfo operatorInfo) {
    this(
      operatorInfo.getOperatorName(),
      operatorInfo.getDocumentTypeName(),
      operatorInfo.getOperatorClassName());
  }

  //////////////////
  // Object overload
  //////////////////

  public boolean equals(Object object) {
    return (object != null) && object instanceof OperatorInfoImpl
      && getOperatorName().equals(((OperatorInfoImpl)object).getOperatorName());
  }

  public String toString() {
    return getOperatorName();
  }

  ////////////////////////////
  // Comparable implementation
  ////////////////////////////

  public int compareTo(Object object) {
    if (object == null || !(object instanceof OperatorInfoImpl))
      throw new IllegalArgumentException();
    return getOperatorName().compareTo(
      ((OperatorInfoImpl)object).getOperatorName());
  }

  //////////////////////////////
  // OperatorInfo implementation
  //////////////////////////////

  public final boolean isValid() {
    return getOperatorName().length() != 0;
  }

  public final String getOperatorName() {
    return ((StringOptionPartImpl)getOptionPart(NAME)).getStringValue();
  }

  public final void setOperatorName(String operatorName) {
    if (operatorName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(NAME)).setStringValue(operatorName);
  }

  public String getDocumentTypeName() {
    return ((StringOptionPartImpl)getOptionPart(TYPENAME)).getStringValue();
  }

  public void setDocumentTypeName(String documentTypeName) {
    if (documentTypeName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(TYPENAME)).
      setStringValue(documentTypeName);
  }

  public final String getOperatorClassName() {
    return ((StringOptionPartImpl)getOptionPart(CLASSNAME)).getStringValue();
  }

  public final void setOperatorClassName(String operatorClassName) {
    if (operatorClassName == null)
      throw new IllegalArgumentException();
    ((StringOptionPartImpl)getOptionPart(CLASSNAME)).
      setStringValue(operatorClassName);
  }
}
