/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.transform;

import java.util.Locale;

import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.transform.AbstractTransformer;

public abstract class AbstractTransformerToHtml extends AbstractTransformer {
  protected String PARAMETER_STYLESHEET = "stylesheet";

  protected static final String headElementName(int level)
    throws TransformerException {
    if (level < 1 || level > 6)
      throw new TransformerException("Illegal head level : " + level);
    return "h" + level;
  }

  protected AbstractTransformerToHtml() {
    this.setParameter(PARAMETER_STYLESHEET, Constants.TOOL + ".css");
  }

  protected abstract void transform(
      Document sourceDocument, Document destinationDocument)
      throws TransformerException;

  public final void transform(
      final Source xmlSource, final Result outputTarget)
      throws TransformerException {
    if (!(xmlSource instanceof DOMSource)
        || !(outputTarget instanceof DOMResult))
      throw new IllegalArgumentException();
    Node sourceNode = ((DOMSource)xmlSource).getNode();
    Node outputNode = ((DOMResult)outputTarget).getNode();
    if (!(sourceNode instanceof Document) || !(outputNode instanceof Document))
      throw new IllegalArgumentException();
    transform((Document)sourceNode, (Document)outputNode);
  }

  protected Element createHtmlElement(Document document) {
    Element htmlElement = document.createElement("html");
    String lang = Locale.getDefault().getLanguage();
    htmlElement.setAttribute("lang", lang);
    htmlElement.setAttribute("xml:lang", lang);
    htmlElement.setAttribute("xmlns", "http://www.w3.org/1999/xhtml");
    return htmlElement;
  }

  protected Element createHeadElement(Document document, String cssStyle,
    String title) {
    Element headElement = document.createElement("head");
    Element titleElement = document.createElement("title");
    titleElement.appendChild(document.createTextNode(title));
    headElement.appendChild(titleElement);
    if (cssStyle != null) {
      Element styleElement = document.createElement("style");
      styleElement.setAttribute("type", "text/css");
      styleElement.appendChild(document.createTextNode(cssStyle));
      headElement.appendChild(styleElement);
    }
    else {
      Element linkElement = document.createElement("link");
      linkElement.setAttribute(
          "href",  this.getParameter(PARAMETER_STYLESHEET).toString());
      linkElement.setAttribute("rel", "stylesheet");
      linkElement.setAttribute("type", "text/css");
      headElement.appendChild(linkElement);
    }
    return headElement;
  }

  protected Element createBodyElement(Document document) {
    return document.createElement("body");
  }
}
