/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.transform;

import java.util.ResourceBundle;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.util.dom.ElementBrowser;
import org.xmloperator.framework.document.id.IdBrowser;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.transform.ToolTransformerToHtml;
import org.xmloperator.transform.PackageTransformer;

public final class SchemaToHtmlTransformer extends ToolTransformerToHtml {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);

  private static final String translateType(String type) {
    return type.equals("standard") ? "" : type;
  }

  private static final String translateMultiplicity(String multiplicity) {
    if (multiplicity.equals("zeroOrOne"))
      return "0,1";
    if (multiplicity.equals("one"))
      return "1";
    if (multiplicity.equals("zeroOrMore"))
      return "0,n";
    if (multiplicity.equals("oneOrMore"))
      return "1,n";
    return "";
  }

  public SchemaToHtmlTransformer() {
  }

  public void transform(
    Document sourceDocument, Document destinationDocument)
    throws TransformerException {
    Element schemaElement = sourceDocument.getDocumentElement();
    if (!schemaElement.getNodeName().equals("schema"))
      throw new TransformerException("<schema> expected");
    Element htmlElement = createHtmlElement(destinationDocument);
    destinationDocument.appendChild(htmlElement);
    final String productName = schemaElement.getAttribute("productName");
    final String productVersion = schemaElement.getAttribute("productVersion");
    final String objectSchema = viewResource.getString(Label.OBJECT_SCHEMA);
    final String title = productName + " - " + objectSchema;
    Element headElement = createHeadElement(destinationDocument, null, title);
    htmlElement.appendChild(headElement);
    Element bodyElement = createBodyElement(destinationDocument);
    htmlElement.appendChild(bodyElement);
    PackageTransformerImpl packageTransformer =
      new PackageTransformerImpl(bodyElement, idBrowser);
    packageTransformer.addTitle(productName + " " + productVersion, objectSchema);
    addHomeRefElement(bodyElement);
    packageTransformer.addDescription(schemaElement);
    packageTransformer.addIndex(schemaElement, "package", 2, "Package list");
    Element packageElement =
      ElementBrowser.getFirstChildElement(schemaElement, "package");
    while (packageElement != null) {
      packageTransformer.transformPackage(packageElement);
      packageElement =
        ElementBrowser.getNextSiblingElement(packageElement, "package");
    }
    addEndFragment(bodyElement, schemaElement.getAttribute("lastUpdate"));
  }

  private final class PackageTransformerImpl extends PackageTransformer {

    public PackageTransformerImpl(Element bodyElement, IdBrowser idBrowser) {
      super(bodyElement, idBrowser);
    }

    public void addDescription(Element parentElement)
      throws TransformerException {
      Element descriptionElement =
        ElementBrowser.getFirstChildElement(parentElement, "description");
      if (descriptionElement != null)
        bodyElement.appendChild(
          packElement("p", descriptionFragment(descriptionElement)));
    }

    public Node descriptionFragment(Element descriptionElement)
      throws TransformerException {
      Node fragment = document.createDocumentFragment();
      Node childNode = descriptionElement.getFirstChild();
      while (childNode != null) {
        switch (childNode.getNodeType()) {
          case Node.TEXT_NODE:
            fragment.appendChild(document.importNode(childNode, false));
            break;
          case Node.ELEMENT_NODE:  {
            String nodeName = childNode.getNodeName();
            if (nodeName.equals("br"))
              fragment.appendChild(document.createElement("br"));
            else if (nodeName.equals("reference"))
              fragment.appendChild(
                createRef(((Element)childNode).getAttribute("reference")));
            break;
          }
        }
        childNode = childNode.getNextSibling();
      }
      return fragment;
    }

    public void transformPackage(Element packageElement)
      throws TransformerException {
      if (!packageElement.getNodeName().equals("package"))
        throw new TransformerException("<package> expected");
      Element headElement = document.createElement("h2");
      headElement.appendChild(document.createTextNode("Package "));
      headElement.appendChild(document.createEntityReference("nbsp"));
      headElement.appendChild(document.createEntityReference("nbsp"));
      headElement.appendChild(packElement("small",
        createPackageStack(packageElement)));
      Element aElement = createAnchor(packageElement);
      aElement.appendChild(
        document.createTextNode(packageElement.getAttribute("name")));
      headElement.appendChild(aElement);
      bodyElement.appendChild(headElement);
      addDescription(packageElement);
      addIndex(packageElement, "package", 1, "Package list");
      addIndex(packageElement, "class", 1, "Class list");
      Element childPackageElement =
        ElementBrowser.getFirstChildElement(packageElement, "package");
      while (childPackageElement != null) {
        transformPackage(childPackageElement);
        childPackageElement =
          ElementBrowser.getNextSiblingElement(childPackageElement, "package");
      }
      Element childClassElement =
        ElementBrowser.getFirstChildElement(packageElement, "class");
      while (childClassElement != null) {
        transformClass(childClassElement);
        childClassElement =
          ElementBrowser.getNextSiblingElement(childClassElement, "class");
      }
    }

    private void transformClass(Element classElement)
      throws TransformerException {
      if (!classElement.getNodeName().equals("class"))
        throw new TransformerException("<class> expected");
      Element headElement = document.createElement("h3");
      headElement.appendChild(document.createTextNode("Class"));
      String type = classElement.getAttribute("type");
      if (!type.equals("standard"))
        headElement.appendChild(document.createTextNode("(" + type + ")"));
      headElement.appendChild(document.createEntityReference("nbsp"));
      headElement.appendChild(document.createEntityReference("nbsp"));
      headElement.appendChild(document.createEntityReference("nbsp"));
      headElement.appendChild(packElement("small",
        createPackageStack(classElement)));
      Element aElement = createAnchor(classElement);
      aElement.appendChild(
        document.createTextNode(classElement.getAttribute("name")));
      headElement.appendChild(aElement);
      bodyElement.appendChild(headElement);
      Node generalisationFragment =
        createTokensFragment(classElement.getAttribute("generalisation"));
      if (generalisationFragment.hasChildNodes()) {
        generalisationFragment.insertBefore(
          document.createTextNode("Extends or implements "),
          generalisationFragment.getFirstChild());
        generalisationFragment.appendChild(document.createTextNode("."));
        addParagraph(generalisationFragment);
      }
      addDescription(classElement);
      transformInstances(classElement);
      if (ElementBrowser.hasChildElements(classElement, "instance")
        && ElementBrowser.hasChildElements(classElement, "property"))
        bodyElement.appendChild(document.createElement("p"));
      transformProperties(classElement);
      if ((ElementBrowser.hasChildElements(classElement, "instance")
        || ElementBrowser.hasChildElements(classElement, "property"))
        && ElementBrowser.hasChildElements(classElement, "method"))
        bodyElement.appendChild(document.createElement("p"));
      transformMethods(classElement);
    }

    private void transformInstances(Element classElement)
      throws TransformerException {
      if (!ElementBrowser.hasChildElements(classElement, "instance"))
        return;
      Element tableElement = createTableElement();
      Element theadElement = document.createElement("thead");
      {
        Element thElement = createThElement(2, 1);
        thElement.appendChild(packElement("small",
          document.createTextNode(classElement.getAttribute("name"))));
        thElement.appendChild(document.createEntityReference("nbsp"));
        thElement.appendChild(document.createTextNode(" instances"));
        theadElement.appendChild(packElement("tr", thElement));
      }
      {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createThElement("name"));
        trElement.appendChild(createThElement("description"));
        theadElement.appendChild(trElement);
      }
      tableElement.appendChild(theadElement);
      Element tbodyElement = document.createElement("tbody");
      Element instanceElement =
        ElementBrowser.getFirstChildElement(classElement, "instance");
      while (instanceElement != null) {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createTdElement(packAnchor(instanceElement,
          document.createTextNode(instanceElement.getAttribute("name")))));
        trElement.appendChild(createDescriptionTdElement(instanceElement));
        tbodyElement.appendChild(trElement);
        instanceElement =
          ElementBrowser.getNextSiblingElement(instanceElement, "instance");
      }
      tableElement.appendChild(tbodyElement);
      bodyElement.appendChild(tableElement);
    }

    private void transformProperties(Element classElement)
      throws TransformerException {
      if (!ElementBrowser.hasChildElements(classElement, "property"))
        return;
      Element tableElement = createTableElement();
      Element theadElement = document.createElement("thead");
      {
        Element thElement = createThElement(7, 1);
        thElement.appendChild(packElement("small",
          document.createTextNode(classElement.getAttribute("name"))));
        thElement.appendChild(document.createEntityReference("nbsp"));
        thElement.appendChild(document.createTextNode(" properties"));
        theadElement.appendChild(packElement("tr", thElement));
      }
      {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createThElement("name"));
        trElement.appendChild(createThElement("kind"));
        trElement.appendChild(createThElement("mult."));
        trElement.appendChild(createThElement("type"));
        trElement.appendChild(createThElement("over"));
        trElement.appendChild(createThElement("init"));
        trElement.appendChild(createThElement("description"));
        theadElement.appendChild(trElement);
      }
      tableElement.appendChild(theadElement);
      Element tbodyElement = document.createElement("tbody");
      Element propertyElement =
        ElementBrowser.getFirstChildElement(classElement, "property");
      while (propertyElement != null) {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createTdElement(packAnchor(propertyElement,
          document.createTextNode(propertyElement.getAttribute("name")))));
        trElement.appendChild(createTdElement(translateType(
          propertyElement.getAttribute("type"))));
        trElement.appendChild(createTdElement(translateMultiplicity(
          propertyElement.getAttribute("multiplicity"))));
        trElement.appendChild(createTdElement(createRef(
          propertyElement.getAttribute("reference"))));
        trElement.appendChild(createTdElement(createTokensFragment(
          propertyElement.getAttribute("generalisation"))));
        String initId = propertyElement.getAttribute("init");
        trElement.appendChild(createTdElement(initId.length() != 0 ?
          (Node)createRef(initId) : document.createTextNode("")));
        trElement.appendChild(createDescriptionTdElement(propertyElement));
        tbodyElement.appendChild(trElement);
        propertyElement =
          ElementBrowser.getNextSiblingElement(propertyElement, "property");
      }
      tableElement.appendChild(tbodyElement);
      bodyElement.appendChild(tableElement);
    }

    private void transformMethods(Element classElement)
      throws TransformerException {
      if (!ElementBrowser.hasChildElements(classElement, "method"))
        return;
      Element tableElement = createTableElement();
      final int descriptionColspan = 3;
      final int argumentColspan = 5;
      Element theadElement = document.createElement("thead");
      {
        Element thElement =
          createThElement(1 + descriptionColspan + argumentColspan, 1);
        thElement.appendChild(packElement("small",
          document.createTextNode(classElement.getAttribute("name"))));
        thElement.appendChild(document.createEntityReference("nbsp"));
        thElement.appendChild(document.createTextNode(" methods"));
        theadElement.appendChild(packElement("tr", thElement));
      }
      {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createThElement("name", 1, 2));
        trElement.appendChild(createThElement("kind"));
        trElement.appendChild(createThElement("returns"));
        trElement.appendChild(createThElement("over"));
        trElement.appendChild(createThElement("arguments", argumentColspan, 1));
        theadElement.appendChild(trElement);
      }
      {
        Element trElement = document.createElement("tr");
        trElement.appendChild(createThElement("description",
          descriptionColspan, 1));
        trElement.appendChild(createThElement("name"));
        trElement.appendChild(createThElement("kind"));
        trElement.appendChild(createThElement("mult."));
        trElement.appendChild(createThElement("type"));
        trElement.appendChild(createThElement("desc."));
        theadElement.appendChild(trElement);
      }
      tableElement.appendChild(theadElement);
      Element tbodyElement = document.createElement("tbody");
      Element methodElement =
        ElementBrowser.getFirstChildElement(classElement, "method");
      while (methodElement != null) {
        int argumentCount =
          ElementBrowser.getChildElementCount(methodElement, "argument");
        int descriptionRowspan = argumentCount > 1 ? argumentCount - 1 : 1;
        Element trElement = document.createElement("tr");
        trElement.appendChild(createTdElement(packAnchor(methodElement,
          document.createTextNode(methodElement.getAttribute("name"))),
          1, descriptionRowspan + 1));
        trElement.appendChild(createTdElement(
          translateType(methodElement.getAttribute("type"))));
        String returnId = methodElement.getAttribute("reference");
        trElement.appendChild(createTdElement(returnId.length() != 0 ?
          (Node)createRef(returnId) : document.createTextNode("")));
        trElement.appendChild(createTdElement(createTokensFragment(
          methodElement.getAttribute("generalisation"))));
        Element argumentElement =
          ElementBrowser.getFirstChildElement(methodElement, "argument");
        argumentCount = 0;
        while (argumentElement != null) {
          if (argumentCount != 0) {
            tbodyElement.appendChild(trElement);
            trElement = document.createElement("tr");
            if (argumentCount == 1)
              trElement.appendChild(createDescriptionTdElement(
                methodElement, descriptionColspan, descriptionRowspan));
          }
          trElement.appendChild(createTdElement(packAnchor(argumentElement,
            document.createTextNode(argumentElement.getAttribute("name")))));
          trElement.appendChild(createTdElement(translateType(
            argumentElement.getAttribute("type"))));
          trElement.appendChild(createTdElement(translateMultiplicity(
            argumentElement.getAttribute("multiplicity"))));
          trElement.appendChild(createTdElement(createRef(
            argumentElement.getAttribute("reference"))));
          trElement.appendChild(createDescriptionTdElement(argumentElement));
          argumentElement =
            ElementBrowser.getNextSiblingElement(argumentElement, "argument");
          argumentCount++;
        }
        if (argumentCount < 2) {
          trElement.appendChild(createTdElement(
            argumentColspan, argumentCount == 0 ? 2 : 1));
          tbodyElement.appendChild(trElement);
          trElement = document.createElement("tr");
          trElement.appendChild(createDescriptionTdElement(
            methodElement, descriptionColspan, 1));
        }
        tbodyElement.appendChild(trElement);
        methodElement =
          ElementBrowser.getNextSiblingElement(methodElement, "method");
      }
      tableElement.appendChild(tbodyElement);
      bodyElement.appendChild(tableElement);
    }

    private Element createTableElement() {
      Element tableElement = document.createElement("table");
      tableElement.setAttribute("border", "2");
      tableElement.setAttribute("cellspacing", "4");
      return tableElement;
    }

    private Element createThElement(String text) {
      return createThElement(text, 1, 1);
    }

    private Element createThElement(String text, int colspan, int rowspan) {
      return createThElement(document.createTextNode(text), colspan, rowspan);
    }

    private Element createThElement(Node fragment) {
      return createThElement(fragment, 1, 1);
    }

    private Element createThElement(Node fragment, int colspan, int rowspan) {
      Element thElement = createThElement(colspan, rowspan);
      thElement.appendChild(fragment);
      return thElement;
    }

    private Element createThElement(int colspan, int rowspan) {
      Element thElement = document.createElement("th");
      thElement.setAttribute("align", "center");
      thElement.setAttribute("colspan", new Integer(colspan).toString());
      thElement.setAttribute("rowspan", new Integer(rowspan).toString());
      return thElement;
    }

    private Element createTdElement(String text) {
      return createTdElement(text, 1, 1);
    }

    private Element createTdElement(String text, int colspan, int rowspan) {
      return createTdElement(document.createTextNode(text), colspan, rowspan);
    }

    private Element createTdElement(Node fragment) {
      return createTdElement(fragment, 1, 1);
    }

    private Element createTdElement(Node fragment, int colspan, int rowspan) {
      Element tdElement = createTdElement(colspan, rowspan);
      tdElement.appendChild(fragment);
      return tdElement;
    }

    private Element createTdElement(int colspan, int rowspan) {
      Element tdElement = document.createElement("td");
      tdElement.setAttribute("align", "left");
      tdElement.setAttribute("colspan", new Integer(colspan).toString());
      tdElement.setAttribute("rowspan", new Integer(rowspan).toString());
      return tdElement;
    }

    private Element createDescriptionTdElement(Element parentElement)
      throws TransformerException {
      return createDescriptionTdElement(parentElement, 1, 1);
    }

    private Element createDescriptionTdElement(Element parentElement,
      int colspan, int rowspan) throws TransformerException {
      Element descriptionElement =
        ElementBrowser.getFirstChildElement(parentElement, "description");
      return descriptionElement != null ?
        createTdElement(packElement("small",
        descriptionFragment(descriptionElement)), colspan, rowspan) :
        createTdElement(colspan, rowspan);
    }
  }
}
