/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.transform;

import java.util.Locale;
import java.util.ResourceBundle;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.transform.AbstractTransformerToHtml;

public abstract class ToolTransformerToHtml extends AbstractTransformerToHtml {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected static final String PARAMETER_HOMEURL = "homeURL";
  protected static final String PARAMETER_HOMELABEL = "homeLabel";
  protected static final String PARAMETER_COPYRIGHT = "copyright";

  protected ToolTransformerToHtml() {
    this.setParameter(PARAMETER_HOMEURL, "index.html");
    this.setParameter(PARAMETER_HOMELABEL, "Home");
    this.setParameter(PARAMETER_COPYRIGHT, Constants.COPYRIGHT);
  }

  protected void addHomeRefElement(Element bodyElement) {
    String url = this.getParameter(PARAMETER_HOMEURL).toString();
    String label = this.getParameter(PARAMETER_HOMELABEL).toString();
    if (url.length() == 0 || label.length() == 0)
      return;
    Document document = bodyElement.getOwnerDocument();
    Element pElement = document.createElement("p");
    Element aElement = document.createElement("a");
    aElement.setAttribute("href", url);
    aElement.appendChild(document.createTextNode(label));
    pElement.appendChild(aElement);
    bodyElement.appendChild(pElement);
  }

  protected void addEndFragment(Element bodyElement, String lastUpdate) {
    Document document = bodyElement.getOwnerDocument();
    bodyElement.appendChild(document.createElement("hr"));
    String copyright = this.getParameter(PARAMETER_COPYRIGHT).toString();
    if (lastUpdate.length() == 0 && copyright.length() == 0)
      return;
    Element tableElement = document.createElement("table");
    tableElement.setAttribute("border" , "0");
    tableElement.setAttribute("cellpadding" , "0");
    tableElement.setAttribute("width" , "100%");
    Element trElement = document.createElement("tr");
    {
      Element tdElement = document.createElement("td");
      tdElement.appendChild(document.createTextNode(
        viewResource.getString(Label.LAST_UPDATE) + " : " + lastUpdate));
      trElement.appendChild(tdElement);
    }
    {
      Element tdElement = document.createElement("td");
      Element smallElement = document.createElement("small");
      smallElement.appendChild(document.createTextNode(copyright));
      tdElement.appendChild(smallElement);
      trElement.appendChild(tdElement);
    }
    tableElement.appendChild(trElement);
    bodyElement.appendChild(tableElement);
  }
}
