/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.transform;

import java.util.Enumeration;
import java.util.TooManyListenersException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import javax.swing.JOptionPane;
import javax.xml.transform.Source;
import javax.xml.transform.Result;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.SourceLocator;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerException;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

import org.apache.xalan.transformer.TransformerImpl;
import org.apache.xalan.trace.TraceManager;
import org.apache.xalan.trace.TraceListener;
import org.apache.xalan.trace.PrintTraceListener;

import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.transform.AbstractTransformer;

public final class XSLTTransformer extends AbstractTransformer
    implements ErrorListener {
  private Options options;
  private String stylesheetSystemId = null;
  private PrintWriter tracePrintWriter = null;
  private boolean areMessagesDisplayed = true;

  public XSLTTransformer() {
  }

  public void setOptions(Options options) {
    this.options = options;
  }

  public void setStylesheetSystemId(String stylesheetSystemId) {
    this.stylesheetSystemId = stylesheetSystemId;
  }

  private void copyParametersTo(Transformer transformer) {
    Enumeration keys = parameters.keys();
    while (keys.hasMoreElements()) {
      String name = keys.nextElement().toString();
      transformer.setParameter(name, parameters.get(name));
    }
  }

  private TransformerException extractTransformerException(
      TransformerException exception) {
    while (true) {
      if (exception.getLocator() != null)
        return exception;
      Throwable containedException = exception.getException();
      if (containedException == null
          || !(containedException instanceof TransformerException))
        return exception;
      exception = (TransformerException)containedException;
    }
  }

  private void noteExceptionLocation(SourceLocator locator) {
    if (options != null && locator != null) {
      options.setIntOptionValue(
          Options.LOCATORLINENUMBER, locator.getLineNumber());
      options.setIntOptionValue(
          Options.LOCATORCOLUMNNUMBER, locator.getColumnNumber());
    }
  }

  public void transform(Source xmlSource, Result outputTarget)
      throws TransformerException {
    areMessagesDisplayed = true;
    if (options != null)
      try {
      OutputStream traceOutputStream = new FileOutputStream(
          options.getStringOptionValue(Options.XSLTTRACEPATHNAME));
      tracePrintWriter =
          new PrintWriter(new OutputStreamWriter(traceOutputStream));
    }
    catch (FileNotFoundException exception) {
    }
    try {
      TransformerFactory transformerFactory =
          TransformerFactory.newInstance();
      Transformer transformer = transformerFactory.newTransformer(
          new StreamSource(stylesheetSystemId));
      transformer.setErrorListener((ErrorListener)this);
      copyParametersTo(transformer);
      if (transformer instanceof TransformerImpl && options != null) {
        TransformerImpl transformerImpl = (TransformerImpl)transformer;
        if (tracePrintWriter != null) {
          PrintTraceListener printTraceListener =
              new PrintTraceListener(tracePrintWriter);
          printTraceListener.m_traceElements =
              options.getThreeStateOptionValue(Options.TRACEELEMENTS)
              == ThreeStateOption.TRUE;
          printTraceListener.m_traceGeneration =
              options.getThreeStateOptionValue(Options.TRACEGENERATION)
              == ThreeStateOption.TRUE;
          printTraceListener.m_traceSelection =
              options.getThreeStateOptionValue(Options.TRACESELECTION)
              == ThreeStateOption.TRUE;
          printTraceListener.m_traceTemplates =
              options.getThreeStateOptionValue(Options.TRACETEMPLATES)
              == ThreeStateOption.TRUE;
          try {
            transformerImpl.getTraceManager().addTraceListener(
                printTraceListener);
          }
          catch (TooManyListenersException exception) {
          }
        }
        transformerImpl.setQuietConflictWarnings(
            options.getThreeStateOptionValue(Options.QUIETCONFLICTWARNINGS)
            == ThreeStateOption.TRUE);
      }
      transformer.transform(xmlSource, outputTarget);
    }
    catch (TransformerException exception) {
      throw new TransformerException(
          extractTransformerException(exception).getMessageAndLocation());
    }
    finally {
      if (tracePrintWriter != null) {
        tracePrintWriter.close();
        tracePrintWriter = null;
      }
    }
  }

  ///////////////////////////////
  // ErrorListener implementation
  ///////////////////////////////

  public void warning(TransformerException exception)
    throws TransformerException {
    displayTransformerException("warning", exception);
  }

  public void error(TransformerException exception)
    throws TransformerException {
    displayTransformerException("error", exception);
  }

  public void fatalError(TransformerException exception)
    throws TransformerException {
    displayTransformerException("fatalError", exception);
    throw exception;
  }

  private void displayTransformerException(String errorType,
    TransformerException exception) throws TransformerException {
    if (tracePrintWriter != null)
      exception.printStackTrace(tracePrintWriter);
    exception = extractTransformerException(exception);
    noteExceptionLocation(exception.getLocator());
    if (areMessagesDisplayed)
      switch (JOptionPane.showConfirmDialog(null,
        exception.getMessageAndLocation() + "\ncontinue ?",
        "XSLT transformer " + errorType,
        JOptionPane.YES_NO_CANCEL_OPTION)) {
        case JOptionPane.NO_OPTION:
          areMessagesDisplayed = false;
          break;
        case JOptionPane.CANCEL_OPTION:
          throw exception;
      }
  }
}
