/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import org.w3c.dom.Node;
import org.w3c.dom.Document;

public abstract class FragmentUtils {

  public static final void clearFragment(Node fragment) {
    while (true) {
      Node childNode = fragment.getLastChild();
      if (childNode == null)
        break;
      fragment.removeChild(childNode);
    }
  }

  public static final void cloneNodesToFragment(Node fragment,
    Node[] nodes) {
    for (int k = 0; k < nodes.length; k++)
      fragment.appendChild(nodes[k].cloneNode(true));
  }

  public static final void importNodesToFragment(Node destinationFragment,
    Node[] nodes) {
    Document document = destinationFragment.getOwnerDocument();
    for (int k = 0; k < nodes.length; k++)
      destinationFragment.appendChild(document.importNode(nodes[k], true));
  }

  public static final void importFragmentToFragment(Node destinationFragment,
    Node sourceFragment) {
    Document document = destinationFragment.getOwnerDocument();
    Node childNode = sourceFragment.getFirstChild();
    while (childNode != null) {
      destinationFragment.appendChild(document.importNode(childNode, true));
      childNode = childNode.getNextSibling();
    }
  }

  public static final int getChildCount(Node parent) {
    int count = 0;
    Node childNode = parent.getFirstChild();
    while (childNode != null) {
      count++;
      childNode = childNode.getNextSibling();
    }
    return count;
  }

  public static final Node[] removeNodesFromFragment(Node fragment) {
    int nodeCount = getChildCount(fragment);
    Node[] nodes = new Node[nodeCount];
    for (int k = 0; k < nodeCount; k++) {
      Node childNode = fragment.getFirstChild();
      nodes[k] = childNode;
      fragment.removeChild(childNode);
    }
    return nodes;
  }

  public static final Node[] cloneNodesFromFragment(Node fragment) {
    int nodeCount = getChildCount(fragment);
    Node[] nodes = new Node[nodeCount];
    Node childNode = fragment.getFirstChild();
    for (int k = 0; k < nodeCount; k++) {
      nodes[k] = childNode.cloneNode(true);
      childNode = childNode.getNextSibling();
    }
    return nodes;
  }
}
