/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;

public abstract class NodesComparator {

  public static final boolean areNodesEqual(Node node1, Node node2) {
    if (node1 == node2)
      return true;
    if (node1.getNodeType() != node2.getNodeType())
      return false;
    switch (node1.getNodeType()) {
      case Node.ELEMENT_NODE:
        return areElementsEqual((Element)node1, (Element)node2);
      case Node.TEXT_NODE:
      case Node.CDATA_SECTION_NODE:
      case Node.COMMENT_NODE:
        return haveNodesEqualValue(node1, node2);
      case Node.ENTITY_REFERENCE_NODE:
        return haveNodesEqualName(node1, node2);
      case Node.PROCESSING_INSTRUCTION_NODE:
        return haveNodesEqualNameAndValue(node1, node2);
      case Node.DOCUMENT_FRAGMENT_NODE:
        return haveNodesEqualChildren(node1, node2);
      default:
        throw new IllegalArgumentException();
    }
  }

  public static final boolean areNodesEqual(Node[] nodes1, Node[] nodes2) {
    if (nodes1.length != nodes2.length)
      return false;
    for (int k = 0; k < nodes1.length; k++)
      if (!areNodesEqual(nodes1[k], nodes2[k]))
        return false;
    return true;
  }

  public static final boolean areElementsEqual(Element element1,
    Element element2) {
    return areElementsEqualWithoutChildren(element1, element2)
      && haveNodesEqualChildren(element1, element2);
  }

  public static final boolean areElementsEqualWithoutChildren(Element element1,
    Element element2) {
    if (!haveNodesEqualName(element1, element2))
      return false;
    NamedNodeMap attrs1 = element1.getAttributes();
    NamedNodeMap attrs2 = element2.getAttributes();
    if (attrs1.getLength() != attrs2.getLength())
      return false;
    for (int k = 0; k < attrs1.getLength(); k++)
      if (!haveNodesEqualNameAndValue(attrs1.item(k), attrs2.item(k)))
        return false;
    return true;
  }

  public static final boolean haveNodesEqualChildren(Node node1, Node node2) {
    Node childNode1 = node1.getFirstChild();
    Node childNode2 = node2.getFirstChild();
    while (childNode1 != null && childNode2 != null) {
      if (!areNodesEqual(childNode1, childNode2))
        return false;
      childNode1 = childNode1.getNextSibling();
      childNode2 = childNode2.getNextSibling();
    }
    return childNode1 == null && childNode2 == null;
  }

  private static final boolean haveNodesEqualName(Node node1, Node node2) {
    return node1.getNodeName().equals(node2.getNodeName());
  }

  private static final boolean haveNodesEqualValue(Node node1, Node node2) {
    return node1.getNodeValue().equals(node2.getNodeValue());
  }

  private static final boolean haveNodesEqualNameAndValue(
    Node node1, Node node2) {
    return haveNodesEqualName(node1, node2)
      && haveNodesEqualValue(node1, node2);
  }
}

