/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.dom;

import org.w3c.dom.Element;

import org.xmloperator.framework.document.namespace.UndefinedPrefixException;
import org.xmloperator.framework.document.namespace.QualifiedName;
import org.xmloperator.util.dom.NamespaceUtils;

public abstract class QualifiedNameUtils {

  public static final String getElementName(
      QualifiedName qualifiedName, Element contextElement)
      throws UndefinedPrefixException {
    return getName(qualifiedName, contextElement, true);
  }

  public static final String getPseudoElementName(
      QualifiedName qualifiedName, Element contextElement) {
    return getPseudoName(qualifiedName, contextElement, true);
  }

  public static final String getAttributeName(
      QualifiedName qualifiedName, Element contextElement)
      throws UndefinedPrefixException {
    return getName(qualifiedName, contextElement, false);
  }

  public static final String getPseudoAttributeName(
      QualifiedName qualifiedName, Element contextElement) {
    return getPseudoName(qualifiedName, contextElement, false);
  }

  private static final String getName(
      QualifiedName qualifiedName, Element contextElement,
      boolean isElementName) throws UndefinedPrefixException {
    if (qualifiedName.isAnyName())
      return ":";
    String namespaceURI = qualifiedName.getNamespaceURI();
    String localName = qualifiedName.getLocalName();
    if (namespaceURI.length() == 0 || (isElementName
      && namespaceURI.equals(NamespaceUtils.defaultNamespace(contextElement))))
      return localName;
    String prefix =
      NamespaceUtils.namespaceToPrefix(contextElement, namespaceURI);
    if (prefix == null)
      throw new UndefinedPrefixException(contextElement, namespaceURI);
    return prefix + ":" + localName;
  }

  private static final String getPseudoName(
      QualifiedName qualifiedName, Element contextElement,
      boolean isElementName) {
    try {
      return getName(qualifiedName, contextElement, isElementName);
    }
    catch (UndefinedPrefixException exception) {
      return "(" + qualifiedName.getNamespaceURI() + "):"
          + qualifiedName.getLocalName();
    }
  }
}
