/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.util.List;
import java.util.Arrays;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import javax.swing.JScrollPane;
import javax.swing.AbstractListModel;
import javax.swing.ListSelectionModel;
import javax.swing.JList;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogListSelection;
import org.xmloperator.util.view.DialogModal;

public final class DialogListSelectionImpl extends DialogModal
    implements DialogListSelection {
  private final JList listPanel = new JList();
  private List itemList = null;

  public DialogListSelectionImpl(Frame parent) {
    super(parent);
    // Volet liste
    this.listPanel.addMouseListener(
      new MouseAdapter() {
        public void mouseClicked(MouseEvent event) {
          if (event.getClickCount() == 2)
            ok();
        }
      });
    this.listPanel.addKeyListener(
      new KeyAdapter() {
        public void keyPressed(KeyEvent event) {
          if (event.getKeyCode() == KeyEvent.VK_ENTER)
            ok();
          else {
            char ch = event.getKeyChar();
            if (ch != KeyEvent.CHAR_UNDEFINED)
              charPressed(ch);
          }
        }
      });
    JScrollPane scrollPanel = new JScrollPane(this.listPanel);
    this.contentPanel.add(scrollPanel);
    this.pack();
  }

  private void charPressed(char ch) {
    {
      String javaVersion = System.getProperty("java.version");
      if (!javaVersion.startsWith("1.2") && !javaVersion.startsWith("1.3"))
        return;
    }
    int refIndex = listPanel.getSelectedIndex();
    int itemCount = this.itemList.size();
    for (int k = 1; k <= itemCount; k++) {
      int index = (refIndex + k) % itemCount;
      if (itemList.get(index).toString().charAt(0) == ch) {
        listPanel.setSelectedIndex(index);
        listPanel.ensureIndexIsVisible(index);
        return;
      }
    }
  }

  public void setContentFont(Font font) {
    listPanel.setFont(font);
  }

  /////////////////////////////////
  // DialogSelection implementation
  /////////////////////////////////

  public void setItemList(final List itemList) {
    if (this.isVisible())
      throw new IllegalStateException();
    if (itemList.size() == 0)
      throw new IllegalArgumentException();
    this.itemList = itemList;
    listPanel.setModel(
      new AbstractListModel() {
        public int getSize() {
          return itemList.size();
        }
        public Object getElementAt(int index) {
          return itemList.get(index);
        }
      });
    listPanel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    listPanel.setSelectedIndex(0);
  }

  public void setItems(final Object[] items) {
    this.setItemList(Arrays.asList(items));
  }

  public final Object getSelectedItem() {
    int index = getSelectedIndex();
    return index >= 0 ? itemList.get(index) : null;
  }

  /////////////////////////////////////
  // DialogListSelection implementation
  /////////////////////////////////////

  public final int getSelectedIndex() {
    return listPanel.getSelectedIndex();
  }

  public final void setSelectedIndex(int index) {
    int itemCount = this.itemList.size();
    if (index < 0 || index >= itemCount)
      index = 0;
    listPanel.setSelectedIndex(index);
    listPanel.ensureIndexIsVisible(
        index < itemCount - 1 ? index + 1 : itemCount - 1);
  }
}
