/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.util.view.DialogModal;

public class DialogValidText extends DialogModal implements DialogText {
  private final JLabel labelInvite = new JLabel();
  private final JTextField textField = new JTextField(30);

  public DialogValidText(Frame parent) {
    super(parent);
    this.setResizable(false);
    JPanel panelValidText = new JPanel(new BorderLayout());
    panelValidText.setBorder(new EmptyBorder(4, 4, 4, 4));
    panelValidText.add(this.labelInvite, BorderLayout.WEST);
    this.textField.addActionListener(oKActionListener);
    this.textField.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          text_changed();
        }
        public void removeUpdate(DocumentEvent event) {
          text_changed();
        }
        public void changedUpdate(DocumentEvent event) {
          text_changed();
        }
      });
    panelValidText.add(textField);
    this.contentPanel.add(panelValidText);
    this.text_changed();
    this.pack();
  }

  private final void text_changed() {
    this.buttonOK.setEnabled(isTextValid());
  }

  protected boolean isTextValid() {
    return this.textField.getText().length() != 0;
  }

  public void setColums(int columns) {
    this.textField.setColumns(columns);
    this.pack();
  }

  public void setContentFont(Font font) {
    this.textField.setFont(font);
  }

  public void setInvite(String invite) {
    try {
      invite = viewResource.getString(invite);
    }
    catch (Exception exception) {
    }
    this.labelInvite.setText(invite + " : ");
    this.pack();
  }

  ////////////////////////////
  // DialogText implementation
  ////////////////////////////

  public final String getText() {
    return this.textField.getText();
  }

  public final void setText(String text) {
    this.textField.setText(text);
    this.textField.selectAll();
  }
}
