/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.util.Arrays;
import java.util.Collections;
import java.util.ResourceBundle;
import java.awt.Dimension;

import org.w3c.dom.Element;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogText;
import org.xmloperator.framework.view.dialog.DialogSelection;
import org.xmloperator.framework.view.dialog.DialogListSelection;
import org.xmloperator.framework.view.global.GlobalViewAccess;
import org.xmloperator.util.dom.NamespaceUtils;
import org.xmloperator.util.view.ViewUtils;

public abstract class NameSelection {
    private static final String SYMBOL_ANY_PREFIX = "***";
    private static final String SYMBOL_ANY_LOCALNAME = "*****";
    private static final String SYMBOL_ANY_NAME =
      SYMBOL_ANY_PREFIX + ":" + SYMBOL_ANY_LOCALNAME;

  public static final String completeName(
      GlobalViewAccess globalViewAccess, Element contextElement, String name) {
    if (name.equals(":")) {
      String[] prefixes = NamespaceUtils.prefixes(contextElement);
      String prefix;
      if (prefixes.length == 1)
        prefix = prefixes[0];
      else {
        DialogSelection dialogSelectionPrefix =
            globalViewAccess.getDialogSelectionPrefix();
        dialogSelectionPrefix.setItems(prefixes);
        dialogSelectionPrefix.show();
        if (!dialogSelectionPrefix.isOK())
          return null;
        prefix = (String)dialogSelectionPrefix.getSelectedItem();
      }
      name = prefix.length() != 0 ? prefix + ":" : "";
    }
    if (name.length() == 0 || name.endsWith(":")) {
      DialogText dialogLocalName = globalViewAccess.getDialogLocalName(null);
      dialogLocalName.show();
      if (!dialogLocalName.isOK())
        return null;
      name += dialogLocalName.getText();
    }
    return name;
  }

  private static final String getString(String resourceKey) {
    return ResourceBundle.getBundle(Label.CLASS).getString(resourceKey);
  }

  public static final String selectName(
      final GlobalViewAccess globalViewAccess, final String title,
      final String[] names) {
    return selectName(globalViewAccess, title, names, null);
  }

  public static final String selectName(
      final GlobalViewAccess globalViewAccess, final String title,
      final String[] names, String preferredName) {
    for (int k = 0; k < names.length; k++) {
      String name = names[k];
      if (name.length() == 0 || name.endsWith(":")) {
        if (name.equals(":"))
          name = SYMBOL_ANY_NAME;
        else
          name += SYMBOL_ANY_LOCALNAME;
        names[k] = name;
      }
    }
    Arrays.sort(names);
    DialogListSelection dialogSelectionName =
        globalViewAccess.getDialogSelectionName();
    dialogSelectionName.setItems(names);
    dialogSelectionName.setSelectedIndex(stringIndex(names, preferredName));
    dialogSelectionName.setTitle(title);
    dialogSelectionName.show();
    if (!dialogSelectionName.isOK())
      return null;
    String name = dialogSelectionName.getSelectedItem().toString();
    if (name.endsWith(SYMBOL_ANY_LOCALNAME))
      if (name.equals(SYMBOL_ANY_NAME))
        name = ":";
      else
        name = name.substring(0, name.length() - SYMBOL_ANY_LOCALNAME.length());
    return name;
  }

  private static final int stringIndex(String[] names, String preferredName) {
    if (preferredName == null)
      return -1;
    for (int k = 0; k < names.length; k++)
      if (names[k].equals(preferredName))
        return k;
    return -1;
  }
}
