/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.WindowEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JPanel;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.observer.TextView;
import org.xmloperator.framework.view.observer.ViewObserver;

public class TextFrame extends JFrame implements TextView {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected ViewObserver viewObserver = null;
  protected JTextArea textField;
  protected JScrollPane scrollPanel;
  protected boolean isSelecting = false;

  public TextFrame() {
    this(null);
  }

  public TextFrame(Dimension contentDimension) {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    // Content panel
    JPanel contentPanel = new JPanel(new BorderLayout());
    this.getContentPane().add(contentPanel);
    contentPanel.setPreferredSize(
      contentDimension != null ? contentDimension : new Dimension(500, 300));
    // Text field
    textField = new JTextArea();
    textField.setLineWrap(true);
    textField.setWrapStyleWord(true);
    textField.addCaretListener(new CaretListener() {
      public void caretUpdate(CaretEvent event) {
        text_caretUpdate(event.getDot());
      }
    });
    textField.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent event) {
        text_mouseClicked();
      }
    });
    scrollPanel = new JScrollPane(textField);
    contentPanel.add(scrollPanel);
    this.pack();
  }

  protected void processWindowEvent(WindowEvent event) {
    if (event.getID() == WindowEvent.WINDOW_CLOSING)
      dispose();
    else
      super.processWindowEvent(event);
  }

  protected void text_caretUpdate(int location) {
  }

  protected void text_mouseClicked() {
  }

  public void selectText(int start, int end) {
    try {
      isSelecting = true;
      textField.select(start, end);
    }
    finally {
      isSelecting = false;
    }
  }

  //////////////////////////
  // TextView implementation
  //////////////////////////

  public void dispose() {
    super.dispose();
    if (viewObserver != null)
      viewObserver.notifyDispose(this);
  }

  public void setViewObserver(ViewObserver viewObserver) {
    this.viewObserver = viewObserver;
  }

  public void setText(String text) {
    textField.setText(text);
  }
}
