/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.util.view;

import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.io.FileExtensionFilter;

public final class XMLFileChooser extends JFileChooser {
  public static final FileFilter XML_FILE_FILTER =
    new FileExtensionFilter("xml");
  public static final FileFilter DTD_FILE_FILTER =
    new FileExtensionFilter("dtd");
  public static final FileFilter RNG_FILE_FILTER =
    new FileExtensionFilter("rng");
  protected static FileFilter currentFileFilter = XML_FILE_FILTER;
  protected JDialog dialogChooser = null;
  protected int returnCode = ERROR_OPTION;

  public XMLFileChooser() {
    this.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        if(dialogChooser == null)
          return;
        String command = event.getActionCommand();
        if (command.equals(JFileChooser.APPROVE_SELECTION)) {
          returnCode = APPROVE_OPTION;
          dialogChooser.setVisible(false);
        }
        else if (command.equals(JFileChooser.CANCEL_SELECTION)) {
          returnCode = CANCEL_OPTION;
          dialogChooser.setVisible(false);
        }
      }
    });
  }

  public int showDialog() {
    String javaVersion = System.getProperty("java.version");
    if (javaVersion.startsWith("1.2") || javaVersion.startsWith("1.3"))
      return this.showDialog(null, null);
    if (this.dialogChooser == null) {
      this.dialogChooser = this.createDialog(null);
      this.dialogChooser.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
          returnCode = CANCEL_OPTION;
        }
      });
    }
    this.returnCode = ERROR_OPTION;
    this.dialogChooser.show();
    this.dialogChooser.dispose();
    return this.returnCode;
  }

  protected final void setCurrentDirectoryPath(String currentDirectoryPath) {
    this.setCurrentDirectory(
        FileSystemView.getFileSystemView().
        createFileObject(currentDirectoryPath));
  }

  public final String chooseDocumentPathName(boolean forSave,
    String title, String currentDirectoryPath, String fileExtensions) {
    this.setCurrentDirectoryPath(currentDirectoryPath);
    this.setDialogType(
      forSave ? JFileChooser.SAVE_DIALOG : JFileChooser.OPEN_DIALOG);
    this.setDialogTitle(title);
    this.addChoosableFileFilter(XML_FILE_FILTER);
    StringTokenizer extensionsTokenizer =
      new StringTokenizer(fileExtensions, " ,;/");
    while (extensionsTokenizer.hasMoreTokens()) {
      String extension = extensionsTokenizer.nextToken();
      if (!extension.equalsIgnoreCase("xml"))
        this.addChoosableFileFilter(
          new FileExtensionFilter(extension));
    }
    this.setFileFilter(currentFileFilter != null ?
      currentFileFilter : this.getAcceptAllFileFilter());
    int returnVal = this.showDialog();
    currentFileFilter = this.getFileFilter();
    if (currentFileFilter == this.getAcceptAllFileFilter())
      currentFileFilter = null;
    if (returnVal != JFileChooser.APPROVE_OPTION)
      return null;
    String documentPathName = this.getSelectedFile().getPath();
    if (forSave && documentPathName.lastIndexOf('.') < 0)
      if (currentFileFilter != null
        && currentFileFilter instanceof FileExtensionFilter)
        documentPathName +=
          "." + ((FileExtensionFilter)currentFileFilter).getExtension();
      else
        documentPathName += ".xml";
    documentPathName = FileUtils.normalisePathName(documentPathName);
    return documentPathName;
  }

  public final String chooseDTDPathName(
      boolean forSave, String currentDirectoryPath) {
    final ResourceBundle viewResource = ResourceBundle.getBundle(Label.CLASS);
    this.setCurrentDirectoryPath(currentDirectoryPath);
    this.setDialogType(
      forSave ? JFileChooser.SAVE_DIALOG : JFileChooser.OPEN_DIALOG);
    this.setDialogTitle(viewResource.getString(
      forSave ? Label.SAVE_DTD : Label.OPEN_DTD));
    this.setFileFilter(DTD_FILE_FILTER);
    if (this.showDialog() != JFileChooser.APPROVE_OPTION)
      return null;
    String dTDPathName = this.getSelectedFile().getPath();
    if (forSave && dTDPathName.lastIndexOf('.') < 0)
      dTDPathName += ".dtd";
    dTDPathName = FileUtils.normalisePathName(dTDPathName);
    return dTDPathName;
  }

  public final String chooseRNGPathName(boolean forSave,
      String currentDirectoryPath) {
    final ResourceBundle viewResource = ResourceBundle.getBundle(Label.CLASS);
    this.setCurrentDirectoryPath(currentDirectoryPath);
    this.setDialogType(
        forSave ? JFileChooser.SAVE_DIALOG : JFileChooser.OPEN_DIALOG);
    this.setDialogTitle(viewResource.getString(
        forSave ? Label.SAVE_RNG : Label.OPEN_RNG));
    this.setFileFilter(RNG_FILE_FILTER);
    if (this.showDialog() != JFileChooser.APPROVE_OPTION)
      return null;
    String rNGPathName = this.getSelectedFile().getPath();
    if (forSave && rNGPathName.lastIndexOf('.') < 0)
      rNGPathName += ".rng";
    rNGPathName = FileUtils.normalisePathName(rNGPathName);
    return rNGPathName;
  }

  public final String chooseDTDXMLPathName(String currentDirectoryPath) {
    this.setCurrentDirectoryPath(currentDirectoryPath);
    final ResourceBundle viewResource = ResourceBundle.getBundle(Label.CLASS);
    this.setDialogType(JFileChooser.SAVE_DIALOG);
    this.setDialogTitle(viewResource.getString(Label.SAVE_DTD));
    this.setFileFilter(XML_FILE_FILTER);
    if (this.showDialog() != JFileChooser.APPROVE_OPTION)
      return null;
    String dTDPathName = this.getSelectedFile().getPath();
    if (dTDPathName.lastIndexOf('.') < 0)
      dTDPathName += ".xml";
    dTDPathName = FileUtils.normalisePathName(dTDPathName);
    return dTDPathName;
  }
}
