/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Collections;
import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonModel;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import org.xmloperator.framework.document.elementKey.ElementKey;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.dialog.DialogElementKey;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.document.elementKey.ElementKeyImpl;
import org.xmloperator.document.elementKey.ElementKeyUtil;

public final class DialogElementKeyImpl extends DialogModal
    implements DialogElementKey {
  private final ButtonGroup buttonGroup = new ButtonGroup();
  private final JRadioButton radioButtonLeafText = new JRadioButton();
  private final JRadioButton radioButtonAllAttributes = new JRadioButton();
  private final JRadioButton radioButtonAllSingleElements = new JRadioButton();
  private final JRadioButton radioButtonAllAttributesAndSingleElements =
      new JRadioButton();
  private final JRadioButton radioButtonOneAttribute = new JRadioButton();
  private final JRadioButton radioButtonOneSingleElement = new JRadioButton();
  private final JComboBox comboBoxAttributeName = new JComboBox();
  private final JComboBox comboBoxElementName = new JComboBox();

  public DialogElementKeyImpl(Frame parent) {
    super(parent);
    this.setResizable(false);
    // Button group
    this.buttonGroup.add(this.radioButtonLeafText);
    this.buttonGroup.add(this.radioButtonAllAttributes);
    this.buttonGroup.add(this.radioButtonAllSingleElements);
    this.buttonGroup.add(this.radioButtonAllAttributesAndSingleElements);
    this.buttonGroup.add(this.radioButtonOneAttribute);
    this.buttonGroup.add(this.radioButtonOneSingleElement);
    {
      // Actions of buttons
      ActionListener actionListener = new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          radioButton_actionPerformed(event.getSource());
        }
      };
      this.radioButtonLeafText.addActionListener(actionListener);
      this.radioButtonAllAttributes.addActionListener(actionListener);
      this.radioButtonAllSingleElements.addActionListener(actionListener);
      this.radioButtonAllAttributesAndSingleElements.addActionListener(
          actionListener);
      this.radioButtonOneAttribute.addActionListener(actionListener);
      this.radioButtonOneSingleElement.addActionListener(actionListener);
    }
    // Enter key
    this.radioButtonLeafText.addKeyListener(this.okOnEnterKeyListener);
    this.radioButtonAllAttributes.addKeyListener(this.okOnEnterKeyListener);
    this.radioButtonAllSingleElements.addKeyListener(this.okOnEnterKeyListener);
    this.radioButtonAllAttributesAndSingleElements.addKeyListener(
        this.okOnEnterKeyListener);
    this.radioButtonOneAttribute.addKeyListener(this.okOnEnterKeyListener);
    this.radioButtonOneSingleElement.addKeyListener(this.okOnEnterKeyListener);
    this.comboBoxAttributeName.addKeyListener(this.okOnEnterKeyListener);
    this.comboBoxElementName.addKeyListener(this.okOnEnterKeyListener);
    // Button labelling
    this.radioButtonLeafText.setText(
        viewResource.getString(Label.TEXT_OF_LEAF_ELEMENT));
    this.radioButtonAllAttributes.setText(
        viewResource.getString(Label.ALL_ATTRIBUTES));
    this.radioButtonAllSingleElements.setText(
        viewResource.getString(Label.ALL_SINGLE_ELEMENTS));
    this.radioButtonAllAttributesAndSingleElements.setText(
        viewResource.getString(Label.ALL_ATTRIBUTES_AND_SINGLE_ELEMENTS));
    this.radioButtonOneAttribute.setText(
        viewResource.getString(Label.THE_ATTRIBUTE_WITH_THAT_NAME) + " :");
    this.radioButtonOneSingleElement.setText(
        viewResource.getString(Label.THE_SINGLE_ELEMENT_WITH_THAT_NAME) + " :");
    {
      // Leaf text panel
      JPanel panelLeafText = new JPanel(new BorderLayout());
      panelLeafText.add(this.radioButtonLeafText);
      this.contentPanel.add(panelLeafText);
    }
    {
      // All attributes panel
      JPanel panelAllAttributes = new JPanel(new BorderLayout());
      panelAllAttributes.add(this.radioButtonAllAttributes);
      this.contentPanel.add(panelAllAttributes);
    }
    {
      // All single elements panel
      JPanel panelAllSingleElements = new JPanel(new BorderLayout());
      panelAllSingleElements.add(this.radioButtonAllSingleElements);
      this.contentPanel.add(panelAllSingleElements);
    }
    {
      // All attributes and single elements panel
      JPanel panelAllAttributesAndSingleElements = new JPanel(new BorderLayout());
      panelAllAttributesAndSingleElements.add(
          this.radioButtonAllAttributesAndSingleElements);
      this.contentPanel.add(panelAllAttributesAndSingleElements);
    }
    {
      // One attribute panel
      JPanel panelOneAttribute = new JPanel(new BorderLayout());
      panelOneAttribute.add(this.radioButtonOneAttribute);
      panelOneAttribute.add(this.comboBoxAttributeName, BorderLayout.EAST);
      this.contentPanel.add(panelOneAttribute);
    }
    {
      // One single element panel
      JPanel panelOneSingleElement = new JPanel(new BorderLayout());
      panelOneSingleElement.add(this.radioButtonOneSingleElement);
      panelOneSingleElement.add(this.comboBoxElementName, BorderLayout.EAST);
      this.contentPanel.add(panelOneSingleElement);
    }
    this.addSeparator();
    // setTitle(...) and pack() to do
  }

  protected void radioButton_actionPerformed(Object source) {
    try {
      this.buttonOK.setEnabled(true);
      this.comboBoxAttributeName.setEnabled(false);
      this.comboBoxElementName.setEnabled(false);
      if (source == this.radioButtonOneAttribute)
        this.setEnabledIfNotEmpty(this.comboBoxAttributeName);
      else if (source == this.radioButtonOneSingleElement)
        this.setEnabledIfNotEmpty(this.comboBoxElementName);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void setEnabledIfNotEmpty(JComboBox comboBox) {
    boolean isNotEmpty = comboBox.getItemCount() != 0;
    comboBox.setEnabled(isNotEmpty);
    this.buttonOK.setEnabled(isNotEmpty);
  }

  public void setContentFont(Font font) {
    this.comboBoxAttributeName.setFont(font);
    this.comboBoxElementName.setFont(font);
  }

  //////////////////////////////////
  // DialogElementKey implementation
  //////////////////////////////////

  public void setNames(Node parentNode, String elementName) {
    setTitle("<" + elementName + "> "
             + viewResource.getString(Label.ELEMENT_LABELLING));
    Set attributeNames = new HashSet();
    Set singleElementNames = new HashSet();
    Set multipleElementNames = new HashSet();
    addNames(parentNode, elementName, attributeNames,
             singleElementNames, multipleElementNames);
    singleElementNames.removeAll(multipleElementNames);
    this.fillCombo(this.comboBoxAttributeName, attributeNames);
    this.fillCombo(this.comboBoxElementName, singleElementNames);
    attributeNames.clear();
    singleElementNames.clear();
    this.pack();
  }

  private static final void addNames(
      Node parentNode, String elementName,
      Set attributeNames, Set singleElementNames, Set multipleElementNames) {
    Node childNode = parentNode.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        if (childNode.getNodeName().equals(elementName)) {
          addAttributeNames(attributeNames, (Element)childNode);
          addElementNames(singleElementNames, multipleElementNames,
                          (Element)childNode);
        }
        addNames(childNode, elementName, attributeNames,
                 singleElementNames, multipleElementNames);
      }
      childNode = childNode.getNextSibling();
    }
  }

  private static final void addAttributeNames(
      Set attributeNames, Element element) {
    NamedNodeMap attributes = element.getAttributes();
    for (int k = 0; k < attributes.getLength(); k++)
      attributeNames.add(attributes.item(k).getNodeName());
  }

  private static final void addElementNames(
      Set singleElementNames, Set multipleElementNames, Element parentElement) {
    Node childNode = parentElement.getFirstChild();
    while (childNode != null) {
      if (childNode.getNodeType() == Node.ELEMENT_NODE) {
        String elementName = childNode.getNodeName();
        if (ElementKeyUtil.hasElementOrEntityRef((Element)childNode)
            || ElementKeyUtil.getSingleElement(parentElement, elementName)
            == null)
          multipleElementNames.add(elementName);
        else
          singleElementNames.add(elementName);
      }
      childNode = childNode.getNextSibling();
    }
  }

  private void fillCombo(JComboBox comboBox, Set names) {
    if (comboBox.getItemCount() != 0)
      comboBox.removeAllItems();
    List nameList = new ArrayList(names);
    Collections.sort(nameList);
    Iterator iterator = nameList.iterator();
    while (iterator.hasNext())
      comboBox.addItem(iterator.next());
    nameList.clear();
  }

  public void setElementKey(ElementKey elementKey) {
    this.radioButtonAllAttributes.setSelected(true);
    this.radioButtonAllAttributes.grabFocus();
    this.comboBoxAttributeName.setEnabled(false);
    this.comboBoxElementName.setEnabled(false);
    this.buttonOK.setEnabled(true);
    if (elementKey == null)
      return;
    switch (elementKey.getKeyType()) {
      case ElementKey.KEYTYPE_TEXTIFLEAF:
        this.radioButtonLeafText.setSelected(true);
        this.radioButtonLeafText.grabFocus();
        break;
      case ElementKey.KEYTYPE_ALLATTRIBUTES:
        /*
        this.radioButtonAllAttributes.setSelected(true);
        this.radioButtonAllAttributes.grabFocus();
        */
        break;
      case ElementKey.KEYTYPE_ALLSINGLEELEMENTS:
        this.radioButtonAllSingleElements.setSelected(true);
        this.radioButtonAllSingleElements.grabFocus();
        break;
      case ElementKey.KEYTYPE_ALLATTRIBUTESANDSINGLEELEMENTS:
        this.radioButtonAllAttributesAndSingleElements.setSelected(true);
        this.radioButtonAllAttributesAndSingleElements.grabFocus();
        break;
      case ElementKey.KEYTYPE_ONEATTRIBUTE: {
        String attributeName = elementKey.getAttributeOrSingleElementName();
        this.comboBoxAttributeName.setSelectedItem(attributeName);
        if (!this.comboBoxAttributeName.getSelectedItem().equals(attributeName)) {
          this.buttonOK.setEnabled(false);
          return;
        }
        this.radioButtonOneAttribute.setSelected(true);
        this.comboBoxAttributeName.setEnabled(true);
        this.comboBoxAttributeName.grabFocus();
        break;
      }
      case ElementKey.KEYTYPE_ONESINGLEELEMENT:
        String singleElementName = elementKey.getAttributeOrSingleElementName();
        this.comboBoxElementName.setSelectedItem(singleElementName);
        if (!this.comboBoxElementName.getSelectedItem().
            equals(singleElementName)) {
          this.buttonOK.setEnabled(false);
          return;
        }
        this.radioButtonOneSingleElement.setSelected(true);
        this.comboBoxElementName.setEnabled(true);
        this.comboBoxElementName.grabFocus();
        break;
      default:
        throw new IllegalArgumentException();
    }
  }

  public ElementKey getElementKey() {
    if (this.radioButtonLeafText.isSelected())
      return ElementKeyImpl.ELEMENTKEY_TEXTIFLEAF;
    if (this.radioButtonAllAttributes.isSelected())
      return ElementKeyImpl.ELEMENTKEY_ALLATTRIBUTES;
    if (this.radioButtonAllSingleElements.isSelected())
      return ElementKeyImpl.ELEMENTKEY_ALLSINGLEELEMENTS;
    if (this.radioButtonAllAttributesAndSingleElements.isSelected())
      return ElementKeyImpl.ELEMENTKEY_ALLATTRIBUTESANDSINGLEELEMENTS;
    if (this.radioButtonOneAttribute.isSelected()) {
      String attributeName =
          (String)this.comboBoxAttributeName.getSelectedItem();
      return attributeName != null ? new ElementKeyImpl(
          ElementKey.KEYTYPE_ONEATTRIBUTE, attributeName) : null;
    }
    if (this.radioButtonOneSingleElement.isSelected()) {
      String singleElementName =
          (String)this.comboBoxElementName.getSelectedItem();
      return singleElementName != null ? new ElementKeyImpl(
          ElementKey.KEYTYPE_ONESINGLEELEMENT, singleElementName) : null;
    }
    return null;
  }
}
