/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Font;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.document.transform.
    TransformerInfoXMLResultPart;
import org.xmloperator.framework.document.transform.TransformerInfo;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.util.io.FileUtils;
import org.xmloperator.util.io.FileExtensionFilter;
import org.xmloperator.util.view.DialogModal;

public final class DialogTransformerInfo extends DialogModal {
  private static final Border borderLigne = new EmptyBorder(2, 4, 2, 4);
  private static final FileFilter XSL_FILE_FILTER =
    new FileExtensionFilter("xsl");
  private TransformerInfo transformerInfo = null;
  private final JTextField fieldTransformerName = new JTextField(20);
  private final JTextField fieldDocumentTypeName = new JTextField(20);
  private final JTextField fieldTransformerClassName = new JTextField(30);
  private final JTextField fieldStylesheetSystemId = new JTextField(30);
  private final JTextField fieldParamNames = new JTextField(30);
  private final JCheckBox fieldXMLResult = new JCheckBox("XML result");
  private final JPanel panelResult = new JPanel();
  private final JTextField fieldDTDPublicId = new JTextField(30);
  private final JTextField fieldDTDSystemId = new JTextField(30);
  private final JComboBox fieldEncodingName =
    new JComboBox(Constants.encodingNames);
  private final JCheckBox fieldStandalone =
    new JCheckBox("Standalone document");
  private final JCheckBox fieldShortcut =
    new JCheckBox("Shortcut used for empty elements");
  private final JButton buttonXSLTTransformer = new JButton("XSLT");
  private final JButton buttonChooseStylesheet =
    new JButton(viewResource.getString(Label.CHOOSE));
  private final DocumentListener fieldUpdateListener =
    new DocumentListener() {
      public void insertUpdate(DocumentEvent event) {
        field_update();
      };
      public void removeUpdate(DocumentEvent event) {
        field_update();
      };
      public void changedUpdate(DocumentEvent event) {
        field_update();
      };
    };

  public DialogTransformerInfo(Frame parent) {
    super(parent);
    this.setResizable(false);
    setTitle(viewResource.getString(Label.TRANSFORMER));
    // Transformer name
    JPanel panelTransformerName = new JPanel(new BorderLayout());
    panelTransformerName.setBorder(borderLigne);
    panelTransformerName.add(
      new JLabel(viewResource.getString(Label.NAME) + " : "),
      BorderLayout.WEST);
    fieldTransformerName.getDocument().
      addDocumentListener(fieldUpdateListener);
    fieldTransformerName.addActionListener(oKActionListener);
    panelTransformerName.add(fieldTransformerName);
    contentPanel.add(panelTransformerName);
    // Document type name
    JPanel panelDocumentTypeName = new JPanel(new BorderLayout());
    panelDocumentTypeName.setBorder(borderLigne);
    panelDocumentTypeName.add(
      new JLabel(viewResource.getString(Label.DOCUMENT_TYPE_NAME) + " : "),
      BorderLayout.WEST);
    fieldDocumentTypeName.getDocument().
      addDocumentListener(fieldUpdateListener);
    fieldDocumentTypeName.addActionListener(oKActionListener);
    panelDocumentTypeName.add(fieldDocumentTypeName);
    contentPanel.add(panelDocumentTypeName);
    // Transformer class name
    JPanel voletTransformerClassName = new JPanel(new BorderLayout());
    voletTransformerClassName.setBorder(borderLigne);
    voletTransformerClassName.add(new JLabel(
      viewResource.getString(Label.CLASS_NAME) + " : "),
      BorderLayout.WEST);
    fieldTransformerClassName.addActionListener(oKActionListener);
    voletTransformerClassName.add(fieldTransformerClassName);
    buttonXSLTTransformer.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonXSLTTransformer_actionPerformed();
        }
      });
    voletTransformerClassName.add(buttonXSLTTransformer, BorderLayout.EAST);
    contentPanel.add(voletTransformerClassName);
    // Style sheet system id
    JPanel voletStylesheetSystemId = new JPanel(new BorderLayout());
    voletStylesheetSystemId.setBorder(borderLigne);
    voletStylesheetSystemId.add(
      new JLabel("Style sheet System Id : "), BorderLayout.WEST);
    fieldStylesheetSystemId.addActionListener(oKActionListener);
    voletStylesheetSystemId.add(fieldStylesheetSystemId);
    buttonChooseStylesheet.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonChooseStylesheet_actionPerformed();
        }
      });
    voletStylesheetSystemId.add(buttonChooseStylesheet, BorderLayout.EAST);
    contentPanel.add(voletStylesheetSystemId);
    // Param names
    JPanel panelParamNames = new JPanel(new BorderLayout());
    panelParamNames.setBorder(borderLigne);
    panelParamNames.add(
      new JLabel(viewResource.getString(Label.PARAM_NAMES) + " : "), BorderLayout.WEST);
    fieldParamNames.addActionListener(oKActionListener);
    panelParamNames.add(fieldParamNames);
    contentPanel.add(panelParamNames);
    // XML result
    JPanel panelXMLResult = new JPanel(new BorderLayout());
    panelXMLResult.setBorder(borderLigne);
    fieldXMLResult.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          fieldXMLResult_actionPerformed();
        }
      });
    panelXMLResult.add(fieldXMLResult);
    contentPanel.add(panelXMLResult);
    panelResult.setBorder(new TitledBorder("Result"));
    panelResult.setLayout(new BoxLayout(panelResult, BoxLayout.Y_AXIS));
    // Result DTD Public Id
    JPanel panelDTDPublicId = new JPanel(new BorderLayout());
    panelDTDPublicId.setBorder(borderLigne);
    panelDTDPublicId.add(new JLabel("DTD Public Id : "), BorderLayout.WEST);
    fieldDTDPublicId.addActionListener(oKActionListener);
    panelDTDPublicId.add(fieldDTDPublicId);
    panelResult.add(panelDTDPublicId);
    // Result DTD System Id
    JPanel panelDTDSystemId = new JPanel(new BorderLayout());
    panelDTDSystemId.setBorder(borderLigne);
    panelDTDSystemId.add(new JLabel("DTD System Id : "), BorderLayout.WEST);
    fieldDTDSystemId.addActionListener(oKActionListener);
    panelDTDSystemId.add(fieldDTDSystemId);
    panelResult.add(panelDTDSystemId);
    // Encoding name
    JPanel panelEncodingName = new JPanel(new BorderLayout());
    panelEncodingName.setBorder(borderLigne);
    panelEncodingName.add(new JLabel("Encoding name : "), BorderLayout.WEST);
    fieldEncodingName.setEditable(true);
    panelEncodingName.add(fieldEncodingName);
    panelResult.add(panelEncodingName);
    // Standalone document
    JPanel panelStandalone = new JPanel(new BorderLayout());
    panelStandalone.setBorder(borderLigne);
    panelStandalone.add(fieldStandalone);
    panelResult.add(panelStandalone);
    // Shortcut for empty elements
    JPanel panelShortcut = new JPanel(new BorderLayout());
    panelShortcut.setBorder(borderLigne);
    panelShortcut.add(fieldShortcut);
    panelResult.add(panelShortcut);
    // Pack
    this.pack();
  }

  private void field_update() {
    buttonOK.setEnabled(fieldTransformerName.getText().length() != 0);
  }

  protected void buttonOK_actionPerformed(ActionEvent event) {
    if (!buttonOK.isEnabled())
      return;
    transformerInfo.setTransformerName(fieldTransformerName.getText());
    transformerInfo.setDocumentTypeName(
      fieldDocumentTypeName.getText());
    transformerInfo.setTransformerClassName(
      fieldTransformerClassName.getText());
    transformerInfo.setStylesheetSystemId(
      fieldStylesheetSystemId.getText());
    transformerInfo.setParamNames(fieldParamNames.getText());
    transformerInfo.setResultXML(fieldXMLResult.isSelected());
    TransformerInfoXMLResultPart xMLResultPart =
        transformerInfo.getXMLResultPart();
    xMLResultPart.setResultDTDPublicId(fieldDTDPublicId.getText());
    xMLResultPart.setResultDTDSystemId(fieldDTDSystemId.getText());
    xMLResultPart.setEncodingName(
      (String)fieldEncodingName.getSelectedItem());
    xMLResultPart.setStandaloneDocument(fieldStandalone.isSelected());
    xMLResultPart.setShortcutUsedForEmptyElement(
      fieldShortcut.isSelected());
    isOK = true;
    dispose();
  }

  private void buttonXSLTTransformer_actionPerformed() {
    fieldTransformerClassName.setText(Constants.XSLT_TRANSFORMER);
  }

  private void buttonChooseStylesheet_actionPerformed() {
    String stylesheetPathName = chooseStylesheetPathName();
    if (stylesheetPathName != null)
      fieldStylesheetSystemId.setText(
        FileUtils.reducedSystemId(stylesheetPathName));
  }

  private String chooseStylesheetPathName() {
    JFileChooser fileChooser = new JFileChooser(".");
    fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
    fileChooser.setDialogTitle(viewResource.getString(Label.STYLE_SHEET));
    fileChooser.setFileFilter(XSL_FILE_FILTER);
    return fileChooser.showDialog(this, null) == JFileChooser.APPROVE_OPTION ?
      fileChooser.getSelectedFile().getPath() : null;
  }

  private void fieldXMLResult_actionPerformed() {
    if (fieldXMLResult.isSelected())
      this.contentPanel.add(panelResult);
    else
      this.contentPanel.remove(panelResult);
    this.pack();
  }

  public void setContentFont(Font font) {
  }

  public void setTransformerInfo(TransformerInfo transformerInfo) {
    this.transformerInfo = transformerInfo;
    TransformerInfoXMLResultPart xMLResultPart =
        transformerInfo.getXMLResultPart();
    this.fieldDTDPublicId.setText(xMLResultPart.getResultDTDPublicId());
    this.fieldDTDSystemId.setText(xMLResultPart.getResultDTDSystemId());
    this.fieldEncodingName.setSelectedItem(xMLResultPart.getEncodingName());
    this.fieldStandalone.setSelected(xMLResultPart.isStandaloneDocument());
    this.fieldShortcut.setSelected(xMLResultPart.isShortcutUsedForEmptyElement());
    this.buttonOK.setEnabled(
        transformerInfo.getTransformerName().length() != 0);
    this.fieldTransformerName.setText(transformerInfo.getTransformerName());
    this.fieldDocumentTypeName.setText(transformerInfo.getDocumentTypeName());
    this.fieldTransformerClassName.setText(
        transformerInfo.getTransformerClassName());
    this.fieldStylesheetSystemId.setText(
        transformerInfo.getStylesheetSystemId());
    this.fieldParamNames.setText(
        transformerInfo.getParamNames());
    this.fieldXMLResult.setSelected(transformerInfo.isResultXML());
    if (transformerInfo.isResultXML())
      this.contentPanel.add(this.panelResult);
    else
      this.contentPanel.remove(this.panelResult);
    this.pack();
  }

  public void setKeyReadOnly(boolean isKeyReadOnly) {
    this.fieldTransformerName.setEnabled(!isKeyReadOnly);
    this.fieldDocumentTypeName.setEnabled(!isKeyReadOnly);
  }
}
