/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.dialog;

import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Collection;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.util.view.DialogModal;
import org.xmloperator.view.dialog.HistoryField;

public class DialogXPath extends DialogModal {
  private Options options;
  private String rootElementName;
  private HistoryField historyField;
  private final String START = viewResource.getString(Label.START);
  private final String NEXT = viewResource.getString(Label.NEXT);
  private final JButton buttonWizzard = new JButton(START);
  private final CardLayout layoutSelection = new CardLayout();
  private final JPanel panelSelection = new JPanel(layoutSelection);
  private final JPanel panelEmpty = new JPanel();
  private final JPanel panelNextStep = new JPanel();
  private final JPanel panelStep = new JPanel();
  private final JPanel panelPredicate = new JPanel();
  private final ButtonGroup groupNextStep = new ButtonGroup();
  private final ButtonGroup groupStep = new ButtonGroup();
  private final ButtonGroup groupPredicate = new ButtonGroup();
  private final JRadioButton radioChild = new JRadioButton("Child");
  private final JRadioButton radioDescendant = new JRadioButton("Descendant");
  private final JRadioButton radioNoStep = new JRadioButton("Terminate");
  private final JRadioButton radioElement = new JRadioButton("Element name : ");
  private final JRadioButton radioText = new JRadioButton("Text node");
  private final JRadioButton radioAny = new JRadioButton("Any node");
  private final JRadioButton radioPredicate = new JRadioButton("Predicate : ");
  private final JRadioButton radioNoPredicate =
    new JRadioButton("No predicate");
  //private final JComboBox comboElement = new JComboBox();
  private final JTextField textFieldElement = new JTextField(20);
  private final JTextField textFieldPredicate = new JTextField(20);
  private static final int STATE_WIZZARD = 0;
  private static final int STATE_NEXTSTEP = 1;
  private static final int STATE_STEP = 2;
  private static final int STATE_PREDICATE = 3;
  private int state = STATE_WIZZARD;
  private String currentElementName = null;

  public DialogXPath(Frame parent, Options options, String rootElementName) {
    super(parent);
    this.options = options;
    this.rootElementName = rootElementName;
    setTitle(viewResource.getString(Label.EVALUATE));
    // XPath expression
    JPanel panelExpression = new JPanel(new BorderLayout());
    panelExpression.setBorder(new TitledBorder("XPath expression"));
    historyField = new HistoryField(options, Options.LASTXPATHEXPRESSION,
      oKActionListener);
    panelExpression.add(historyField.getComponent());
    panelExpression.add(new JLabel("context node = document element"),
      BorderLayout.SOUTH);
    contentPanel.add(panelExpression);
    // Selection panel
    JPanel panelWizzard = new JPanel(new BorderLayout());
    panelWizzard.setBorder(
      new TitledBorder(viewResource.getString(Label.LOCATION_PATH_WIZZARD)));
    panelWizzard.add(panelSelection);
    contentPanel.add(panelWizzard);
    // Empty panel
    panelSelection.add(panelEmpty, "Empty");
    // Next step panel
    groupNextStep.add(radioChild);
    groupNextStep.add(radioDescendant);
    groupNextStep.add(radioNoStep);
    panelNextStep.setLayout(new BorderLayout());
    JPanel boxNextStep = new JPanel();
    boxNextStep.setLayout(new BoxLayout(boxNextStep, BoxLayout.Y_AXIS));
    JPanel panelRadioChild = new JPanel(new BorderLayout());
    panelRadioChild.add(radioChild);
    boxNextStep.add(panelRadioChild);
    JPanel panelRadioDescendant = new JPanel(new BorderLayout());
    panelRadioDescendant.add(radioDescendant);
    boxNextStep.add(panelRadioDescendant);
    JPanel panelRadioNoStep = new JPanel(new BorderLayout());
    panelRadioNoStep.add(radioNoStep);
    boxNextStep.add(panelRadioNoStep);
    panelNextStep.add(boxNextStep, BorderLayout.NORTH);
    panelSelection.add(panelNextStep, "NextStep");
    // Step panel
    groupStep.add(radioElement);
    groupStep.add(radioText);
    groupStep.add(radioAny);
    panelStep.setLayout(new BorderLayout());
    JPanel boxStep = new JPanel();
    boxStep.setLayout(new BoxLayout(boxStep, BoxLayout.Y_AXIS));
    JPanel panelRadioElement = new JPanel(new BorderLayout());
    panelRadioElement.add(radioElement, BorderLayout.WEST);
    textFieldElement.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          element_update();
        };
        public void removeUpdate(DocumentEvent event) {
          element_update();
        };
        public void changedUpdate(DocumentEvent event) {
          element_update();
        };
      });
    panelRadioElement.add(textFieldElement);
    boxStep.add(panelRadioElement);
    JPanel panelRadioText = new JPanel(new BorderLayout());
    panelRadioText.add(radioText);
    boxStep.add(panelRadioText);
    JPanel panelRadioAny = new JPanel(new BorderLayout());
    panelRadioAny.add(radioAny);
    boxStep.add(panelRadioAny);
    panelStep.add(boxStep, BorderLayout.WEST);
    panelSelection.add(panelStep, "Step");
    // Predicate panel
    groupPredicate.add(radioPredicate);
    groupPredicate.add(radioNoPredicate);
    panelPredicate.setLayout(new BorderLayout());
    JPanel boxPredicate = new JPanel();
    boxPredicate.setLayout(new BoxLayout(boxPredicate, BoxLayout.Y_AXIS));
    JPanel panelPredicateValue = new JPanel(new BorderLayout());
    JPanel panelPredicateValueRadio = new JPanel(new BorderLayout());
    panelPredicateValueRadio.add(radioPredicate, BorderLayout.NORTH);
    panelPredicateValue.add(panelPredicateValueRadio, BorderLayout.WEST);
    JPanel panelPredicateValueHelp = new JPanel(new BorderLayout());
    textFieldPredicate.getDocument().addDocumentListener(
      new DocumentListener() {
        public void insertUpdate(DocumentEvent event) {
          predicate_update();
        };
        public void removeUpdate(DocumentEvent event) {
          predicate_update();
        };
        public void changedUpdate(DocumentEvent event) {
          predicate_update();
        };
      });
    panelPredicateValueHelp.add(textFieldPredicate);
    panelPredicateValueHelp.add(
      new JLabel("precedence : 0 \"\" () | * + - < > = != and or"),
      BorderLayout.SOUTH);
    panelPredicateValue.add(panelPredicateValueHelp);
    boxPredicate.add(panelPredicateValue);
    JPanel panelRadioNoPredicate = new JPanel(new BorderLayout());
    panelRadioNoPredicate.add(radioNoPredicate);
    boxPredicate.add(panelRadioNoPredicate);
    panelPredicate.add(boxPredicate, BorderLayout.NORTH);
    panelSelection.add(panelPredicate, "Predicate");
    // Wizzard button
    JPanel panelButton = new JPanel(new FlowLayout());
    buttonWizzard.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          wizzard_actionPerformed();
        }
      });
    panelButton.add(buttonWizzard);
    panelWizzard.add(panelButton, BorderLayout.SOUTH);
    // Pack
    pack();
  }

  protected void element_update() {
    radioElement.setSelected(true);
  }

  protected void predicate_update() {
    radioPredicate.setSelected(true);
  }

  protected void buttonOK_actionPerformed(ActionEvent event) {
    historyField.ok();
    super.buttonOK_actionPerformed(event);
  }

  private void wizzard_actionPerformed() {
    try {
      switch (state) {
        case STATE_WIZZARD:
          buttonOK.setEnabled(false);
          historyField.setEnabled(false);
          historyField.setText(".");
          currentElementName = rootElementName;
          buttonWizzard.setText(NEXT);
          layoutSelection.show(panelSelection, "NextStep");
          radioNoStep.setSelected(true);
          radioChild.setEnabled(true);
          radioDescendant.setEnabled(true);
          state = STATE_NEXTSTEP;
          break;
        case STATE_NEXTSTEP:
          if (radioNoStep.isSelected()) {
            historyField.setEnabled(true);
            buttonWizzard.setText(START);
            layoutSelection.show(panelSelection, "Empty");
            buttonOK.setEnabled(true);
            state = STATE_WIZZARD;
            break;
          }
          if (radioChild.isSelected())
            appendExpression("/");
          else if (radioDescendant.isSelected())
            appendExpression("//");
          layoutSelection.show(panelSelection, "Step");
          radioElement.setSelected(true);
          state = STATE_STEP;
          break;
        case STATE_STEP:
          currentElementName = null;
          if (radioElement.isSelected()) {
            currentElementName = textFieldElement.getText();
            appendExpression(currentElementName);
          }
          else if (radioText.isSelected()) {
            appendExpression("text()");
          }
          else if (radioAny.isSelected()) {
            appendExpression("node()");
          }
          layoutSelection.show(panelSelection, "Predicate");
          textFieldPredicate.setText("");
          radioNoPredicate.setSelected(true);
          state = STATE_PREDICATE;
          break;
        case STATE_PREDICATE:
          if (radioPredicate.isSelected()) {
            String predicate = textFieldPredicate.getText();
            if (predicate.length() != 0) {
              appendExpression("[" + predicate + "]");
              textFieldPredicate.setText("");
              radioNoPredicate.setSelected(true);
              break;
            }
          }
          layoutSelection.show(panelSelection, "NextStep");
          radioNoStep.setSelected(true);
          radioChild.setEnabled(currentElementName != null);
          radioDescendant.setEnabled(currentElementName != null);
          state = STATE_NEXTSTEP;
          break;
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  private void appendExpression(String text) {
    historyField.setText(historyField.getText() + text);
  }
}
