/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.CompoundBorder;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.status.ToolStatusBar;
import org.xmloperator.framework.control.global.GlobalControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.EditControl;

public class ToolStatusBarImpl extends JPanel implements ToolStatusBar {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected GlobalControl globalControl;
  protected EditControl editControl;
  protected FileControl fileControl;
  protected Options options;
  protected int currentEditStatus = EDIT_NONE;
  protected long lastMessageTimeMillis = 0;
  protected final JLabel champMessage = new JLabel(" ");
  protected final JComboBox comboBoxEdit = new JComboBox();
  protected final JCheckBox checkBoxLazy = new JCheckBox("Lazy");
  protected final JButton buttonCancel =
      new JButton(viewResource.getString(Label.CANCEL));
  protected final CardLayout layoutCommand = new CardLayout();
  protected final JPanel panelCommand = new JPanel(layoutCommand);
  protected boolean isEditStatusDisplaying = false;

  public ToolStatusBarImpl(FullyFledgedDocument fullyFledgedDocument,
    GlobalControl globalControl, EditControl editControl,
    FileControl fileControl, Options options) {
    super(new BorderLayout());
    this.globalControl = globalControl;
    this.editControl = editControl;
    this.fileControl = fileControl;
    this.options = options;
    // Panel tmoins
    final JPanel panelTemoins = new JPanel();
    panelTemoins.setLayout(new BoxLayout(panelTemoins, BoxLayout.X_AXIS));
    if (options.getThreeStateOptionValue(Options.STATUSSHOWN)
        == ThreeStateOption.TRUE) {
      JPanel panelTemoinModif = new JPanel(new BorderLayout());
      panelTemoinModif.setPreferredSize(new Dimension(70, 12));
      comboBoxEdit.setEnabled(false);
      comboBoxEdit.addActionListener(
        new ActionListener() {
          public void actionPerformed(ActionEvent event) {
            comboBoxEdit_actionPerformed();
          }
        });
      panelTemoinModif.add(comboBoxEdit);
      panelTemoins.add(panelTemoinModif);
    }
    if (options.getThreeStateOptionValue(Options.LAZYSHOWN)
        == ThreeStateOption.TRUE) {
      JPanel panelLazy = new JPanel(new BorderLayout());
      checkBoxLazy.setSelected(fullyFledgedDocument.isLazy());
      checkBoxLazy.addActionListener(
        new ActionListener() {
          public void actionPerformed(ActionEvent event) {
            checkBoxLazy_actionPerformed();
          }
        });
      panelLazy.add(checkBoxLazy);
      panelTemoins.add(panelLazy);
    }
    panelCommand.add(panelTemoins, "T");
    // Panel Cancel
    final JPanel panelCancel = new JPanel(new BorderLayout());
    panelCancel.setPreferredSize(new Dimension(100, 12));
    buttonCancel.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          buttonCancel_actionPerformed();
        }
      });
    panelCancel.add(buttonCancel);
    panelCommand.add(panelCancel, "C");
    this.add(panelCommand, BorderLayout.WEST);
    // Panel message
    JPanel panelMessage = new JPanel(new BorderLayout());
    panelMessage.setBorder(new CompoundBorder(
      new BevelBorder(BevelBorder.LOWERED), new EmptyBorder(0, 6, 0, 0)));
    panelMessage.add(champMessage);
    this.add(panelMessage);
  }

  protected void comboBoxEdit_actionPerformed() {
    try {
      if (isEditStatusDisplaying)
        return;
      String selectedItem = (String)comboBoxEdit.getSelectedItem();
      boolean isReadOnly =  selectedItem != null &&
        selectedItem.equals(viewResource.getString(Label.LECTURE_SEULE));
      editControl.setReadOnly(isReadOnly);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void checkBoxLazy_actionPerformed() {
    try {
      editControl.setLazy(checkBoxLazy.isSelected());
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  protected void buttonCancel_actionPerformed() {
    try {
      globalControl.getTaskControl().cancelTasks();
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
  }

  public void setMenuEnabled(boolean isEnabled) {
    this.layoutCommand.show(panelCommand, isEnabled ? "T" : "C");
  }

  ////////////////////////////////
  // MessageDisplay implementation
  ////////////////////////////////

  public void displayMessage(String message) {
    champMessage.setText(message);
    if (message.length() > 1)
      lastMessageTimeMillis = System.currentTimeMillis();
    Thread.yield();
  }

  //////////////////////////////////////////
  // DocumentModifiedListener implementation
  //////////////////////////////////////////

  public void notifyDocumentModified() {
    displayEditStatus(EDIT_MODIFIED);
  }

  ///////////////////////////////
  // ToolStatusBar implementation
  ///////////////////////////////

  public void clear() {
    displayEditStatus(EDIT_NONE);
    checkBoxLazy.setSelected(false);
    displayMessage(" ");
  }

  public String getMessage() {
    return champMessage.getText();
  }

  public long getLastMessageTimeMillis() {
    return lastMessageTimeMillis;
  }

  public void displayEditStatus(int editStatus) {
    if (editStatus == currentEditStatus)
      return;
    isEditStatusDisplaying = true;
    try {
      if (comboBoxEdit.getItemCount() != 0)
        comboBoxEdit.removeAllItems();
      switch (editStatus) {
        case EDIT_NONE:
          comboBoxEdit.setEnabled(false);
          break;
        case EDIT_READONLY_NOTSWITCHABLE:
          comboBoxEdit.addItem(
            viewResource.getString(Label.LECTURE_SEULE));
          comboBoxEdit.setEnabled(false);
          break;
        case EDIT_READONLY_SWITCHABLE:
          comboBoxEdit.addItem(
            viewResource.getString(Label.LECTURE_SEULE));
          comboBoxEdit.addItem(
            viewResource.getString(Label.READY));
          comboBoxEdit.setSelectedIndex(0);
          comboBoxEdit.setEnabled(true);
          break;
        case EDIT_READY:
          comboBoxEdit.addItem(
            viewResource.getString(Label.LECTURE_SEULE));
          comboBoxEdit.addItem(
            viewResource.getString(Label.READY));
          comboBoxEdit.setSelectedIndex(1);
          comboBoxEdit.setEnabled(true);
          break;
        case EDIT_MODIFIED:
          comboBoxEdit.addItem(
            viewResource.getString(Label.DOCUMENT_MODIFIE));
          comboBoxEdit.setEnabled(false);
          break;
        default:
          throw new IllegalArgumentException();
      }
      currentEditStatus = editStatus;
    }
    finally {
      isEditStatusDisplaying = false;
    }
  }

  public void displayLazy(boolean isLazy) {
    checkBoxLazy.setSelected(isLazy);
  }
}
