/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.util.exception.ExceptionControler;

public class Menu extends JMenu {
  protected final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  protected final Shortcuts shortcuts;

  protected Menu(String resourceKey, Shortcuts shortcuts) {
    this.shortcuts = shortcuts;
    setText(this.getString(resourceKey));
    setMnemonic(this.getString(Label.MNEMO + resourceKey).charAt(0));
  }

  protected final String getString(String resourceKey) {
    return this.viewResource.getString(resourceKey);
  }

  protected final JMenuItem createMenuItem(String resourceKey, Action action) {
    return this.createMenuItem(resourceKey, null, false, action);
  }

  protected final JMenuItem createMenuItem(
      String resourceKey, String shortcutId, Action action) {
    return this.createMenuItem(resourceKey, shortcutId, false, action);
  }

  protected final JMenuItem createMenuItem(
      String resourceKey, String shortcutId, boolean isContinued,
      Action action) {
    ActionListener actionListener = ExceptionControler.control(action);
    JMenuItem menuItem = new JMenuItem(viewResource.getString(resourceKey)
      + (isContinued ? "..." : ""));
    menuItem.setMnemonic(
      viewResource.getString(Label.MNEMO + resourceKey).charAt(0));
    menuItem.addActionListener(actionListener);
    if (shortcutId != null) {
      Shortcut shortcut = shortcuts.getShortcut(shortcutId);
      shortcut.setActionListener(actionListener);
      menuItem.setAccelerator(shortcut.getKeyStroke());
    }
    return menuItem;
  }

  protected final JMenu createMenu(String resourceKey) {
    JMenu menu = new JMenu(viewResource.getString(resourceKey));
    menu.setMnemonic(
        viewResource.getString(Label.MNEMO + resourceKey).charAt(0));
    return menu;
  }
}
