/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.view.instance.ToolView;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.view.main.menu.Menu;

public final class MenuMove extends Menu {
  private static final int GOTO_PREVIOUS = 0;
  private static final int GOTO_NEXT = 1;
  private static final int GOTO_CHILD = 2;
  private static final int GOTO_PARENT = 3;
  private EditControl editControl;
  private DocumentViewControl documentViewControl;
  private ToolView toolView;

  public MenuMove(
      ToolView toolView, EditControl editControl,
      DocumentViewControl documentViewControl, Shortcuts shortcuts) {
    super(Label.MOVE, shortcuts);
    this.toolView = toolView;
    this.editControl = editControl;
    this.documentViewControl = documentViewControl;
    // Dplacements
    this.add(this.createMenuFairePrecedent());
    this.add(this.createMenuFaireSuivant());
    this.add(this.createMenuFaireDescendant());
    this.add(this.createMenuFaireParent());
    this.addSeparator();
    this.add(this.createMenuToBeMoved());
    this.addSeparator();
    // Slections relatives
    shortcuts.getShortcut(Shortcuts.SHORTCUT_SELECTIONNER_PRECEDENT).
        setActionListener(ExceptionControler.control(new Action() {
      public void execute(ActionEvent event) throws Exception {
        selectionner_execute(GOTO_PREVIOUS);
      }
    }));
    shortcuts.getShortcut(Shortcuts.SHORTCUT_SELECTIONNER_SUIVANT).
        setActionListener(ExceptionControler.control(new Action()  {
      public void execute(ActionEvent event) throws Exception {
        selectionner_execute(GOTO_NEXT);
      }
    }));
    shortcuts.getShortcut(Shortcuts.SHORTCUT_SELECTIONNER_DESCENDANT).
        setActionListener(ExceptionControler.control(new Action()  {
      public void execute(ActionEvent event) throws Exception {
        selectionner_execute(GOTO_CHILD);
      }
    }));
    shortcuts.getShortcut(Shortcuts.SHORTCUT_SELECTIONNER_PARENT).
        setActionListener(ExceptionControler.control(new Action()  {
      public void execute(ActionEvent event) throws Exception {
        selectionner_execute(GOTO_PARENT);
      }
    }));
    shortcuts.getShortcut(Shortcuts.SHORTCUT_MONTER).
        setActionListener(ExceptionControler.control(new Action()  {
      public void execute(ActionEvent event) throws Exception {
        monter_execute();
      }
    }));
    shortcuts.getShortcut(Shortcuts.SHORTCUT_DESCENDRE).
        setActionListener(ExceptionControler.control(new Action()  {
      public void execute(ActionEvent event) throws Exception {
        descendre_execute();
      }
    }));
    // Reorder
    this.add(this.createMenuReorderAscending());
    this.add(this.createMenuReorderDescending());
  }

  public JMenuItem createMenuFairePrecedent() {
    return this.createMenuItem(
        Label.PASSER_PRECEDENT, Shortcuts.SHORTCUT_PASSER_PRECEDENT,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        deplacer_execute(EditControl.MOVE_PREVIOUS);
      }
    });
  }

  public JMenuItem createMenuFaireSuivant() {
    return this.createMenuItem(
        Label.PASSER_SUIVANT, Shortcuts.SHORTCUT_PASSER_SUIVANT,
        new Action() {
      public void execute(ActionEvent event) throws Exception {
        deplacer_execute(EditControl.MOVE_NEXT);
      }
    });
  }

  public JMenuItem createMenuFaireDescendant() {
    return this.createMenuItem(
        Label.PASSER_DESCENDANT, Shortcuts.SHORTCUT_PASSER_DESCENDANT,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        deplacer_execute(EditControl.MOVE_CHILD);
      }
    });
  }

  public JMenuItem createMenuFaireParent() {
    return this.createMenuItem(
        Label.PASSER_PARENT, Shortcuts.SHORTCUT_PASSER_PARENT,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        deplacer_execute(EditControl.MOVE_PARENT);
      }
    });
  }

  public JMenuItem createMenuToBeMoved() {
    return this.createMenuItem(
        Label.TO_BE_MOVED, Shortcuts.SHORTCUT_TO_BE_MOVED,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        toBeMoved_execute();
      }
    });
  }

  public JMenuItem createMenuReorderAscending() {
    return this.createMenuItem(
        Label.REORDER_ASCENDING, null,
        new Action()  {
        public void execute(ActionEvent event) throws Exception {
          reorder_execute(false);
        }
      });
  }

  public JMenuItem createMenuReorderDescending() {
    return this.createMenuItem(
        Label.REORDER_DESCENDING, null,
        new Action()  {
        public void execute(ActionEvent event) throws Exception {
          reorder_execute(true);
        }
      });
  }

  private void deplacer_execute(int moveType) {
    editControl.moveNodeInterval(null, moveType);
  }

  private void toBeMoved_execute() {
    editControl.storeSelectedNodesForMove();
  }

  private void reorder_execute(boolean isReverse) {
    editControl.reorderNodeInterval(null, null, isReverse);
  }

  private void selectionner_execute(int gotoType) {
    Node node = toolView.getDocumentTreeView().getSelectedNode();
    if (node == null)
      return;
    switch (gotoType) {
      case GOTO_PREVIOUS:
        node = node.getPreviousSibling();
        break;
      case GOTO_NEXT:
        node = node.getNextSibling();
        break;
      case GOTO_CHILD:
        node = node.getFirstChild();
        break;
      case GOTO_PARENT:
        node = node.getParentNode();
        break;
      default:
        return;
    }
    if (node != null)
      documentViewControl.gotoNode(node);
  }

  private void monter_execute() {
    toolView.getDocumentTreeView().selectPreviousRow();
  }

  private void descendre_execute() {
    toolView.getDocumentTreeView().selectNextRow();
  }
}
