/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.main.menu;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.util.ObserverView;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.view.observer.TextFileEditor;
import org.xmloperator.framework.view.tree.DocumentTreeView;
import org.xmloperator.framework.view.shortcut.Shortcut;
import org.xmloperator.framework.view.shortcut.Shortcuts;
import org.xmloperator.framework.control.task.Action;
import org.xmloperator.framework.control.global.GlobalViewControl;
import org.xmloperator.framework.control.instance.ViewControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.view.stat.DialogElementsStatistic;
import org.xmloperator.view.main.menu.Menu;

public final class MenuView extends Menu {
  private Options options;
  private ViewControl viewControl;
  private DocumentViewControl documentViewControl;
  private GlobalViewControl globalViewControl;
  private FullyFledgedDocument fullyFledgedDocument;

  public MenuView(
      FullyFledgedDocument fullyFledgedDocument,
      Options options, ViewControl viewControl,
      DocumentViewControl documentViewControl,
      GlobalViewControl globalViewControl, Shortcuts shortcuts) {
    super(Label.VOIR, shortcuts);
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.options = options;
    this.viewControl = viewControl;
    this.documentViewControl = documentViewControl;
    this.globalViewControl = globalViewControl;
    boolean isAdvancedUse =
      options.getThreeStateOptionValue(Options.ADVANCEDUSE)
        == ThreeStateOption.TRUE;
    // Ouverture de fentres
    this.add(this.createMenuItem(
        Label.BROWSING, Shortcuts.SHORTCUT_NAVIGATION, new Action() {
      public void execute(ActionEvent event) throws Exception {
        seeNavigation_execute();
      }
    }));
    if (isAdvancedUse)
      this.add(this.createMenuItem(
          Label.OPERATIONS, Shortcuts.SHORTCUT_OPERATIONS,
          new Action() {
        public void execute(ActionEvent event) throws Exception {
          seeOperations_execute();
        }
      }));
    this.add(this.createMenuItem(
        Label.FRAGMENT, Shortcuts.SHORTCUT_FRAGMENT, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        seeFragment_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.CONTENT_MODEL, Shortcuts.SHORTCUT_CONTENT_MODEL,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        seeContentModel_execute();
      }
    }));
    addSeparator();
    this.add(this.createMenuItem(
        Label.ELEMENT_LABELLING, Shortcuts.SHORTCUT_CHANGE_ELEMENT_LABELLING,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        elementLabelling_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.ELEMENTS_REFERENCANT, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        seeReferencers_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.INVALID_ELEMENT, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        searchInvalidContent_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.ERREURS, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        seeErrors_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.ELEMENT_AT_LOCATION, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        elementAtLocation_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.EXCEPTION_TRACE, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        seeException_execute();
      }
    }));
    addSeparator();
    // Recherche
    this.add(this.createMenuItem(
        Label.CHERCHER, Shortcuts.SHORTCUT_CHERCHER, true, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        search_execute();
      }
    }));
    this.add(this.createMenuItem(
        Label.EVALUATE, Shortcuts.SHORTCUT_EVALUATE, true, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        evaluate_execute();
      }
    }));
    // Stats
    this.add(this.createMenuItem(
        Label.STATISTIQUE, Shortcuts.SHORTCUT_STATISTIQUE, true, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        statistic_execute();
      }
    }));
    addSeparator();
    // Expansion
    this.add(this.createMenuItem(
        Label.EXPANSER, Shortcuts.SHORTCUT_EXPANSER, false, new Action()  {
      public void execute(ActionEvent event) throws Exception {
        expandNodes_execute();
      }
    }));
    // Change layout
    this.add(this.createMenuItem(
        Label.CHANGE_LAYOUT, Shortcuts.SHORTCUT_RECONSTRUIRE_AUTREMENT,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        changeOrientation_execute();
      }
    }));
    // Reconstruction
    this.add(this.createMenuItem(
        Label.RECONSTRUIRE, Shortcuts.SHORTCUT_RECONSTRUIRE_TEL_QUEL,
        new Action()  {
      public void execute(ActionEvent event) throws Exception {
        rebuildUserInterface_execute();
      }
    }));
  }

  private void search_execute() {
    viewControl.searchText();
  }

  private void evaluate_execute() {
    viewControl.evaluateXPathExpression();
  }

  private void statistic_execute() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    DialogElementsStatistic dialogueStat = new DialogElementsStatistic(null,
        document.getDocumentElement(), options);
    dialogueStat.setTitle(
        viewResource.getString(Label.STATISTIQUE));
    ViewUtils.moveComponentToCentre(dialogueStat);
    dialogueStat.show();
  }

  private void expandNodes_execute() {
    this.documentViewControl.expandNodes(null);
  }

  private void seeNavigation_execute() {
    viewControl.showNodeStackView();
  }

  private void seeOperations_execute() {
    viewControl.showOperationStackView();
  }

  private void seeFragment_execute() {
    viewControl.showDocumentFragmentView();
  }

  private void seeContentModel_execute() {
    viewControl.createContentModelView().show();
    documentViewControl.displaySelectedNode();
  }

  private void elementLabelling_execute() {
    this.viewControl.changeElementKey(null);
  }

  private void seeReferencers_execute() {
    viewControl.showSelectedNodeReferencersView();
  }

  private void searchInvalidContent_execute() {
    viewControl.searchInvalidContent();
  }

  private void seeErrors_execute() {
    viewControl.showParseErrorListView();
  }

  private void elementAtLocation_execute() {
    viewControl.showElementAt(0, 0);
  }

  private void seeException_execute() {
    globalViewControl.showExceptionTrace();
  }

  private void rebuildUserInterface_execute() {
    viewControl.rebuildUserInterface(null);
  }

  private void changeOrientation_execute() {
    viewControl.changeOrientation();
  }
}
