/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.model;

import java.util.List;
import java.util.ArrayList;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.util.exception.ExceptionControler;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.tree.DomTree;
import org.xmloperator.view.model.DomTreeNode;

public class DomTreeModel implements DomTree, TreeModel {
  protected final FullyFledgedDocument fullyFledgedDocument;
  protected final Object root = new DomTreeNode();
    // TreeModel requires a root node != null
  protected final Object nullChild = new DomTreeNode();
    // TreeModel requires child nodes != null
  protected DomTreeNode rootDomTreeNode = null;
    // Alone root child if !isRootNodeTreeNode
    // Maps to root if isRootNodeTreeNode
  protected boolean isRootNodeRootTreeNode = false;
  protected List listenerList = new ArrayList();

  public DomTreeModel(FullyFledgedDocument fullyFledgedDocument) {
    if (fullyFledgedDocument == null)
      throw new IllegalArgumentException();
    this.fullyFledgedDocument = fullyFledgedDocument;
  }

  protected static class DomTreePath extends TreePath {
    public DomTreePath(TreePath parent, Object lastElement) {
      super(parent, lastElement);
    }
  }

  public TreePath getTreePath(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    return getTreePath(domTreeNode(node));
  }

  protected TreePath getTreePath(Object lastTreeNode) {
    if (lastTreeNode == null || !(lastTreeNode instanceof DomTreeNode))
      throw new IllegalArgumentException();
    if (lastTreeNode == root
        || (lastTreeNode == rootDomTreeNode && isRootNodeRootTreeNode))
      return new TreePath(root);
    Object parentTreeNode;
    if (lastTreeNode == rootDomTreeNode)
      parentTreeNode = root;
    else {
      Node node = ((DomTreeNode)lastTreeNode).getNode();
      Node parentNode = node.getParentNode();
      if (parentNode == null)
        throw new RuntimeException(lastTreeNode + " has no parent");
      parentTreeNode = domTreeNode(parentNode);
    }
    return new DomTreePath(getTreePath(parentTreeNode), lastTreeNode);
  }

  protected DomTreeNode domTreeNode(Node node) {
    return DomTreeNode.domTreeNodeFromNode(
        node,
        this.fullyFledgedDocument.getNodeLinker(),
        this.fullyFledgedDocument.getElementKeyProvider());
  }

  /////////////////////////
  // DomTree implementation
  /////////////////////////

  public void setRootNode(Node rootNode, boolean isRootNodeRootTreeNode) {
    this.rootDomTreeNode = domTreeNode(rootNode);
    this.isRootNodeRootTreeNode = isRootNodeRootTreeNode;
    structureChanged();
  }

  public Node getRootNode() {
    return rootDomTreeNode.getNode();
  }

  public boolean isRootNodeRootTreeNode() {
    return isRootNodeRootTreeNode;
  }

  public TreeModelEvent createTreeModelEvent(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    Node parentNode = node.getParentNode();
    if (parentNode == null)
      throw new IllegalArgumentException();
    return new TreeModelEvent(
        this, getTreePath(parentNode), new int[] {getNodeIndex(node)},
        new Object[] {domTreeNode(node)});
  }

  public TreeModelEvent createTreeModelEvent(Node[] nodes) {
    if (nodes == null || nodes.length == 0 || nodes[0] == null)
      throw new IllegalArgumentException();
    Node parentNode = nodes[0].getParentNode();
    if (parentNode == null)
      throw new IllegalArgumentException();
    for (int k = 1; k < nodes.length; k++) {
      Node node = nodes[k];
      if (node == null || node.getParentNode() != parentNode)
        throw new IllegalArgumentException();
    }
    int[] childIndices = new int[nodes.length];
    Object[] children = new DomTreeNode[nodes.length];
    for (int k = 0; k < nodes.length; k++) {
      Node node = nodes[k];
      childIndices[k] = getNodeIndex(node);
      children[k] = domTreeNode(node);
    }
    return new TreeModelEvent(
        this, getTreePath(parentNode), childIndices, children);
  }

  public void fireTreeNodesChanged(TreeModelEvent nodesChangedEvent) {
    for (int k = 0; k < listenerList.size(); k++)
      ((TreeModelListener)listenerList.get(k)).
        treeNodesChanged(nodesChangedEvent);
  }

  public void fireTreeNodesInserted(TreeModelEvent nodesInsertedEvent) {
    for (int k = 0; k < listenerList.size(); k++)
      ((TreeModelListener)listenerList.get(k)).
        treeNodesInserted(nodesInsertedEvent);
  }

  public void fireTreeNodesRemoved(TreeModelEvent nodesRemovedEvent) {
    for (int k = 0; k < listenerList.size(); k++)
      ((TreeModelListener)listenerList.get(k)).
        treeNodesRemoved(nodesRemovedEvent);
  }

  protected void fireTreeStructureChanged(Object treeNode) {
    TreeModelEvent treeStructureChangedEvent =
      new TreeModelEvent(this, getTreePath(treeNode));
    for (int k = 0; k < listenerList.size(); k++)
      ((TreeModelListener)listenerList.get(k)).
        treeStructureChanged(treeStructureChangedEvent);
  }

  public void nodeStructureChanged(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    fireTreeStructureChanged(domTreeNode(node));
  }

  public void structureChanged() {
    fireTreeStructureChanged(root);
  }

  public void nodeChanged(Node node) {
    fireTreeNodesChanged(createTreeModelEvent(node));
  }

  public void nodeInserted(Node node) {
    fireTreeNodesInserted(createTreeModelEvent(node));
  }

  public void nodesInserted(Node[] nodes) {
    fireTreeNodesInserted(createTreeModelEvent(nodes));
  }

  ///////////////////////////
  // TreeModel implementation
  ///////////////////////////

  public Object getRoot() {
    return root;
  }

  public Object getChild(Object parent, int index) {
    try {
      if (parent == null || !(parent instanceof DomTreeNode))
        throw new IllegalArgumentException();
      if (parent == root)
        if (isRootNodeRootTreeNode)
          parent = rootDomTreeNode;
        else
          return index == 0 ? rootDomTreeNode : nullChild;
      Node parentNode = ((DomTreeNode)parent).getNode();
      Node childNode = parentNode.getFirstChild();
      while (childNode != null) {
        if (index == 0) {
          DomTreeNode domTreeNode = domTreeNode(childNode);
          return domTreeNode != null ? domTreeNode : nullChild;
        }
        childNode = childNode.getNextSibling();
        index--;
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
    }
    return nullChild;
  }

  public int getChildCount(Object parent) {
    try {
      if (parent == null || !(parent instanceof DomTreeNode))
        throw new IllegalArgumentException();
      if (parent == root)
        if (isRootNodeRootTreeNode)
          if (rootDomTreeNode != null)
            parent = rootDomTreeNode;
          else
            return 0;
        else
          return rootDomTreeNode != null ? 1 : 0;
      Node parentNode = ((DomTreeNode)parent).getNode();
      return getChildCount(parentNode);
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return 0;
    }
  }

  protected static final int getChildCount(Node parent) {
    int count = 0;
    Node childNode = parent.getFirstChild();
    while (childNode != null) {
      count++;
      childNode = childNode.getNextSibling();
    }
    return count;
  }

  public boolean isLeaf(Object treeNode) {
    try {
      if (treeNode == null || !(treeNode instanceof DomTreeNode))
        throw new IllegalArgumentException();
      if (treeNode == root)
        return false;
      Node node = ((DomTreeNode)treeNode).getNode();
      switch (node.getNodeType()) {
        case Node.DOCUMENT_NODE:
          return false;
        case Node.ELEMENT_NODE:
          return !node.hasChildNodes();
        default:
          return true;
      }
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return false;
    }
  }

  public void valueForPathChanged(TreePath path, Object newValue) {
    // Not used
  }

  public int getIndexOfChild(Object parent, Object child) {
    try {
      if (parent == null || !(parent instanceof DomTreeNode)
        || child == null || !(child instanceof DomTreeNode))
        throw new IllegalArgumentException();
      if (parent == root)
        if (isRootNodeRootTreeNode)
          parent = rootDomTreeNode;
        else
          return child == rootDomTreeNode ? 0 : -1;
      Node parentNode = ((DomTreeNode)parent).getNode();
      Node childNode = ((DomTreeNode)child).getNode();
      return childNode.getParentNode() == parentNode ?
        getNodeIndex(childNode) : -1;
    }
    catch (Exception exception) {
      ExceptionControler.printStackTrace(exception);
      return -1;
    }
  }

  protected static final int getNodeIndex(Node node) {
    int index = 0;
    while (true) {
      node = node.getPreviousSibling();
      if (node == null)
        return index;
      index++;
    }
  }

  public void addTreeModelListener(TreeModelListener listener) {
    listenerList.add(listener);
  }

  public void removeTreeModelListener(TreeModelListener listener) {
    listenerList.remove(listener);
  }
}
