/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.util.ResourceBundle;
import java.awt.Font;
import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.util.view.ViewUtils;

public final class PanelDocumentType extends JPanel {
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private final JPanel panelDTDPublicId = new JPanel(new BorderLayout());
  private final JLabel labelDTDPublicId = new JLabel("Public Id : ");
  private final JTextField fieldDTDPublicId = new JTextField();
  private final JPanel panelDTDSystemId = new JPanel(new BorderLayout());
  private final JLabel labelDTDSystemId = new JLabel("System Id : ");
  private final JTextField fieldDTDSystemId = new JTextField();

  public PanelDocumentType(FullyFledgedDocument fullyFledgedDocument,
    Options options, boolean isLandscape) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    EmptyBorder thinEmptyBorder = new EmptyBorder(2, 4, 2, 4);
    JPanel voletNord = new JPanel();
    voletNord.setLayout(new BoxLayout(voletNord, BoxLayout.Y_AXIS));
    this.add(voletNord, BorderLayout.NORTH);
    Font font = options.getThreeStateOptionValue(Options.FONT)
      == ThreeStateOption.TRUE ? Constants.FONT_LARGE : Constants.FONT_NORMAL;
    // DTD Public Id
    panelDTDPublicId.setBorder(thinEmptyBorder);
    panelDTDPublicId.add(labelDTDPublicId,
                         isLandscape ? BorderLayout.WEST : BorderLayout.NORTH);
    fieldDTDPublicId.setFont(font);
    fieldDTDPublicId.setEnabled(false);
    panelDTDPublicId.add(fieldDTDPublicId);
    voletNord.add(panelDTDPublicId);
    // DTD System Id
    panelDTDSystemId.setBorder(thinEmptyBorder);
    panelDTDSystemId.add(labelDTDSystemId,
                         isLandscape ? BorderLayout.WEST : BorderLayout.NORTH);
    fieldDTDSystemId.setFont(font);
    fieldDTDSystemId.setEnabled(false);
    panelDTDSystemId.add(fieldDTDSystemId);
    voletNord.add(panelDTDSystemId);
  }

  public void setLayout(boolean isLandscape) {
    ViewUtils.setBorderLayout(
        this.panelDTDPublicId, this.labelDTDPublicId, isLandscape);
    ViewUtils.setBorderLayout(
        this.panelDTDSystemId, this.labelDTDSystemId, isLandscape);
  }

  public void display() {
    Document document = fullyFledgedDocument.getDocument();
    if (document == null)
      return;
    DocumentType documentType = document.getDoctype();
    fieldDTDPublicId.setText(documentType != null ?
      documentType.getPublicId() : "");
    fieldDTDSystemId.setText(documentType != null ?
      documentType.getSystemId() : "");
  }
}
