/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Component;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPanel;
import javax.swing.BoxLayout;

import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.id.IdManagerRead;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.node.NodeDetailView;
import org.xmloperator.framework.view.node.AttributeSelectionListener;
import org.xmloperator.framework.view.node.TextSelection;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.FileControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;
import org.xmloperator.util.view.ViewUtils;
import org.xmloperator.view.node.PanelNodeType;
import org.xmloperator.view.node.PanelDocument;
import org.xmloperator.view.node.PanelDocumentType;
import org.xmloperator.view.node.PanelElementName;
import org.xmloperator.view.node.PanelAttributes;
import org.xmloperator.view.node.PanelTextNode;
import org.xmloperator.view.node.PanelEntityReference;
import org.xmloperator.view.node.PanelPITarget;
import org.xmloperator.view.node.PanelMoveCancel;
import org.xmloperator.view.node.PanelNodeInterval;
import org.xmloperator.view.main.menu.MenuEdit;
import org.xmloperator.framework.view.instance.IdsDialogAccess;

import org.w3c.dom.Node;
import org.w3c.dom.EntityReference;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

public final class PanelNodeDetail extends JPanel
    implements NodeDetailView, AttributeSelectionListener {
  private FullyFledgedDocument fullyFledgedDocument;
  private final JPanel panelA = new JPanel(new BorderLayout());
  private final CardLayout layout_A1 = new CardLayout();
  private final CardLayout layout_A2 = new CardLayout();
  private final CardLayout layout_B = new CardLayout();
  private JPanel panelA1;
  private JPanel panelA2;
  private JPanel panelB;
  private JPanel panelViergeA1;
  private JPanel panelViergeA2;
  private JPanel panelViergeB;
  private PanelNodeType panelNodeType;
  private PanelDocument panelDocument;
  private PanelDocumentType panelDocumentType;
  private PanelElementName panelElementName;
  private PanelAttributes panelAttributes;
  private PanelTextNode panelTextNode;
  private PanelEntityReference panelEntityReference;
  private PanelPITarget panelPITarget;
  private PanelMoveCancel panelMoveCancel;
  private PanelNodeInterval panelNodeInterval;
  private Node currentNode = null;
  private String selectedAttributeName = null;

  public PanelNodeDetail(Frame frame, MenuEdit menuEdit,
    FullyFledgedDocument fullyFledgedDocument,
    Options options, EditControl editControl, FileControl fileControl,
    DocumentViewControl documentViewControl, IdsDialogAccess idsDialogAccess,
    boolean isLandscape) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    // Panel A1
    this.panelA1 = new JPanel(layout_A1);
    this.panelViergeA1 = new JPanel();
    this.panelNodeType = new PanelNodeType(fullyFledgedDocument,
        editControl, documentViewControl, options);
    this.panelMoveCancel = new PanelMoveCancel(editControl, options);
    panelA1.add(panelViergeA1, "Blank");
    panelA1.add(panelNodeType, "NodeType");
    panelA1.add(panelMoveCancel, "Move");
    // Panel A2
    this.panelA2 = new JPanel(layout_A2);
    this.panelViergeA2 = new JPanel();
    this.panelElementName = new PanelElementName(fullyFledgedDocument,
        (AttributeSelectionListener)this, options);
    this.panelPITarget =
      new PanelPITarget(fullyFledgedDocument, editControl, options);
    panelA2.add(panelViergeA2, "Blank");
    panelA2.add(panelElementName, "Element");
    panelA2.add(panelPITarget, "PI");
    // Panel A
    JPanel panelAN = new JPanel();
    panelAN.setLayout(new BoxLayout(panelAN, BoxLayout.Y_AXIS));
    panelAN.add(panelA1);
    panelAN.add(panelA2);
    panelA.add(panelAN, BorderLayout.NORTH);
    this.add(panelA, isLandscape ? BorderLayout.WEST : BorderLayout.NORTH);
    // Panel B
    this.panelB = new JPanel(layout_B);
    this.panelViergeB = new JPanel();
    this.panelDocument = new PanelDocument(fullyFledgedDocument,
      editControl, fileControl, options, isLandscape);
    this.panelDocumentType = new PanelDocumentType(
      fullyFledgedDocument, options, isLandscape);
    this.panelAttributes =
      new PanelAttributes(frame, menuEdit, fullyFledgedDocument,
      editControl, documentViewControl, idsDialogAccess, options,
      (AttributeSelectionListener)this);
    this.panelTextNode =
        new PanelTextNode(fullyFledgedDocument, editControl, options);
    this.panelEntityReference =
      new PanelEntityReference(fullyFledgedDocument, editControl, options);
    this.panelNodeInterval =
        new PanelNodeInterval(options);
    panelB.add(panelViergeB, "Blank");
    panelB.add(panelDocument, "Document");
    panelB.add(panelDocumentType, "DocType");
    panelB.add(panelAttributes, "Element");
    panelB.add(panelTextNode, "TextNode");
    panelB.add(panelEntityReference, "EntityReference");
    panelB.add(panelNodeInterval, "Move");
    this.add(panelB);
  }

  public void setLayout(boolean isLandscape) {
    ViewUtils.setBorderLayout(this, panelA, isLandscape);
    this.panelDocument.setLayout(isLandscape);
    this.panelDocumentType.setLayout(isLandscape);
  }

  ////////////////////////////////////////////
  // AttributeSelectionListener implementation
  ////////////////////////////////////////////

  public void selectAttribute(String attributeName) {
    this.selectedAttributeName = attributeName;
    this.setNode(currentNode);
  }

  ////////////////////////////////
  // NodeDetailView implementation
  ////////////////////////////////

  public void setNode(Node node) {
    if (fullyFledgedDocument.getDocument() == null)
      node = null;
    currentNode = node;
    String component_A1_name = "Blank";
    String component_A2_name = "Blank";
    String component_B_name = "Blank";
    if (node != null) {
      component_A1_name = "NodeType";
      panelNodeType.display(node);
      switch (node.getNodeType()) {
        case Node.DOCUMENT_NODE:
          component_B_name = "Document";
          panelDocument.display();
          break;
        case Node.DOCUMENT_TYPE_NODE: {
          component_B_name = "DocType";
          panelDocumentType.display();
          break;
        }
        case Node.ELEMENT_NODE: {
          component_A2_name = "Element";
          panelElementName.display((Element)node, selectedAttributeName);
          if (selectedAttributeName == null) {
            component_B_name = "Element";
            panelAttributes.display((Element)node);
          }
          else if (((Element)node).getAttributeNode(selectedAttributeName)
                   != null) {
            component_B_name = "TextNode";
            panelTextNode.display(
                ((Element)node).getAttributeNode(selectedAttributeName));
          }
          break;
        }
        case Node.TEXT_NODE:
        case Node.CDATA_SECTION_NODE:
        case Node.COMMENT_NODE: {
          component_B_name = "TextNode";
          panelTextNode.display(node);
          break;
        }
        case Node.ENTITY_REFERENCE_NODE: {
          component_B_name = "EntityReference";
          panelEntityReference.display((EntityReference)node);
          break;
        }
        case Node.PROCESSING_INSTRUCTION_NODE: {
          component_A2_name = "PI";
          component_B_name = "TextNode";
          panelPITarget.display((ProcessingInstruction)node);
          panelTextNode.display(node);
          break;
        }
      }
    }
    layout_A1.show(panelA1, component_A1_name);
    layout_A2.show(panelA2, component_A2_name);
    layout_B.show(panelB, component_B_name);
  }

  public void setNodeInterval(Node firstNode, Node lastNode) {
    if (fullyFledgedDocument.getDocument() == null) {
      firstNode = null;
      lastNode = null;
    }
    boolean isInterval = firstNode != null && lastNode != null;
    String componentName = isInterval ? "Move" : "Blank";
    if (isInterval)
      panelNodeInterval.display(firstNode, lastNode);
    layout_A1.show(panelA1, componentName);
    layout_A2.show(panelA2, "Blank");
    layout_B.show(panelB, componentName);
  }

  public Node getNode() {
    return currentNode;
  }

  public Attr getCurrentAttribute() {
    return panelAttributes.getCurrentAttribute();
  }

  public TextSelection getTextSelection() {
    return (TextSelection)panelTextNode;
  }

  public void selectLastSearchNextInstance() {
    if (currentNode == null)
      return;
    if (currentNode.getNodeType() == Node.ELEMENT_NODE)
      panelAttributes.selectLastSearchNextInstance();
    else
      panelTextNode.selectLastSearchNextInstance();
  }

  public void refresh() {
    panelNodeType.refresh();
  }
}
