/*
 * The_xmloperator_project Software License, Version 1.7
 *
 * Copyright (c) 2000 - 2003 The_xmloperator_project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes or uses software developped
 *       by The_xmloperator_project (http://www.xmloperator.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. Products derived from this software may not be called "xmloperator",
 *    nor may "xmloperator" appear in their name, without prior written
 *    permission. For written permission, please contact
 *    the xmloperator project administrator.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE_XMLOPERATOR_PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * Further information can be found on the project's site
 * (http://www.xmloperator.net/).
 */
package org.xmloperator.view.node;

import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.Collection;
import java.util.List;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;

import org.xmloperator.framework.option.Constants;
import org.xmloperator.framework.option.ThreeStateOption;
import org.xmloperator.framework.option.Options;
import org.xmloperator.framework.document.manage.DocumentUtil;
import org.xmloperator.framework.document.namespace.UndefinedNamespaceException;
import org.xmloperator.framework.document.FullyFledgedDocument;
import org.xmloperator.framework.view.resource.Label;
import org.xmloperator.framework.control.instance.EditControl;
import org.xmloperator.framework.control.instance.DocumentViewControl;

public final class PanelNodeType extends JPanel {
  private static final String DOCUMENT = "DOCUMENT";
  private static final String DOCUMENT_TYPE = "DOC. TYPE";
  private static final String ELEMENT = "ELEMENT";
  private static final String TEXT = "TEXT";
  private static final String CDATA_SECTION = "CDATA SECT.";
  private static final String COMMENT = "COMMENT";
  private static final String ENTITY_REFERENCE = "ENTITY REF.";
  private static final String PROCESSING_INSTRUCTION = "PROC. INSTR.";
  private static final Border borderLigne = new EmptyBorder(4, 4, 4, 4);
  private final ResourceBundle viewResource =
    ResourceBundle.getBundle(Label.CLASS);
  private FullyFledgedDocument fullyFledgedDocument;
  private EditControl editControl;
  private Options options;
  private boolean isDisplaying = false;
  private final JComboBox comboBoxType = new JComboBox();
  private String displayedTypeName = null;

  public PanelNodeType(FullyFledgedDocument fullyFledgedDocument,
    EditControl editControl, DocumentViewControl documentViewControl,
    Options options) {
    super(new BorderLayout());
    this.fullyFledgedDocument = fullyFledgedDocument;
    this.editControl = editControl;
    this.options = options;
    this.setBorder(borderLigne);
    comboBoxType.setFont(Constants.FONT_NORMAL);
    comboBoxType.addActionListener(
      new ActionListener() {
        public void actionPerformed(ActionEvent event) {
          comboBoxType_actionPerformed();
        }
      });
    this.add(comboBoxType);
  }

  private void comboBoxType_actionPerformed() {
    if (isDisplaying)
      return;
    String typeName = (String)comboBoxType.getSelectedItem();
    int nodeType = getNodeType(typeName);
    editControl.replaceNode(null, nodeType, null);
  }

  public void display(Node node) {
    if (node == null)
      throw new IllegalArgumentException();
    isDisplaying = true;
    try {
      if (comboBoxType.getItemCount() != 0)
        comboBoxType.removeAllItems();
      int nodeType = node.getNodeType();
      Node parentNode = node.getParentNode();
      int parentNodeType = parentNode != null ? parentNode.getNodeType() : -1;
      if (nodeType == Node.DOCUMENT_NODE)
        comboBoxType.addItem(DOCUMENT);
      else if (nodeType == Node.DOCUMENT_TYPE_NODE)
        comboBoxType.addItem(DOCUMENT_TYPE);
      else if (parentNodeType == Node.DOCUMENT_NODE) {
        if (nodeType == Node.ELEMENT_NODE)
          comboBoxType.addItem(ELEMENT);
        else {
          if (nodeType == Node.COMMENT_NODE
            || isNodeTypeAllowed(Node.COMMENT_NODE))
            comboBoxType.addItem(COMMENT);
          if (nodeType == Node.PROCESSING_INSTRUCTION_NODE
            || isNodeTypeAllowed(Node.PROCESSING_INSTRUCTION_NODE))
            comboBoxType.addItem(PROCESSING_INSTRUCTION);
        }
      }
      else if (parentNodeType == Node.ELEMENT_NODE) {
        DocumentUtil documentUtil = fullyFledgedDocument.getDocumentUtil();
        Collection elementNamesForReplace = documentUtil != null ?
          documentUtil.elementNamesForReplace(
          fullyFledgedDocument.isLazy(), node) : null;
        if (nodeType == Node.ELEMENT_NODE
          || (elementNamesForReplace != null
          && !elementNamesForReplace.isEmpty()))
          comboBoxType.addItem(ELEMENT);
        Node[] nodes = new Node[1];
        nodes[0] = node;
        if (nodeType != Node.ELEMENT_NODE
          || fullyFledgedDocument.isLazy() || documentUtil == null
          || documentUtil.areNodesRemovable(nodes)) {
          if (nodeType == Node.TEXT_NODE || documentUtil == null
            || documentUtil.canElementHaveText((Element)parentNode))
            comboBoxType.addItem(TEXT);
          if (nodeType == Node.COMMENT_NODE
            || isNodeTypeAllowed(Node.COMMENT_NODE))
            comboBoxType.addItem(COMMENT);
          if (nodeType == Node.PROCESSING_INSTRUCTION_NODE
            || isNodeTypeAllowed(Node.PROCESSING_INSTRUCTION_NODE))
            comboBoxType.addItem(PROCESSING_INSTRUCTION);
          if (nodeType == Node.CDATA_SECTION_NODE
            || isNodeTypeAllowed(Node.CDATA_SECTION_NODE))
            comboBoxType.addItem(CDATA_SECTION);
          if ((nodeType == Node.ENTITY_REFERENCE_NODE
            || isNodeTypeAllowed(Node.ENTITY_REFERENCE_NODE))
            && doesAnEntityExist())
            comboBoxType.addItem(ENTITY_REFERENCE);
        }
      }
      displayedTypeName = getTypeName(nodeType);
      comboBoxType.setSelectedItem(displayedTypeName);
      boolean isEnabled = false;
      if (parentNodeType == Node.DOCUMENT_NODE)
        isEnabled = nodeType == Node.COMMENT_NODE
          || nodeType == Node.PROCESSING_INSTRUCTION_NODE;
      else if (parentNodeType == Node.ELEMENT_NODE)
        isEnabled = nodeType != Node.ELEMENT_NODE || !node.hasChildNodes();
      comboBoxType.setEnabled(isEnabled && isNodeTypeAllowed(nodeType)
        && !fullyFledgedDocument.isReadOnly());
    }
    finally {
      isDisplaying = false;
    }
  }

  public void refresh() {
    if (!isDisplaying && displayedTypeName != null
      && !displayedTypeName.equals(comboBoxType.getSelectedItem())) {
      isDisplaying = true;
      comboBoxType.setSelectedItem(displayedTypeName);
      isDisplaying = false;
    }
  }

  private boolean doesAnEntityExist() {
    List entityNameList =
      fullyFledgedDocument.getDTD().getEntityNameList();
    return entityNameList != null && entityNameList.size() != 0;
  }

  private boolean isNodeTypeAllowed(int nodeType) {
    String optionId = null;
    switch (nodeType) {
      case Node.COMMENT_NODE:
        optionId = Options.ALLOWSCOMMENT;
        break;
      case Node.PROCESSING_INSTRUCTION_NODE:
        optionId = Options.ALLOWSPROCESSINGINSTRUCTION;
        break;
      case Node.CDATA_SECTION_NODE:
        optionId = Options.ALLOWSCDATASECTION;
        break;
      case Node.ENTITY_REFERENCE_NODE:
        optionId = Options.ALLOWSENTITYREFERENCE;
        break;
    }
    return optionId == null
      || options.getThreeStateOptionValue(optionId) == ThreeStateOption.TRUE;
  }

  private static final String getTypeName(int nodeType) {
    switch (nodeType) {
      case Node.DOCUMENT_NODE:
        return DOCUMENT;
      case Node.DOCUMENT_TYPE_NODE:
        return DOCUMENT_TYPE;
      case Node.ELEMENT_NODE:
        return ELEMENT;
      case Node.TEXT_NODE:
        return TEXT;
      case Node.CDATA_SECTION_NODE:
        return CDATA_SECTION;
      case Node.COMMENT_NODE:
        return COMMENT;
      case Node.ENTITY_REFERENCE_NODE:
        return ENTITY_REFERENCE;
      case Node.PROCESSING_INSTRUCTION_NODE:
        return PROCESSING_INSTRUCTION;
      default:
        throw new IllegalArgumentException();
    }
  }

  private static final int getNodeType(String typeName) {
    if (typeName.equals(DOCUMENT))
      return Node.DOCUMENT_NODE;
    if (typeName.equals(DOCUMENT_TYPE))
      return Node.DOCUMENT_TYPE_NODE;
    if (typeName.equals(ELEMENT))
      return Node.ELEMENT_NODE;
    if (typeName.equals(TEXT))
      return Node.TEXT_NODE;
    if (typeName.equals(CDATA_SECTION))
      return Node.CDATA_SECTION_NODE;
    if (typeName.equals(COMMENT))
      return Node.COMMENT_NODE;
    if (typeName.equals(ENTITY_REFERENCE))
      return Node.ENTITY_REFERENCE_NODE;
    if (typeName.equals(PROCESSING_INSTRUCTION))
      return Node.PROCESSING_INSTRUCTION_NODE;
    throw new IllegalArgumentException();
  }
}
